﻿/* Copyright 2008, 2009, StrongMail Systems, Inc.  All rights reserved. */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace ConsoleApplication1
{
    class AttachmentSample
    {
        private MailingService service;
        private SampleHelper helper;

        public AttachmentSample(MailingService service)
        {
            this.service = service;
            this.helper = new SampleHelper(service);
        }


        /* Create an attachment using a file.
         * 
         * attachmentName is the contentBlockName of the attachment to be created.
         * attachmentFile is the actual file for the attachment.
         * return attachment ID of the created attachment.
         */
        public AttachmentId createAttachment(string attachmentName, FileStream dataFile)
        {
            Attachment attachment = new Attachment();
            attachment.name = attachmentName;
            attachment.description = "Attachment from dot net";

            // Use filename and content: 
            // The order has to be the same as defined in ItemsChoiceType: content, fileName, and then fileReference
            ItemsChoiceType1[] types = new ItemsChoiceType1[2];
            types[0] = ItemsChoiceType1.content;
            types[1] = ItemsChoiceType1.fileName;
            attachment.ItemsElementName = types;

            // Set filename and content data
            BinaryReader br = new BinaryReader(dataFile);

            object[] data = new object[2];
            data[0] = br.ReadBytes((int)dataFile.Length);
            data[1] = dataFile.Name;
            attachment.Items = data;

            return (AttachmentId) helper.create(attachment);
        }


        /* Get ant print a attachment: print out the size instead of content.
         * 
         * attachmentId is the ID of the attachment to be printed out.
         */
        public void getAndPrint(AttachmentId attachmentId)
        {
            Attachment attachment = (Attachment) helper.get(attachmentId);
            Console.WriteLine("\nAttachment:");
            Console.WriteLine("\tID: " + attachment.objectId.id);
            Console.WriteLine("\tName: " + attachment.name);
            Console.WriteLine("\tCampaign: " + attachment.campaignId);
            Console.WriteLine("\tCreatedTime: " + attachment.createdTime);
            Console.WriteLine("\tDescription: " + attachment.description);
            Console.WriteLine("\tModifiedTime: " + attachment.modifiedTime);
            Console.WriteLine("\tOrganization: " + attachment.organizationId.id);
            Console.WriteLine("\tOwenr: " + attachment.ownerId.id);
            Console.WriteLine("\tSize: " + attachment.size);
        }


        /* Import attachment.
         * 
         * attachmentId is the attachment ID that will be imported to.
         * attachmentFile is the attachment file that will be imported.
         * return if the importing operation is successfule.
         */
        public Boolean importContent(AttachmentId attachmentId, FileStream attachmentFile)
        {
            // Create request
            ImportAttachmentContentRequest request = new ImportAttachmentContentRequest();
            request.attachmentId = attachmentId;
            request.fileName = attachmentFile.Name;

            // Set content
            BinaryReader br = new BinaryReader(attachmentFile);
            request.content = br.ReadBytes((int) attachmentFile.Length);

            return service.importContent(new importContent(request)).importContentResponse.success;
        }

    }
}
