﻿/* Copyright 2008, 2009, StrongMail Systems, Inc.  All rights reserved. */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace ConsoleApplication1
{
    class TemplateSample
    {
        private MailingService service;
        private SampleHelper helper;
        private SystemAddressId bounceId;
        private SystemAddressId fromId;
        private SystemAddressId replyId;

        public TemplateSample(MailingService service, SystemAddressId bounceId, SystemAddressId fromId, SystemAddressId replyId)
        {
            this.service = service;
            this.bounceId = bounceId;
            this.fromId = fromId;
            this.replyId = replyId;
            this.helper = new SampleHelper(service);
        }


        /* Create a template.
         * 
         * templateName is the name of template.
         * attachmentId is the ID of attachment used by this template.
         * contentBlockId is the ID of content block used by this tempalte.
         * 
         * return the ID of created template.
         */
        public TemplateId createTemplate(string templateName, AttachmentId attachmentId, ContentBlockId contentBlockId)
        {
            Template template = new Template();

            template.bodyEncoding = Encoding.BASE64;
            template.description = "Template from dotnet";
            template.fromName = "From Name";
            template.headerEncoding = Encoding.EIGHT_BIT;
            template.isApproved = true;
            template.name = templateName;
            template.bounceAddressId = bounceId;
            template.fromAddressId = fromId;
            template.replyAddressId = replyId;

            template.subject = "template subject from dotnet";
            template.header = new String[] { "Template header" };
            template.attachmentId = new AttachmentId[] { attachmentId };
            template.contentBlockId = new ContentBlockId[] { contentBlockId };

            // Use outputBodyCharSet(Token)
            template.Item = CharSet.ASCII;

            // Use outputHeaderCharSet(Token)
            template.Item1 = CharSet.UTF8;

            // Set message part
            MessagePart messagePart = new MessagePart();
            messagePart.content = "<html><body>HTML: hello, HTML message</body></html>";
            messagePart.format = MessageFormat.HTML;
            messagePart.isXsl = false;
            template.messagePart = new MessagePart[] { messagePart };

            return (TemplateId)helper.create(template);
        }


        public void getAndPrint(TemplateId templateId)
        {
            Template template = (Template)helper.get(templateId);
            Console.WriteLine("\nTemplate:");
            Console.WriteLine("\tID: " + template.objectId.id);
            Console.WriteLine("\tName: " + template.name);
            Console.WriteLine("\tCampaign: " + template.campaignId);
            Console.WriteLine("\tCreatedTime: " + template.createdTime);
            Console.WriteLine("\tDescription: " + template.description);
            Console.WriteLine("\tModifiedTime: " + template.modifiedTime);
            Console.WriteLine("\tOrganization: " + template.organizationId.id);
            Console.WriteLine("\tOwenr: " + template.ownerId.id);
            Console.WriteLine("\tAttchment Size: " + template.attachmentId.Length);
            Console.WriteLine("\tContentBlock Size: " + template.contentBlockId.Length);
            Console.WriteLine("\tHeader Size: " + template.header.Length);
            Console.WriteLine("\tBodyEncoding: " + template.bodyEncoding);
            Console.WriteLine("\tFromName: " + template.fromName);
            Console.WriteLine("\tSubject: " + template.subject);

            Console.WriteLine("\tMessagePart: ");
            for (int i = 0; i < template.messagePart.Length; i++)
            {
                Console.WriteLine("\tMessagePart size: " + template.messagePart[i].content.Length);
            }
        }


        /* Import message part using a zip file and media server.
         * 
         * templateId is the ID of template where message part will be imported.
         * mediaServerId is the media server ID used to store the image files.
         * zipFile is the zip file that contains HTML and image files.
         * folderName is the directory to unzip the zip file.
         * 
         * return if the importing operaion is successful.
         * 
         */
        public bool importMessagePart(TemplateId templateId, MediaServerId mediaServerId, FileStream zipFile, String folderName)
        {
            // Create request
            ImportMessagePartRequest request = new ImportMessagePartRequest();
            request.folderName = folderName;
            request.isXsl = false;
            request.mediaServerId = mediaServerId;
            request.templateId = templateId;

            // Set content from zip file
            BinaryReader reader = new BinaryReader(zipFile);
            byte[] data = reader.ReadBytes((int) zipFile.Length);
            request.zipFile = data;

            return service.importMessagePart(new importMessagePart(request)).importMessagePartResponse.success;
        }


        /* Test template for its dynamic content. 
         * Email will be sent and tokens in the template will be replaced wit values.
         * 
         * templateId is the templateId that will be tested.
         * return if the test request is sent successfully.
         */
        public bool testTemplate(TemplateId templateId)
        {
            // Create request
            TestTemplateRequest request = new TestTemplateRequest();
            request.subjectPrefix = "Subject Prefix: ";
            request.templateId = templateId;
            request.useMultiPart = false;
            request.address = new String[] {"test_address@testtemplate.com"};
            request.format = new MessageFormat[] {MessageFormat.HTML};

            // Make sure you have these two tokens in your template
            NameValuePair pair1 = new NameValuePair();
            pair1.name = "EMAIL";
            pair1.value = "testmail@testtemplate.com";

            NameValuePair pair2 = new NameValuePair();
            pair2.name = "NAME";
            pair2.value = "testmail name";

            // Set data to request and make call
            request.tokenValue = new NameValuePair[] {pair1, pair2};
            return service.test(new test(request)).testResponse.success;
        }

        
        /* Validate if a template's xsl content is valid.
         */
        public bool validateXsl(TemplateId templateId)
        {
            ValidateXslRequest request = new ValidateXslRequest();
            request.templateId = templateId;
            request.messageFormat = MessageFormat.HTML;

            return service.validateXsl(new validateXsl(request)).validateXslResponse.valid;
        }
    }


}
