﻿/* Copyright 2008, 2009, StrongMail Systems, Inc.  All rights reserved. */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ConsoleApplication1
{
    class SeedListSample
    {
        private MailingService service;
        private SampleHelper helper;

        public SeedListSample(MailingService service)
        {
            this.service = service;
            this.helper = new SampleHelper(service);
        }


        /* Create a seed list and add some data.
         * 
         * seedListName is the name of the seed list.
         * return the ID of create seed list.
         */
        public SeedListId createSeedList(string seedListName)
        {
            SeedList seed = new SeedList();
            seed.description = "SeedList from dotnet";
            seed.isTestList = true;
            seed.name = seedListName;

            SeedListId seedListId = (SeedListId) helper.create(seed);

            // Add some records
            addRecords(seedListId);

            return seedListId;
        }


        /* Add 2 records to seed list using inline records.
         * 
         * seedListId is the ID of the seed list where records will add.
         */
        public void addRecords(SeedListId seedListId)
        {
            // Create request
            AddSeedListRecordsRequest request = new AddSeedListRecordsRequest();
            request.seedListId = seedListId;

            // Create data
            string[] seeds = new string[2];
            seeds[0] = "seed1@dotnet.com";
            seeds[1] = "seed2@dotnet.com";

            request.Items = seeds;
            service.addRecords(request);
        }


        /* Remove one record that was created in addRecordsInine() from the seed list.
         * 
         * seedListId is the seed list ID whose record will be removed.
         * return the number of records that was removed.
         */
        public int removeRecordsInline(SeedListId seedListId)
        {
            // Create request
            RemoveSeedListRecordsRequest request = new RemoveSeedListRecordsRequest();
            request.seedListId = seedListId;

            // Set record to be removed
            String[] addresses = new String[1];
            addresses[0] = "seed_from_dotnet_1@dotnet.com";
            request.Items = addresses;

            return service.removeRecords(request).recordsRemoved;
        }


        public void getAndPrint(SeedListId seedId)
        {
            SeedList seed = (SeedList) helper.get(seedId);
            Console.WriteLine("\nSeedList:");
            Console.WriteLine("\tID: " + seed.objectId.id);
            Console.WriteLine("\tName: " + seed.name);
            Console.WriteLine("\tCampaign: " + seed.campaignId);
            Console.WriteLine("\tCreatedTime: " + seed.createdTime);
            Console.WriteLine("\tDescription: " + seed.description);
            Console.WriteLine("\tModifiedTime: " + seed.modifiedTime);
            Console.WriteLine("\tOrganization: " + seed.organizationId.id);
            Console.WriteLine("\tOwenr: " + seed.ownerId.id);
            Console.WriteLine("\tIsTestList: " + seed.isTestList);
            Console.WriteLine("\tTotal Records: " + seed.totalRecords);
        }

    }
}
