<?php
/* Copyright 2009, StrongMail Systems, Inc.  All rights reserved. */

require_once "SampleHelper.php";
require_once "./generated/MailingService.php";

class AttachmentSample
{
  private $service;
  private $helper;

  public function __construct($service)
  {
    $this->service = $service;
    $this->helper = new SampleHelper($service);
  }

  /**
   * Create an attachment using a file.
   *
   * @param attachmentName the name of the attachment
   * @param attachmentFile the actual file for the attachment
   * @return attachmentId
   * @throws Exception any exception
   */
  public function createAttachment($attachmentName, $attachmentFile)
  {
    $attachment = new Attachment();

    $attachment->description = "Attachment from PHP";
    $attachment->fileName = basename($attachmentFile);
    $attachment->name = $attachmentName;
    $attachment->content = file_get_contents($attachmentFile);

    $request = SampleHelper::getSoapCreateRequestObject($attachment, 'Attachment');
    $response = $this->service->create($request);

    return $response->createResponse[0]->objectId;
  }

  /**
   * Get and print attachment object to standard out.
   *
   * @param attachmentId the attachment ID that will be retrieved and printed.
   * @throws Exception any exception
   */
  public function getAndPrint($attachmentId)
  {
    $attachments = $this->helper->get(array(SampleHelper::getSoapEntityIdObject(new AttachmentId(),
										$attachmentId->id,
										"AttachmentId")));
    $attachment = $attachments[0];

    // Print Attachment to standard out
    print("\nAttachment:" . "\n");
    print("AttachmentId: " . $attachment->objectId->id . "\n");
    print("Name: " . $attachment->name . "\n");
    print("CampaignId: " . $attachment->campaignId . "\n");
    print("CreatedTime: " . $attachment->createdTime . "\n");
    print("Description: " . $attachment->description . "\n");
    print("ModifiedTime: " . $attachment->modifiedTime . "\n");
    print("OrganizationId: " . $attachment->organizationId->id . "\n");
    print("OwnerId: " . $attachment->ownerId->id . "\n");
    print("Size: " . $attachment->size . "\n");
  }

  /**
   * Import attachment.
   *
   * @param attachmentId   the AttachmentId that will be imported to.
   * @param attachmentFile the attachment file that will be imported.
   * @return if the importing operation is successful.
   * @throws Exception any exception
   */
  public function importContent($attachmentId, $attachmentFile)
  {
    $request = new ImportAttachmentContentRequest();
    $request->attachmentId = SampleHelper::getSoapEntityIdObject(new AttachmentId(),
								 $attachmentId->id,
								 "AttachmentId");
    $request->fileName = basename($attachmentFile);

    // Set content
    $request->content = file_get_contents($attachmentFile);

    $response = $this->service->importContent($request);

    return $response->importContentResponse->success;
  }
}
?>
