<?php
/* Copyright 2009, StrongMail Systems, Inc.  All rights reserved. */

require_once "SampleHelper.php";
require_once "./generated/MailingService.php";

class ContentBlockSample
{
  private $service;
  private $helper;

  public function __construct($service)
  {
    $this->service = $service;
    $this->helper = new SampleHelper($service);
  }

  /**
   * Create a content block.
   *
   * @param contentBlockName the name of the content block
   * @return contentBlockId
   * @throws Exception any exception
   */
  public function createContentBlock($contentBlockName)
  {
    $contentBlock = new ContentBlock();

    $contentBlock->description = "ContentBlock from PHP";
    $contentBlock->content = "hiking {\n" . "  I love hiking.\n" . "}\n" .
                            "scuba {\n" . "  I love SCUBA diving.\n" . "}";
    $contentBlock->name = $contentBlockName;

    $request = SampleHelper::getSoapCreateRequestObject($contentBlock, 'ContentBlock');
    $response = $this->service->create($request);

    return $response->createResponse[0]->objectId;
  }

  /**
   * Get and print contentBlock object to standard out.
   *
   * @param contentBlockId the contentBlock ID that will be retrieved and printed.
   * @throws Exception any exception
   */
  public function getAndPrint($contentBlockId)
  {
    $contentBlocks = $this->helper->get(array(SampleHelper::getSoapEntityIdObject(new ContentBlockId(),
										$contentBlockId->id,
										"ContentBlockId")));
    $contentBlock = $contentBlocks[0];

    // Print ContentBlock to standard out
    print("\nContentBlock:" . "\n");
    print("ContentBlockId: " . $contentBlock->objectId->id . "\n");
    print("Name: " . $contentBlock->name . "\n");
    print("CampaignId: " . $contentBlock->campaignId . "\n");
    print("CreatedTime: " . $contentBlock->createdTime . "\n");
    print("Description: " . $contentBlock->description . "\n");
    print("ModifiedTime: " . $contentBlock->modifiedTime . "\n");
    print("OrganizationId: " . $contentBlock->organizationId->id . "\n");
    print("OwnerId: " . $contentBlock->ownerId->id . "\n");
    print("Content: " . $contentBlock->content . "\n");
    print("Size: " . $contentBlock->size . "\n");
  }
}
?>
