<?php
/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2011 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

require_once "SampleHelper.php";
require_once "./generated/MailingService.php";

class RoleSample
{
  private $service;
  private $helper;

  public function __construct($service)
  {
    $this->service = $service;
    $this->helper = new SampleHelper($service);
  }

  public function createRole($roleName)
  {
    $role = new Role();

    $role->name = $roleName;

    // Set all permissions for IDS module.
    $rolePermissions = new RolePermissions();
    $rolePermissions->create = true;
    $rolePermissions->update = true;
    $rolePermissions->delete = false;
    $rolePermissions->view = true;
    $permissions = new Permissions();
    $permissions->internalDataSources = $rolePermissions;
    $role->permissions = $permissions;

    $orgId = new OrganizationId();
    $orgId->id = "1";
    $role->organizationId = $orgId;

    $request = SampleHelper::getSoapCreateRequestObject($role, "Role");
    $response = $this->service->create($request);

    return $response->createResponse[0]->objectId;
  }

  /**
   * Get and print role object to standard out.
   *
   * @param roleId the role ID that will be retrieved and printed.
   * @throws Exception any exception
   */
  public function getAndPrint($roleId)
  {
    $roles = $this->helper->get(array(SampleHelper::getSoapEntityIdObject(new RoleId(),
										$roleId->id,
										"RoleId")));
    $role = $roles[0];

    // Print role to standard out
    print("\nRole:" . "\n");
    print("RoleId: " . $role->objectId->id . "\n");
    print("Name: " . $role->name . "\n");
    print("IDS permissions: \n");
    print("   Create:" . $role->permissions->internalDataSources->create . "\n");
    print("   Update:" . $role->permissions->internalDataSources->update . "\n");
    print("   Delete:" . $role->permissions->internalDataSources->delete . "\n");
    print("   View:" . $role->permissions->internalDataSources->view . "\n");
    print("Organization ID: " . $role->organizationId->id . "\n");
  }
}
?>
