<?php
/* Copyright 2009, StrongMail Systems, Inc.  All rights reserved. */

require_once "SampleHelper.php";
require_once "./generated/MailingService.php";

class RuleSample
{
  private $service;
  private $helper;

  public function __construct($service)
  {
    $this->service = $service;
    $this->helper = new SampleHelper($service);
  }

  /**
   * Create a rule using the Target created in TargetSample.
   *
   * @param ruleName the name of the rule to be created
   * @param targetId the targetId to be used with this rule.
   * @return ruleId
   * @throws Exception any exception
   */
  public function createRule($ruleName, $targetId)
  {
    $rule = new Rule();

    $rule->description = "Rule from PHP";
    $rule->name = $ruleName;
    $rule->targetId = array();
    $rule->targetId[] = SampleHelper::getSoapEntityIdObject(new TargetId(),
                                                            $targetId->id,
                                                            "TargetId");

    // Create two conditions using the target created in TargetSample
    $targets = $this->helper->get(array(SampleHelper::getSoapEntityIdObject(new TargetId(),
                                                                                $targetId->id,
                                                                                "TargetId")));
    $target = $targets[0];

    $condition1 = new RuleIfPartCondition();
    $condition1->column = $target->name . ".email_address";
    $condition1->op = "EQUALS";
    $condition1->value = "email_for_php@php.com";

    $condition2 = new RuleIfPartCondition();
    $condition2->column = $target->name . ".name";
    $condition2->op = "CONTAINS";
    $condition2->value = "php";

    // Set If part, format for conditions are: Condition1 AND/OR Condition AND/OR Condition ...
    // The above two conditions will return true
    $ifpart = new RuleIfPart();
    $ifpart->condition1 = $condition1; // Set the first condition
    $ifpart->logicalOperation = "AND"; // Set logical operator
    $ifpart->condition = $condition2;   // Set another condition
    $rule->ifPart = $ifpart;

    // Set Then part
    $thenvalue = new TextRuleValue();
    $thenvalue->value = "This is then part";
    $thenpart = new RuleThenPart();
    $thenpart->textRuleValue = $thenvalue;
    $rule->thenPart = $thenpart;

    // Set Else part
    $elsevalue = new ColumnRuleValue();
    $elsevalue->value = $target->name . ".name";
    $elsepart = new RuleElsePart();
    $elsepart->columnRuleValue = $elsevalue;
    $rule->elsePart = $elsepart;

    // Make call
    $request = SampleHelper::getSoapCreateRequestObject($rule, "Rule");
    $response = $this->service->create($request);

    return $response->createResponse[0]->objectId;
  }

  public function getAndPrint($ruleId)
  {
    $rules = $this->helper->get(array(SampleHelper::getSoapEntityIdObject(new RuleId(),
                                                                                $ruleId->id,
                                                                                "RuleId")));
    $rule = $rules[0];

    // Print Rule to standard out
    print("\nRule:" . "\n");
    print("RuleId: " . $rule->objectId->id . "\n");
    print("Name: " . $rule->name . "\n");
    print("TargetId: " . SampleHelper::formatIdList($rule->targetId) . "\n");
    print("CampaignId: " . $rule->campaignId . "\n");
    print("CreatedTime: " . $rule->createdTime . "\n");
    print("Description: " . $rule->description . "\n");
    print("ModifiedTime: " . $rule->modifiedTime . "\n");
    print("OrganizationId: " . $rule->organizationId->id . "\n");
    print("OwnerId: " . $rule->ownerId->id . "\n");

    // Print condition 1
    $condition1 = $rule->ifPart->condition1;
    print("Rule-If:" . "\n");
    print("\t" . $condition1->column . " " . $condition1->op . " " . $condition1->value . "\n");

    // Print condition 2
    $logicalOp = $rule->ifPart->logicalOperationAndCondition[0];
    $condition2 = $rule->ifPart->logicalOperationAndCondition[1];
    print("\t" . $logicalOp . " " . $condition2->column . " " . $condition2->op . " " . $condition2->value . "\n");

    // Print Then part: use text value since it was set above
    print("Rule-Then: " . $rule->thenPart->textRuleValue->value . "\n");

    // Print Else part: use column value since it was set above
    print("Rule-Else: " . $rule->elsePart->columnRuleValue->value . "\n");
  }
}
