<?php
/* Copyright 2009, StrongMail Systems, Inc.  All rights reserved. */

require_once "SampleHelper.php";
require_once "./generated/MailingService.php";

class SeedListSample
{
  private $service;
  private $helper;

  public function __construct($service)
  {
    $this->service = $service;
    $this->helper = new SampleHelper($service);
  }

  /**
   * Create a seed list and add some data.
   *
   * @param seedListName the name of the seed list
   * @return seedListId
   * @throws Exception any exception
   */
  public function createSeedList($seedListName)
  {
    $seedList = new SeedList();

    $seedList->description = "SeedList from PHP";
    $seedList->isTestList = true;
    $seedList->name = $seedListName;

    // Make call
    $request = SampleHelper::getSoapCreateRequestObject($seedList, "SeedList");
    $response = $this->service->create($request);
    $seedListId = $response->createResponse[0]->objectId;

    // Add some records
    $this->addRecordsInline($seedListId);

    return $seedListId;
  }

  public function addRecordsInline($seedListId)
  {
    // Create request
    $request = new AddSeedListRecordsRequest();
    $request->seedListId = SampleHelper::getSoapEntityIdObject(new SeedListId(),
                                                               $seedListId->id,
                                                               "SeedListId");
    $request->address = array();
    $request->address[] = "seed1@seed.php.com";
    $request->address[] = "seed2@seed.php.com";

    $this->service->addRecords($request);
  }

  /**
   * Remove one record that was created in addRecordsInline() from the SeedList.
   *
   * @param seedListId the SeedListId whose record will be removed.
   * @return the number of record that was removed.
   * @throws Exception any exception
   */
  public function removeRecordsInline($seedListId)
  {
    $request = new RemoveSeedListRecordsRequest();
    $request->seedListId = SampleHelper::getSoapEntityIdObject(new SeedListId(),
                                                               $seedListId->id,
                                                               "SeedListId");
    $request->address = array();
    $request->address[] = "seed1@seed.php.com";

    $response = $this->service->removeRecords($request);
    return $response->recordsRemoved;
  }

  /**
   * Get and print seedList object to standard out.
   *
   * @param seedListId the seedList ID that will be retrieved and printed.
   * @throws Exception any exception
   */
  public function getAndPrint($seedListId)
  {
    $seedLists = $this->helper->get(array(SampleHelper::getSoapEntityIdObject(new SeedListId(),
                                                                                $seedListId->id,
                                                                                "SeedListId")));
    $seedList = $seedLists[0];

    // Print SeedList to standard out
    print("\nSeedList:" . "\n");
    print("SeedListId: " . $seedList->objectId->id . "\n");
    print("Name: " . $seedList->name . "\n");
    print("CampaignId: " . $seedList->campaignId . "\n");
    print("CreatedTime: " . $seedList->createdTime . "\n");
    print("Description: " . $seedList->description . "\n");
    print("ModifiedTime: " . $seedList->modifiedTime . "\n");
    print("OrganizationId: " . $seedList->organizationId->id . "\n");
    print("OwnerId: " . $seedList->ownerId->id . "\n");
    print("Total Records: " . $seedList->totalRecords . "\n");
  }
}

?>
