<?php
/* Copyright 2009, StrongMail Systems, Inc.  All rights reserved. */

require_once "SampleHelper.php";
require_once "./generated/MailingService.php";

class TargetSample
{
  private $service;
  private $helper;

  public function __construct($service)
  {
    $this->service = $service;
    $this->helper = new SampleHelper($service);
  }

  public function createTarget($targetName, $dataSourceId)
  {
    $target = new Target();

    // Fill in the target
    $target->name = $targetName;
    $target->dataSourceId = SampleHelper::getSoapEntityIdObject(new InternalDataSourceId(),
								$dataSourceId->id,
								"InternalDataSourceId");
    $target->type = "ADVANCED";
    $target->description = "Target from PHP";
    $target->emailAddressFieldName = "email_address";   // From the data source
    $target->bounceFieldName = "email_address_status";  // system generated based on email column
    $target->excludeBounce = true;
    $target->unsubscribeFieldName = "unsub";  // system generated
    $target->excludeUnsubscribe = true;

    // Wait for data source status to be idle before adding
    $this->helper->waitForIdle($dataSourceId);

    $request = SampleHelper::getSoapCreateRequestObject($target, "Target");
    $response = $this->service->create($request);

    return $response->createResponse[0]->objectId;
  }

  /**
   * Get and print target object to standard out.
   *
   * @param targetId the target ID that will be retrieved and printed.
   * @throws Exception any exception
   */
  public function getAndPrint($targetId)
  {
    $targets = $this->helper->get(array(SampleHelper::getSoapEntityIdObject(new TargetId(),
										$targetId->id,
										"TargetId")));
    $target = $targets[0];

    // Print target to standard out
    print("\nTarget:" . "\n");
    print("TargetId: " . $target->objectId->id . "\n");
    print("Name: " . $target->name . "\n");
    print("Type: " . $target->type . "\n");
    print("DataSource: " . $target->dataSourceId->id . "\n");
    print("EmailField: " . $target->emailAddressFieldName . "\n");
    print("SMSField: " . $target->smsAddressFieldName . "\n");
    print("BounceField: " . $target->bounceFieldName . "\n");
    print("UnsubField: " . $target->unsubscribeFieldName . "\n");
    print("Description: " . $target->description . "\n");
    print("Query: " . $target->advancedQuery . "\n");
    print("CampaignId: " . $target->campaignId . "\n");
    print("CreatedTime: " . $target->createdTime . "\n");
    print("ModifiedTime: " . $target->modifiedTime . "\n");
    print("OrganizationId: " . $target->organizationId->id . "\n");
    print("OwnerId: " . $target->ownerId->id . "\n");
    print("Total Records: " . $target->totalRecords . "\n");
  }
}
?>
