<?php
/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2011 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */


require_once "SampleHelper.php";
require_once "SystemAddressSample.php";
require_once "./generated/MailingService.php";

class TemplateSample
{
  private $service;
  private $helper;

  public function __construct($service)
  {
    $this->service = $service;
    $this->helper = new SampleHelper($service);
  }

  /**
   * Create a template.
   *
   * @param templateName   the name of the template
   * @param attachmentId   the attachment used for this template
   * @param contentBlockId the content block used for this template
   * @return TemplateId
   * @throws Exception any exception
   */
  public function createTemplate($templateName, $attachmentId, $contentBlockId)
  {
    $template = new Template();

    $template->bodyEncoding = "BASE64";
    $template->description = "Template from PHP";
    $template->fromName = "From Name";
    $template->headerEncoding = "EIGHT_BIT";
    $template->isApproved = true;
    $template->name = $templateName;
    $template->outputBodyCharSet = "ASCII";
    $template->outputHeaderCharSet = "UTF-8";
    $template->subject = "PHP subject";
    $template->attachmentId = array();
    $template->attachmentId[] = SampleHelper::getSoapEntityIdObject(new AttachmentId(),
								$attachmentId->id,
								"AttachmentId");
    $template->contentBlockId = array();
    $template->contentBlockId[] = SampleHelper::getSoapEntityIdObject(new ContentBlockId(),
								      $contentBlockId->id,
								      "ContentBlockId");
    $template->header = array("PHP template header");

    // Create a message part
    $messagePart = new MessagePart();
    $messagePart->content = "<html><body>hello, HTML message</body></html>";
    $messagePart->format = "HTML";
    $messagePart->isXsl = false;

    $template->messagePart = array($messagePart);

    // Make call
    $request = SampleHelper::getSoapCreateRequestObject($template, "Template");
    $response = $this->service->create($request);

    return $response->createResponse[0]->objectId;
  }

  /**
   * Get and print template object to standard out.
   *
   * @param templateId the template ID that will be retrieved and printed.
   * @throws Exception any exception
   */
  public function getAndPrint($templateId)
  {
    $templates = $this->helper->get(array(SampleHelper::getSoapEntityIdObject(new TemplateId(),
										$templateId->id,
										"TemplateId")));
    $template = $templates[0];

    // Print Template to standard out
    print("\nTemplate:" . "\n");
    print("TemplateId: " . $template->objectId->id . "\n");
    print("Name: " . $template->name . "\n");
    print("CampaignId: " . $template->campaignId . "\n");
    print("CreatedTime: " . $template->createdTime . "\n");
    print("Description: " . $template->description . "\n");
    print("ModifiedTime: " . $template->modifiedTime . "\n");
    print("OrganizationId: " . $template->organizationId->id . "\n");
    print("OwnerId: " . $template->ownerId->id . "\n");
    print("AttachmentId: " . SampleHelper::formatIdList($template->attachmentId) . "\n");
    print("ContentBlockId: " . SampleHelper::formatIdList($template->contentBlockId) . "\n");
    print("Headers: " . SampleHelper::formatStringList($template->header) . "\n");
    print("BodyEncoding: " . $template->bodyEncoding . "\n");
    print("HeaderEncoding: " . $template->headerEncoding . "\n");
    print("MessagePart: " . SampleHelper::formatMessagePartList($template->messagePart) . "\n");
    print("BodyCharset: " . $template->outputBodyCharSet . "\n");
    print("BodyOutputToken: " . $template->outputBodyCharSetToken . "\n");
    print("HeaderCharset: " . $template->outputHeaderCharSet . "\n");
    print("HeaderOutputToken: " . $template->outputHeaderCharSetToken . "\n");
    print("BounceAddressId: " . $template->bounceAddressId . "\n");
    print("FromAddressId: " . $template->fromAddressId . "\n");
    print("FromName: " . $template->fromName . "\n");
    print("ReplyAddressId: " . $template->replyAddressId . "\n");
    print("Subject: " . $template->subject . "\n");
  }
}
?>
