# Copyright 2008, 2009, StrongMail Systems, Inc.  All rights reserved.

require 'SampleHelper'
require 'base64'

class AttachmentSample
  def initialize(svc)
    @svc = svc
  end


  # Create an attachment using a file. 
  #  attachmentName is the contentBlockName of the attachment to be created.
  #  attachmentFile is the actual file for the attachment.
  #  return attachment ID of the created attachment.
  #
  def createAttachment(attachmentName, dataFile)
    # Create InternalDataSource and fill in properties.
    attachment = Attachment.new()
    attachment.name = attachmentName
    attachment.description = "Attachment from Ruby"

    # Set filename and content data
    attachment.fileName = File.basename(dataFile)
    attachment.content = Base64.encode64(File.open(dataFile, "rb") {|io| io.read})

    return SampleHelper::create(@svc, attachment)
  end


  #Get ant print a attachment: print out the size instead of content.
  # attachmentId is the ID of the attachment to be printed out.
  def getAndPrint(attachmentId)
    attachment = SampleHelper::get(@svc, attachmentId)
    puts attachment.class
    puts "\tID: " + attachment.objectId.id
    puts "\tName: " + attachment.name
    puts "\tCreated Time: " + attachment.createdTime.to_s()
    puts "\tDescription: " + attachment.description
    puts "\tModified Time: " + attachment.modifiedTime.to_s()
    puts "\tOrganization: " + attachment.organizationId.id
    puts "\tUserId: " + attachment.ownerId.id
    puts "\tSize: " + attachment.size.to_s()
  end

  
  # Import attachment.
  #  attachmentId is the attachment ID that will be imported to.
  #  attachmentFile is the attachment file that will be imported.
  #  return if the importing operation is successfule.
  def importContent(attachmentId, dataFile)
    request = ImportAttachmentContentRequest.new
    request.attachmentId = attachmentId
    request.fileName = File.basename(dataFile)
    request.content = Base64.encode64(File.open(dataFile, "r") {|io| io.read})

    @svc.importContent(request)
  end

end