# Copyright 2008, 2009, StrongMail Systems, Inc.  All rights reserved.

require 'SampleHelper'

class RuleSample
  def initialize(svc)
    @svc = svc
  end


  # Crate a rule using the target create in TargetSample.
  #  ruleName is the name of the rule to be created.
  #  targetId is the target to be used with the rule.
  #  return the ID of the created rule.
  def createRule(ruleName, targetId)
    rule = Rule.new
    rule.name = ruleName
    rule.description = "Rule from Ruby"

    # Get the target used by the rule
    target = SampleHelper::get(@svc, targetId)
    rule.targetId = targetId

    # Specify IF part
    ifpart = RuleIfPart.new

    # Create three conditions useing the target created in TargetSample
    cond_1 = RuleIfPartCondition.new
    cond_1.column = target.name + ".email_address"
    cond_1.op = "EQUALS"
    cond_1.value = "email_for_ruby@ruby.com"
    ifpart.condition1 = cond_1

    # IF condition 2
    cond_2 = RuleIfPartCondition.new
    cond_2.column = target.name + ".name"
    cond_2.op = "CONTAINS"
    cond_2.value = "ruby"

    # Set IF part, format for conditions are: Condition1 AND/OR Conditoni AND/OR Condition2 ...
    ifpart.logicalOperation = ["AND"]
    ifpart.condition = [cond_2]
    rule.ifPart = ifpart

    # Specify THEN part
    thenpart = RuleThenPart.new
    textValue = TextRuleValue.new
    textValue.value = "This is then part"
    thenpart.textRuleValue = textValue
    rule.thenPart = thenpart

    # Specify ELSE part
    elsepart = RuleElsePart.new
    colValue = ColumnRuleValue.new
    colValue.value = target.name + ".name"
    elsepart.columnRuleValue = colValue
    rule.elsePart = elsepart

    return SampleHelper::create(@svc, rule)
  end


  # Get and print out a rule.
  #  ruleId is the ID of the rule to be printed.
  def getAndPrint(ruleId)
    rule = SampleHelper::get(@svc, ruleId)
    puts rule.class
    puts "\tID: " + rule.objectId.id
    puts "\tName: " + rule.name
    puts "\tCreated Time: " + rule.createdTime.to_s()
    puts "\tDescription: " + rule.description
    puts "\tModified Time: " + rule.modifiedTime.to_s()
    puts "\tOrganization: " + rule.organizationId.id
    puts "\tUserId: " + rule.ownerId.id

    # Print condition 1
    puts "\tRule-If: "
    cond_1 = rule.ifPart.condition1
    puts "\t\t" + cond_1.column.to_s + " " + cond_1.op.to_s + " " + cond_1.value.to_s

    # Print condition 2
    logicalOp1 = rule.ifPart.logicalOperation[0]
    cond_2 = rule.ifPart.condition[0]
    puts "\t\t" + logicalOp1.to_s + " " + cond_2.column.to_s + " " + cond_2.op.to_s + " " + cond_2.value.to_s

    # Print Then part
    puts "\tRule-Then: " + rule.thenPart.textRuleValue.value

    # Print Else part
    puts "\tRule-Else: " + rule.elsePart.columnRuleValue.value
  end
  
end
