# Copyright 2008, 2009, StrongMail Systems, Inc.  All rights reserved.

require 'SampleHelper'

class SeedListSample
  def initialize(svc)
    @svc = svc
  end


  # Create a seed list and add some data.
  #  seedListName is the name of the seed list.
  #  return the ID of create seed list.
  def createSeedList(seedListName)
    seed = SeedList.new
    seed.name = seedListName
    seed.description = "SeedList from Ruby"
    seed.isTestList = true

    seedListId = SampleHelper::create(@svc, seed)

    # Add some records
    addRecords(seedListId);

    return seedListId;
  end


  # Add 2 records to seed list using inline records.
  #  seedListId is the ID of the seed list where records will add.
  def addRecords(seedListId)
    # Create request
    request = AddSeedListRecordsRequest.new
    request.seedListId = seedListId

    # Create and set data
    request.address = ["seed1@ruby.com", "seed2@ruby.com"]

    # Make call
    @svc.addRecords(request)
  end


  # Remove one record that was created in addRecordsInine() from the seed list.
  #  seedListId is the seed list ID whose record will be removed.
  #  return the number of records that was removed.
  def removeRecordsInline(seedListId)
    request = RemoveSeedListRecordsRequest.new
    request.seedListId = seedListId
    request.address = ["seed_1@ruby.com"]

    return @svc.removeRecords(request)
  end

  
  def getAndPrint(seedListId)
    seed = SampleHelper::get(@svc, seedListId)
    puts seed.class
    puts "\tID: " + seed.objectId.id
    puts "\tName: " + seed.name
    puts "\tCreated Time: " + seed.createdTime.to_s
    puts "\tDescription: " + seed.description
    puts "\tModified Time: " + seed.modifiedTime.to_s
    puts "\tOrganization: " + seed.organizationId.id
    puts "\tUserId: " + seed.ownerId.id
    puts "\tIsTestList: " + seed.isTestList.to_s
    puts "\tTotal Records: " + seed.totalRecords.to_s
  end
end
