# Copyright 2008, 2009, StrongMail Systems, Inc.  All rights reserved.

require 'SampleHelper'

class TemplateSample
  def initialize(svc, bounceId, fromId, replyId)
    @svc = svc
    @bounceId = bounceId
    @fromId = fromId
    @replyId = replyId
  end


  # Create a template.
  #  templateName is the name of template.
  #  attachmentId is the ID of attachment used by this template.
  #  contentBlockId is the ID of content block used by this tempalte.
  #  return the ID of created template.
  def createTemplate(templateName, attachmentId, contentBlockId)
    template = Template.new
    template.name = templateName
    template.description = "Template from dotnet"
    template.bodyEncoding = "BASE64"
    template.fromName = "From Name"
    template.headerEncoding = "EIGHT_BIT"
    template.isApproved = true
    template.bounceAddressId = @bounceId
    template.fromAddressId = @fromId
    template.replyAddressId = @replyId
    template.subject = "template subject from Ruby"
    template.header = ["Template header"]
    template.attachmentId = attachmentId
    template.contentBlockId = contentBlockId

    template.forward2FriendOffer = ""
    template.forward2FriendOfferTrackingOption = "NONE"

    # Use outputBodyCharSet(Token)
    template.outputBodyCharSet = "ASCII"

    # Use outputHeaderCharSet(Token)
    template.outputHeaderCharSet = "UTF-8"

    # Set message part
    messagePart = MessagePart.new
    messagePart.content = "<html><body>HTML: hello, HTML message</body></html>"
    messagePart.format = "HTML"
    messagePart.isXsl = false
    template.messagePart = [messagePart]

    return SampleHelper::create(@svc, template)
  end

  
  # Get ant print a template.
  # templateId is the ID of the template to be printed out.
  def getAndPrint(templateId)
    template = SampleHelper::get(@svc, templateId)
    puts template.class
    puts "\tID: " + template.objectId.id
    puts "\tName: " + template.name
    puts "\tCreated Time: " + template.createdTime.to_s()
    puts "\tDescription: " + template.description
    puts "\tModified Time: " + template.modifiedTime.to_s()
    puts "\tOrganization: " + template.organizationId.id
    puts "\tUserId: " + template.ownerId.id
    puts "\tAttchment: " + template.attachmentId[0].id
    puts "\tContentBlock: " + template.contentBlockId[0].id
    puts "\tHeader: " + template.header.to_s
    puts "\tBodyEncoding: " + template.bodyEncoding.to_s
    puts "\tFromName: " + template.fromName
    puts "\tSubject: " + template.subject
    puts "\tMessagePart: " + template.messagePart[0].content
  end

  
  # Import message part using a zip file and media server.
  #  templateId is the ID of template where message part will be imported.
  #  mediaServerId is the media server ID used to store the image files.
  #  zipFile is the zip file that contains HTML and image files.
  #  folderName is the directory to unzip the zip file.
  #  return if the importing operaion is successful.
  def importMessagePart(templateId, mediaServerId, zipFile, folderName)
    # Create request
    request = ImportMessagePartRequest.new
    request.templateId = templateId
    request.folderName = folderName
    request.isXsl = false
    request.mediaServerId = mediaServerId

    # Set content from zip file
    request.zipFile = Base64.encode64(File.open(zipFile, "r") {|io| io.read})

    return @svc.importMessagePart(request).success
  end


  # Test template for its dynamic content.
  #  Email will be sent and tokens in the template will be replaced wit values.
  #  templateId is the templateId that will be tested.
  #  return if the test request is sent successfully.
  def testTemplate(templateId)
    # Create request
    request = TestTemplateRequest.new
    request.subjectPrefix = "Subject Prefix: "
    request.templateId = templateId
    request.useMultiPart = false
    request.address = ["test_address@testtemplate.com"]
    request.format = "HTML"

    # Make sure you have these two tokens in your template
    pair1 = NameValuePair.new
    pair1.name = "EMAIL"
    pair1.value = "testmail@testtemplate.com"
    pair2 = NameValuePair.new
    pair2.name = "NAME"
    pair2.value = "testmail name"

    # Set data to request and make call
    request.tokenValue = [pair1, pair2]
    return @svc.test(request).success
  end


  # Validate if a template's xsl content is valid.
  def validateXsl(templateId)
    request = ValidateXslRequest.new
    request.templateId = templateId
    request.messageFormat = "HTML"

    return @svc.validateXsl(request).valid
    
  end
end
