# Copyright 2014 StrongMail Systems, Inc.  All rights reserved.

require 'MailingServiceDriver'
require 'soap/header/simplehandler'


#
# Use header handler to set the authentication headers in each request.
#
class WsseAuthHeader < SOAP::Header::SimpleHandler
  NAMESPACE = 'http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd'
  SM_SCH = "http://www.strongmail.com/services/2009/03/02/schema"

  Header = XSD::QName.new(NAMESPACE, 'Security')
  OrgToken = XSD::QName.new(SM_SCH, 'OrganizationToken')


  def initialize(username, password, organization)
    super(Header)
    @username = username
    @password = password
    @organization = organization
  end

  #
  # Callback method called by soap4r on each request
  # to insert the header.
  #
  def on_simple_outbound
    pwd = SOAP::SOAPElement.new(XSD::QName.new(nil, "Password"))
    pwd.extraattr["Type"] = "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#PasswordText"
    pwd.text = @password

    {
            "UsernameToken" => {
                    "Username" => @username,
                    "Password" => pwd,
                    },
            OrgToken => {
                    XSD::QName.new(SM_SCH, 'organizationName') => @organization,
                    #XSD::QName.new(SM_SCH, 'subOrganizationId') => "",
            },
    }
  end


  #
  # Static method to return the service end point
  #
  def self.sm_service(opts={})
    # create the service
    obj = MailingService.new(opts["endpoint_url"])

    # run ruby with -d to see SOAP wiredumps.
    obj.wiredump_dev = STDERR if $DEBUG

    # attach auth header handler
    obj.headerhandler << WsseAuthHeader.new(opts["username"], opts["password"], opts["organization"])

    obj.options['protocol.http.ssl_config.verify_mode']  = OpenSSL::SSL::VERIFY_NONE
    # Since MS uses TLS protocol, we need to set the below property to make client to use TLS protocol.
    obj.options['protocol.http.ssl_config.ssl_version']  = :TLSv1_2

    obj
  end
end
