module SM

  #
  # Use header handler to set the authentication headers in each
  # request.
  #
  class WsseAuthHeader < SOAP::Header::SimpleHandler
    NAMESPACE = 'http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd'
    USERNAME  = 'admin'
    PASSWORD  = 'admin'
    ORGANIZATION = 'admin'
    SM_SCH = "http://www.strongmail.com/services/2009/03/02/schema"

    Header = XSD::QName.new(NAMESPACE, 'Security')
    OrgToken = XSD::QName.new(SM_SCH, 'OrganizationToken')

    def initialize()
      super(Header)
    end

    #
    # Callback method called by soap4r on each request
    # to insert the header.
    #
    def on_simple_outbound
      pwd = SOAP::SOAPElement.new(XSD::QName.new(nil, "Password"))
      pwd.text = "admin"
      pwd.extraattr["Type"] = "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#PasswordText"

      {
        "UsernameToken" => {
          "Username" => USERNAME,
          "Password" => pwd,
        },
        OrgToken => {
          XSD::QName.new(SM_SCH, 'organizationName') => ORGANIZATION,
          XSD::QName.new(SM_SCH, 'subOrganizationId') => "",
        },
      }
    end
  end


  #
  # Static method to return the service end point
  #
  def self.sm_service(opts={})
    # create the service
    obj = MailingService.new(opts[:endpoint_url])

    # run ruby with -d to see SOAP wiredumps.
    obj.wiredump_dev = STDERR if $DEBUG

    # attach auth header handler
    obj.headerhandler << SM::WsseAuthHeader.new()

    obj
  end

  #
  # Static method to get a list of mailing id in 'EDITING'
  # status.
  #
  def self.list_mailing_by_status(svc,opts={:status => 'EDITING'
                                           ,:page_number => 0
                                           ,:records_per_page=>100})

    #setup te condition
    sts_cond = ArrayStringFilterCondition.new([opts[:status]])
    sts_cond.operator = "IN"

    #setup the filter
    m_filt = MailingFilter.new
    m_filt.statusCondition = sts_cond
    m_filt.recordsPerPage = opts[:records_per_page]

    #setup the request
    l_req = ListRequest.new
    l_req.filter = m_filt
    m_filt.pageNumber = opts[:page_number]

    #issue the request and return the result
    svc.list(l_req)
  end

  #
  # Static method to return the mailing object by ID
  #
  def self.get_mailing_by_id(svc,opts={})

    #setup the mailing id
    mid = MailingId.new(opts[:id])

    #setup the request
    req = GetRequest.new([mid])

    #issue the request
    orig_resp = svc.get(req)

    #extract the base object if no errors
    resp = nil
    begin
      if orig_resp.success[0] == true
        resp = orig_resp.getResponse[0].baseObject
      end
    rescue => bang
      resp = nil
    end
  end


  #
  # Static method to approve the :mailing (in opt's hash)
  #
  def self.approve_mailing(svc,opts={})

    #setup the mailing object
    mailing = opts[:mailing]
    mailing.isApproved = true

    #setup the request
    req = UpdateRequest.new
    req.push(mailing)

    #update the mailing
    begin
      svc.update(req)
    rescue => bang
      $stderr << bang.to_s << "\n"
    end
  end

  #
  # Static method to create an IDS. The IDS name
  # is in the opts hash (:ids_name). The IDS
  # created will contain a single field 'email'
  #
  def self.create_ids(svc,opts = {})

    #setup the single email field
    field = DataSourceField.new
    field.name = "email"
    field.isPrimaryKey = true
    field.writebackEnabled = false
    field.fieldType = "EMAIL"
    field.dataType = "DATA"

    #setup the field array
    fields = []
    fields.push(field)

    #setup the ids
    ids = InternalDataSource.new
    ids.name = opts[:ids_name]
    ids.field = fields

    #setup the request; this is an ::Array base type
    req = CreateRequest.new
    req << ids

    #issue the request and return result
    svc.create(req)
  end

  #
  # Static method to add records to IDS. Note that
  # the recs array elements is of Hash type containing
  # :email field.
  #
  def self.add_records(svc,recs=[], opts={})

    #setup the the data source id
    ds_id = InternalDataSourceId.new
    ds_id.id = opts[:ids_id]

    #setup the records to be inserted
    d_recs = []
    recs.each do |rec|
      nv = NameValuePair.new
      nv.name = "email"
      nv.value = rec[:email]
      d_recs.push [nv]
    end

    #setup the request
    req = AddDataSourceRecordsRequest.new
    req.dataSourceId = ds_id
    req.containsFieldNames = false
    req.dataSourceRecord = d_recs

    #issue the request and return result
    svc.addRecords(req)
  end

end
