/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2013 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

package com.strongmail.webserviceclient.samples.cxf;

import com.strongmail.webserviceclient.samples.cxf.generated.MailingService;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.Attachment;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.AttachmentId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ImportAttachmentContentRequest;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import javax.activation.DataHandler;
import javax.mail.util.ByteArrayDataSource;


public class AttachmentSample
{
  private MailingService service;
  private SampleHelper helper;

  public AttachmentSample(MailingService service)
  {
    this.service = service;
    helper = new SampleHelper(service);
  }

  /**
   * Create an attachment using a file.
   *
   * @param attachmentName the name of the attachment
   * @param attachmentFile the actual file for the attachment
   * @return attachmentId
   * @throws Exception any exception
   */
  public AttachmentId createAttachment(String attachmentName,
                                       File attachmentFile) throws Exception
  {
    Attachment attachment = new Attachment();

    attachment.setDescription("Attachment from CXF");
    attachment.setFileName(attachmentFile.getName());
    attachment.setName(attachmentName);

    // Set content
    InputStream is = new FileInputStream(attachmentFile);
    DataHandler dh = new DataHandler(new ByteArrayDataSource(is, "application/octet-stream"));
    attachment.setContent(dh);

    // Make call
    return (AttachmentId)helper.create(attachment);
  }

  /**
   * Get and print attachment object to standard out.
   *
   * @param attachmentId the attachment ID that will be retrieved and printed.
   * @throws Exception any exception
   */
  public void getAndPrint(AttachmentId attachmentId) throws Exception
  {
    Attachment attachment = (Attachment)helper.get(attachmentId);

    // Print Attachment to standard out
    System.out.println("\nAttachment:");
    System.out.println("AttachmentId: " + attachment.getObjectId().getId());
    System.out.println("Name: " + attachment.getName());
    System.out.println("CampaignId: " + attachment.getCampaignId());
    System.out.println("CreatedTime: " + attachment.getCreatedTime());
    System.out.println("Description: " + attachment.getDescription());
    System.out.println("ModifiedTime: " + attachment.getModifiedTime());
    System.out.println("OrganizationId: " + attachment.getOrganizationId().getId());
    System.out.println("OwnerId: " + attachment.getOwnerId().getId());
    System.out.println("Size: " + attachment.getSize());
  }

  /**
   * Import attachment.
   *
   * @param attachmentId   the AttachmentId that will be imported to.
   * @param attachmentFile the attachment file that will be imported.
   * @return if the importing operation is successful.
   * @throws Exception any exception
   */
  public boolean importContent(AttachmentId attachmentId, File attachmentFile) throws Exception
  {
    ImportAttachmentContentRequest request = new ImportAttachmentContentRequest();
    request.setAttachmentId(attachmentId);
    request.setFileName(attachmentFile.getName());

    // Set content
    InputStream is = new FileInputStream(attachmentFile);
    DataHandler dh = new DataHandler(new ByteArrayDataSource(is, "application/octet-stream"));
    request.setContent(dh);

    return service.importContent(request).isSuccess();
  }
}
