/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2014 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

package com.strongmail.webserviceclient.samples.cxf;

import com.strongmail.webserviceclient.samples.cxf.generated.BadHandleFault;
import com.strongmail.webserviceclient.samples.cxf.generated.MailingService;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ArrayStringFilterCondition;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.AttachmentId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.CancelRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.CharSet;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.CloseRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ContentBlockId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.Encoding;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.FilterArrayOperator;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.FilterStringScalarOperator;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.GetTxnEasInfoRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.GetTxnEasInfoResponse;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.GetTxnMailingHandleRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.LaunchRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ListRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ListResponse;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.LoadRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.Mailing;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.MailingFilter;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.MailingId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.MailingOrderBy;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.MailingPriority;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.MailingStatus;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.MailingType;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.MessageFormat;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.MessageType;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.NameValuePair;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ObjectId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.PauseRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ResumeRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ScalarStringFilterCondition;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SchedulableMailing;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SeedListId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ScheduleRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SendRecord;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.StandardMailing;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.StandardMailingId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SuppressionListId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SystemAddressId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TargetId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TemplateId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TransactionalMailing;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TransactionalMailingId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TxnSendRequest;

import java.util.Date;
import java.util.List;


public class MailingSample
{
  private MailingService service;
  private SampleHelper helper;
  private SystemAddressId bounceId;
  private SystemAddressId fromId;
  private SystemAddressId replyId;

  public MailingSample(MailingService service, SystemAddressId bounceId,
                       SystemAddressId fromId, SystemAddressId replyId)
  {
    this.service = service;
    helper = new SampleHelper(service);
    this.bounceId = bounceId;
    this.fromId = fromId;
    this.replyId = replyId;
  }


  /**
   * Create a recurring mailing with no excluded target.
   *
   * @param mailingName    the name of the mailing to be created
   * @param targetId       the TargetId to be used with this mailing.
   * @param seedId         the SeedListId to be used with this mailing.
   * @param suppressId     the SuppressionListId to be used with this mailing.
   * @param attachmentId   the AttachmentId to be used with this mailing.
   * @param contentBlockId the ContentBlockId to be used with this mailing.
   * @param templateId     the TempalteId to be used with this mailing.
   * @return MailingId
   * @throws Exception any exception
   */
  public StandardMailingId createStandardMailing(String mailingName,
                                                 TargetId targetId,
                                                 SeedListId seedId,
                                                 SuppressionListId suppressId,
                                                 AttachmentId attachmentId,
                                                 ContentBlockId contentBlockId,
                                                 TemplateId templateId) throws Exception
  {
    StandardMailing mailing = new StandardMailing();
    setMailingCommonFields(mailing, mailingName, bounceId, fromId, replyId, templateId);

    mailing.setDescription("StandardMailing from CXF");
    mailing.setEliminateDuplicates(true);
    mailing.setType(MailingType.RECURRING);

    // Set schedule: daily recurrence with every two days
    SchedulableMailing.Schedule.Recurrence.DailyRecurrence daily =
      new SchedulableMailing.Schedule.Recurrence.DailyRecurrence();
    daily.setInterval(2);

    SchedulableMailing.Schedule.Recurrence recurrence = new SchedulableMailing.Schedule.Recurrence();
    recurrence.setEndAfterXMailings(100); // launch 100 mailing;
    recurrence.setDailyRecurrence(daily);

    SchedulableMailing.Schedule schedule = new SchedulableMailing.Schedule();
    schedule.setRecurrence(recurrence);
    schedule.setStartDateTime(new Date(System.currentTimeMillis() + 3600000)); // one hour later
    mailing.setSchedule(schedule);

    // Set dependencies
    mailing.getAttachmentId().add(attachmentId);
    mailing.getContentBlockId().add(contentBlockId);
    mailing.getIncludedTargetId().add(targetId);
    mailing.getSeedListId().add(seedId);
    mailing.getSuppressionListId().add(suppressId);

    // Make call
    return (StandardMailingId)helper.create(mailing);
  }

 /**
   * Create a one time mailing with no excluded target.
   *
   * @param mailingName    the name of the mailing to be created
   * @param targetId       the TargetId to be used with this mailing.
   * @param seedId         the SeedListId to be used with this mailing.
   * @param suppressId     the SuppressionListId to be used with this mailing.
   * @param attachmentId   the AttachmentId to be used with this mailing.
   * @param contentBlockId the ContentBlockId to be used with this mailing.
   * @param templateId     the TempalteId to be used with this mailing.
   * @return MailingId
   * @throws Exception any exception
   */
 public StandardMailingId createOneTimeStandardMailing(
   String mailingName,
   TargetId targetId,
   SeedListId seedId,
   SuppressionListId suppressId,
   AttachmentId attachmentId,
   ContentBlockId contentBlockId,
   TemplateId templateId
 ) throws Exception
 {
   StandardMailing mailing = new StandardMailing();
   setMailingCommonFields(mailing, mailingName, bounceId, fromId, replyId, templateId);

   mailing.setDescription("StandardMailing from CXF");
   mailing.setEliminateDuplicates(true);
   mailing.setType(MailingType.ONE_TIME);

   // Set dependencies
   mailing.getAttachmentId().add(attachmentId);
   mailing.getContentBlockId().add(contentBlockId);
   mailing.getIncludedTargetId().add(targetId);
   mailing.getSeedListId().add(seedId);
   mailing.getSuppressionListId().add(suppressId);

   // Make call
   return (StandardMailingId)helper.create(mailing);
 }


  /**
   * Create a Transactional Mailing.
   *
   * @param mailingName the name of the mailing
   * @param templateId  the template used for the mailing
   * @param targetId    the target used for the mailing
   * @return TransactionalMailingId
   * @throws Exception any exception
   */
  public TransactionalMailingId createTransactionalMailing(String mailingName,
                                                           TemplateId templateId,
                                                           TargetId targetId)
    throws Exception
  {

    TransactionalMailing mailing = new TransactionalMailing();
    setMailingCommonFields(mailing, mailingName, bounceId, fromId, replyId, templateId);

    mailing.setDescription("TransactionalMailing from CXF");
    mailing.setType(MailingType.TRANSACTIONAL);
    mailing.setTargetId(targetId);
    mailing.setMessageType(MessageType.EMAIL);

    return (TransactionalMailingId)helper.create(mailing);
  }


  private void setMailingCommonFields(Mailing mailing,
                                      String mailingName,
                                      SystemAddressId bounceId,
                                      SystemAddressId fromId,
                                      SystemAddressId replyId,
                                      TemplateId templateId)
  {
    mailing.setBodyEncoding(Encoding.BASE_64);
    mailing.setBounceAddressId(bounceId);
    mailing.setFieldDelimiter("::");
    mailing.setFromAddressId(fromId);
    mailing.setFromName("From Name");
    mailing.setHeaderEncoding(Encoding.EIGHT_BIT);
    mailing.setIsApproved(true);
    mailing.setIsCompliant(true);
    mailing.setName(mailingName);
    mailing.setOutputBodyCharSet(CharSet.UTF_8);
    mailing.setOutputHeaderCharSet(CharSet.ASCII);
    mailing.setPriority(MailingPriority.NORMAL);
    mailing.setReplyAddressId(replyId);
    mailing.setRowDelimiter("\n");
    mailing.setSubject("mailing subject");
    mailing.setTemplateId(templateId);
    mailing.getFormat().add(MessageFormat.HTML);
    mailing.getHeader().add("mailing header");
  }

  public void getAndPrint(MailingId mailingId) throws Exception
  {
    Mailing mailing = (Mailing)helper.get(mailingId);

    // Print Mailing to standard out
    if (mailingId instanceof StandardMailingId)
    {
      System.out.println("\nStandard Mailing");
    }
    else if (mailingId instanceof TransactionalMailingId)
    {
      System.out.println("\nTransactional Mailing");
    }
    System.out.println("MailingId: " + mailing.getObjectId().getId());
    System.out.println("Name: " + mailing.getName());
    System.out.println("CampaignId: " + mailing.getCampaignId());
    System.out.println("CreatedTime: " + mailing.getCreatedTime());
    System.out.println("Description: " + mailing.getDescription());
    System.out.println("ModifiedTime: " + mailing.getModifiedTime());
    System.out.println("OrganizationId: " + mailing.getOrganizationId().getId());
    System.out.println("OwnerId: " + mailing.getOwnerId().getId());
    System.out.println("FieldDelimiter: " + mailing.getFieldDelimiter());
    System.out.println("RowDelimiter: " + mailing.getRowDelimiter());
    System.out.println("MessageFormats: " + mailing.getFormat());
    System.out.println("FromName: " + mailing.getFromName());
    System.out.println("Headers: " + mailing.getHeader());
    System.out.println("Template: " + mailing.getTemplateId().getId());
    System.out.println("Bounce: " + mailing.getBounceAddressId().getId());
    System.out.println("From: " + mailing.getFromAddressId().getId());
    System.out.println("Reply: " + mailing.getReplyAddressId().getId());
    System.out.println("BodyEncoding: " + mailing.getBodyEncoding().value());
    System.out.println("HeaderEncoding: " + mailing.getHeaderEncoding().value());
    System.out.println("LastGoodStatus: " + mailing.getLastGoodStatus());
    System.out.println("MailingClass: " + mailing.getMailingClassId().getId());
    System.out.println("BodyCharset: " + mailing.getOutputBodyCharSet().value());
    System.out.println("BodyOutputToken: " + mailing.getOutputBodyCharSetToken());
    System.out.println("HeaderCharset: " + mailing.getOutputHeaderCharSet().value());
    System.out.println("HeaderOutputToken: " + mailing.getOutputHeaderCharSetToken());
    System.out.println("Parent: " + mailing.getParentId());
    System.out.println("PlannedLaunchDate: " + mailing.getPlannedLaunchDate());
    System.out.println("Priority: " + mailing.getPriority());
    System.out.println("ServerErrorCode: " + mailing.getServerErrorCode());
    System.out.println("ServerErrorMessage: " + mailing.getServerErrorMessage());
    System.out.println("Status: " + mailing.getStatus());
    System.out.println("Subject: " + mailing.getSubject());
    System.out.println("Type: " + mailing.getType());

    if (mailing instanceof StandardMailing)
    {
      StandardMailing m = (StandardMailing)mailing;
      System.out.println("IncludedTarget: " + SampleHelper.formatIdList(m.getIncludedTargetId()));
      System.out.println("SeedList: " + SampleHelper.formatIdList(m.getSeedListId()));
      System.out.println("SuppressionList: " + SampleHelper.formatIdList(m.getSuppressionListId()));
      System.out.println("AttachmentId: " + SampleHelper.formatIdList(mailing.getAttachmentId()));
      System.out.println("ContentBlockId: " + SampleHelper.formatIdList(mailing.getContentBlockId()));
    }
    else if (mailing instanceof TransactionalMailing)
    {
      TransactionalMailing m = (TransactionalMailing)mailing;
      System.out.println("FormatField: " + m.getFormatFieldName());
      System.out.println("MailingConfig: " + m.getMailingConfig());
      System.out.println("MessageType: " + m.getMessageType());
      System.out.println("RecordStructure: " + m.getRecordStructure());
      System.out.println("SenderNumber: " + m.getSenderNumber());
      System.out.println("Target: " + m.getTargetId().getId());
    }
  }

  /**
   * Launch a Standard Mailing.
   *
   * @param mailingId the ID of Standard Mailing that will be launched
   * @return if launch operation is successful
   * @throws Exception any exception
   */
  public boolean launchMailing(StandardMailingId mailingId) throws Exception
  {
    LaunchRequest request = new LaunchRequest();
    request.setMailingId(mailingId);

    return service.launch(request).isSuccess();
  }

  /**
   * Launch a Standard Mailing.
   *
   * @param mailingId the ID of Standard Mailing that will be launched
   * @return if launch operation is successful
   * @throws Exception any exception
   */
  public boolean scheduleMailing(StandardMailingId mailingId) throws Exception
  {
    ScheduleRequest request = new ScheduleRequest();
    request.setMailingId(mailingId);

    return service.schedule(request).isSuccess();
  }

  /**
   * Load a Transactional Mailing.
   *
   * @param mailingId the ID of Transactional Mailing that will be loaded
   * @return if load operation is successful
   * @throws Exception any exception
   */
  public boolean loadMailing(TransactionalMailingId mailingId) throws Exception
  {
    LoadRequest request = new LoadRequest();
    request.setMailingId(mailingId);

    return service.load(request).isSuccess();
  }


  /**
   * Fetch the handle for a tx mailing, that is to be used in the txnSend API.   
   *
   * @param mailingId
   * @return mailing handle
   * @throws Exception
   */
  public String getTxnMailingHandle(TransactionalMailingId mailingId) throws Exception
  {
    // Create request and set mailingId
    GetTxnMailingHandleRequest request = new GetTxnMailingHandleRequest();
    request.setMailingId(mailingId);
    return service.getTxnMailingHandle(request).getHandle();
  }

  public boolean txnSend(String mailingHandle) throws Exception
  {
    // Create request and set mailingId
    TxnSendRequest request = new TxnSendRequest();
    request.setHandle(mailingHandle);

    // Create record based on the target created in TargetSample
    SendRecord record = new SendRecord();

    NameValuePair pair1 = new NameValuePair();
    pair1.setName("ID");
    pair1.setValue("10");
    record.getField().add(pair1);

    NameValuePair pair2 = new NameValuePair();
    pair2.setName("EMAIL_ADDRESS");
    pair2.setValue("sample1@cxf.sample.com");
    record.getField().add(pair2);

    NameValuePair pair3 = new NameValuePair();
    pair3.setName("NAME");
    pair3.setValue("First Last");
    record.getField().add(pair3);

    // Add record and make call
    request.getSendRecord().add(record);
    boolean success = false;
    try
    {
      success = service.txnSend(request).isSuccess();
    }
    catch (BadHandleFault e)
    {
      System.out.println("Encountered a BadHandleFault. " + e.getFaultInfo().getFaultMessage());
      System.out.println("This indicates that the handle being used to attempt the txnSend call is incorrect.");
      System.out.println("Please ensure that you send the correct handle using the getTxnMailingHandle() API.");
    }
    return success;
  }

  public void printTxnEASInfo(TransactionalMailingId mailingId) throws Exception
  {
    // Create request and set mailingId
    GetTxnEasInfoRequest request = new GetTxnEasInfoRequest();
    request.setMailingId(mailingId);
    GetTxnEasInfoResponse response = service.getTxnEasInfo(request);

    System.out.println("EAS Info for Tx mailing ID: " + mailingId.getId());
    System.out.println("EAS on which the txn mailing has been loaded: " + response.getHostname());
    System.out.println("Txn mailing handle: " + response.getMailingHandle());
    System.out.println("Txn mailing name: " + response.getMailingName());
  }

  /**
   * Close a Transactional Mailing.
   *
   * @param mailingId the ID of Transactional Mailing that will be closed
   * @return if close operation is successful
   * @throws Exception any exception
   */
  public boolean close(TransactionalMailingId mailingId) throws Exception
  {
    CloseRequest request = new CloseRequest();
    request.setMailingId(mailingId);

    return service.close(request).isSuccess();
  }

  public boolean cancel(MailingId mailingId) throws Exception
  {
    CancelRequest request = new CancelRequest();
    request.setMailingId(mailingId);

    return service.cancel(request).isSuccess();
  }


  public boolean pause(MailingId mailingId) throws Exception
  {
    PauseRequest request = new PauseRequest();
    request.setMailingId(mailingId);

    return service.pause(request).isSuccess();
  }


  public boolean resume(MailingId mailingId) throws Exception
  {
    ResumeRequest request = new ResumeRequest();
    request.setMailingId(mailingId);

    return service.resume(request).isSuccess();
  }

  /**
   * List mailings by name: this should return one mailing if name is matched.
   *
   * @param name the mailing name to be listed
   * @throws Exception any exception
   */
  public void listByName(String name) throws Exception
  {
    MailingFilter filter = new MailingFilter();
    helper.setFilter(filter);
    filter.getOrderBy().add(MailingOrderBy.MODIFIED_TIME);

    // Create name condition
    ScalarStringFilterCondition nameCondition = new ScalarStringFilterCondition();
    nameCondition.setOperator(FilterStringScalarOperator.EQUAL);
    nameCondition.setValue(name);
    filter.setNameCondition(nameCondition);

    // Set the filter to request
    ListRequest request = new ListRequest();
    request.setFilter(filter);

    // Make call and print results
    printListResponse(service.list(request));
  }

  /**
   * List mailing in an organization: this can be done using an empty filter. The organization is
   * the one from header.
   *
   * @throws Exception any exception
   */
  public void listByOrganization() throws Exception
  {
    // Create empty filter: no conditions
    MailingFilter filter = new MailingFilter();
    helper.setFilter(filter);
    filter.getOrderBy().add(MailingOrderBy.MODIFIED_TIME);

    // Set the filter to request
    ListRequest request = new ListRequest();
    request.setFilter(filter);

    // Make call and print results
    printListResponse(service.list(request));
  }

  /**
   * List mailings by mailing type and status.
   *
   * @throws Exception any exception
   */
  public void listByTypeAndStatus() throws Exception
  {
    // Create a mailing filter and set attribute and order by list
    MailingFilter filter = new MailingFilter();
    helper.setFilter(filter);
    filter.getOrderBy().add(MailingOrderBy.NAME);
    filter.getOrderBy().add(MailingOrderBy.MODIFIED_TIME);

    // Create status condition: status in (ACTIVE, EDITING, COMPLETED)
    ArrayStringFilterCondition statusCondition = new ArrayStringFilterCondition();
    statusCondition.setOperator(FilterArrayOperator.IN);
    statusCondition.getValue().add(MailingStatus.ACTIVE.value());
    statusCondition.getValue().add(MailingStatus.EDITING.value());
    statusCondition.getValue().add(MailingStatus.COMPLETED.value());
    filter.setStatusCondition(statusCondition);

    // Create Type Condition: type in (RECURRING, TRANSACTIONAL)
    ArrayStringFilterCondition typeCondition = new ArrayStringFilterCondition();
    typeCondition.setOperator(FilterArrayOperator.IN);
    typeCondition.getValue().add(MailingType.RECURRING.value());
    typeCondition.getValue().add(MailingType.TRANSACTIONAL.value());
    filter.setTypeCondition(typeCondition);

    // Set the filter to request
    ListRequest request = new ListRequest();
    request.setFilter(filter);

    // Make call and print results
    printListResponse(service.list(request));
  }

  private void printListResponse(ListResponse response) throws Exception
  {
    List<ObjectId> mailingIds = response.getObjectId();

    System.out.println("\nList Response:");
    for (ObjectId each : mailingIds)
    {
      Mailing mailing = (Mailing)helper.get(each);
      System.out.println("\t" + each.getId() + " " + mailing.getName());
    }
  }

  public MailingStatus getMailingStatus(MailingId mailingId) throws Exception
  {
    Mailing mailing = (Mailing)helper.get(mailingId);

    return mailing.getStatus();
  }

  public MailingStatus waitForStatus(MailingId mailingId,
                                     MailingStatus desiredStatus)
    throws Exception
  {
    MailingStatus status;

    // Return immediately if desired status is received, or if any permanent
    // status is encountered.
    while ((status = getMailingStatus(mailingId)) != desiredStatus &&
           status != MailingStatus.ERROR &&
           status != MailingStatus.FAILED &&
           status != MailingStatus.COMPLETED &&
           status != MailingStatus.CANCELED &&
           status != MailingStatus.SPEC_CANCELED)
    {
      System.out.println("waitForStatus on " + mailingId.getId() + ": " + status + "!=" + desiredStatus);
      Thread.sleep(10000);
    }
    System.out.println("waitForStatus on " + mailingId.getId() + " final: " + status +
                       (status == desiredStatus ? "==" : "!=") + desiredStatus);
    return status;
  }
}
