/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2014 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

package com.strongmail.webserviceclient.samples.cxf;

import com.strongmail.webserviceclient.samples.cxf.generated.MailingService;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.AttachmentId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ContentBlockId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.DataSourceId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.InternalDataSourceId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.InternalDataSourceExtnId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.MailingId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.MailingStatus;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ObjectId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.RuleId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SeedListId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.StandardMailingId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SuppressionListId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SystemAddress;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SystemAddressId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SystemAddressType;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TargetId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TemplateId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TransactionalMailingId;

import java.io.File;
import java.util.ArrayList;
import java.util.List;


public class SampleProgram
{
  private static final String IDS_DATA_FILE = "../data/Shared_DataSource_Records.txt";
  private static final String ATTACHMENT_FILE = "../data/Shared_Attachment.gif";

  private DataSourceSample dsSample;
  private TargetSample targetSample;
  private SeedListSample seedListSample;
  private SuppressionListSample suppressionListSample;
  private TemplateSample templateSample;
  private AttachmentSample attachmentSample;
  private ContentBlockSample contentBlockSample;
  private RuleSample ruleSample;
  private MailingSample mailingSample;
  private ProgramSample programSample;

  public SampleProgram(MailingService svc) throws Exception
  {
    dsSample = new DataSourceSample(svc);
    targetSample = new TargetSample(svc);
    seedListSample = new SeedListSample(svc);
    suppressionListSample = new SuppressionListSample(svc);
    templateSample = new TemplateSample(svc);
    attachmentSample = new AttachmentSample(svc);
    contentBlockSample = new ContentBlockSample(svc);
    ruleSample = new RuleSample(svc);
    programSample = new ProgramSample(svc);

    SystemAddressSample addressSample;
    addressSample = new SystemAddressSample(svc);

    // You need to create these data from UI
    SystemAddress bounce = addressSample.listByEmailAddress(SystemAddressType.BOUNCE,
                                                            "bounce@sample.com");
    SystemAddress from = addressSample.listByEmailAddress(SystemAddressType.FROM,
                                                          "from@sample.com");
    SystemAddress reply = addressSample.listByEmailAddress(SystemAddressType.REPLY,
                                                           "reply@sample.com");

    mailingSample = new MailingSample(svc,
                                      (SystemAddressId)bounce.getObjectId(),
                                      (SystemAddressId)from.getObjectId(),
                                      (SystemAddressId)reply.getObjectId());
  }

  /**
   * Create 11 objects: Data Sources:    IDS, EDS, Target, SeedList, SuppressionList, Content
   * Library: Attachment, ContentBlock, Template, Rule, Mailings:        StandardMailing, and
   * TransactionalMailing
   *
   * @param sequence a sequence number to differentiate the name for each run
   * @return list of ObjectIds that was created
   * @throws Exception any exception
   */
  public List<ObjectId> createAllData(String sequence) throws Exception
  {
    List<ObjectId> objectIds = new ArrayList<ObjectId>();

    objectIds.add(dsSample.createIDS("ids_cxf_" + sequence));

    // create IDX
    objectIds.add(dsSample.createIDX("idx_cxf_" + sequence));

    // EDS: need to change DataSourceSample to point to your database
    //objectIds.add(dsSample.createEDS("eds_cxf_" + sequence));

    objectIds.add(targetSample.createTarget(
      "target_cxf_" + sequence,
      (InternalDataSourceId)getObjectId(objectIds, InternalDataSourceId.class))
    );

    objectIds.add(seedListSample.createSeedList("seedList_cxf_" + sequence));

    objectIds.add(suppressionListSample.createSuppressionList("suppressionList_cxf_" + sequence));

    objectIds.add(attachmentSample.createAttachment(
      "attachment_cxf_" + sequence,
      new File(ATTACHMENT_FILE)));

    objectIds.add(contentBlockSample.createContentBlock("contentBlock_cxf_" + sequence));

    objectIds.add(ruleSample.createRule(
      "rule_cxf_" + sequence,
      (TargetId)getObjectId(objectIds, TargetId.class))
    );

    objectIds.add(templateSample.createTemplate(
      "template_cxf_" + sequence,
      (AttachmentId)getObjectId(objectIds, AttachmentId.class),
      (ContentBlockId)getObjectId(objectIds, ContentBlockId.class))
    );

    objectIds.add(mailingSample.createStandardMailing(
      "stdmailing_cxf_" + sequence,
      (TargetId)getObjectId(objectIds, TargetId.class),
      (SeedListId)getObjectId(objectIds, SeedListId.class),
      (SuppressionListId)getObjectId(objectIds, SuppressionListId.class),
      (AttachmentId)getObjectId(objectIds, AttachmentId.class),
      (ContentBlockId)getObjectId(objectIds, ContentBlockId.class),
      (TemplateId)getObjectId(objectIds, TemplateId.class))
    );

    objectIds.add(mailingSample.createTransactionalMailing(
      "txmailing_cxf_" + sequence,
      (TemplateId)getObjectId(objectIds, TemplateId.class),
      (TargetId)getObjectId(objectIds, TargetId.class))
    );

    return objectIds;
  }


  /**
   * Get and print out the object content.
   *
   * @param objectIds the list objectId whose content will be printed.
   * @throws Exception any exception
   */
  public void getAndPrintAllData(List<ObjectId> objectIds) throws Exception
  {
    for (ObjectId objectId : objectIds)
    {
      if (objectId instanceof DataSourceId)
      {
        dsSample.getAndPrint((DataSourceId)objectId);
      }
      else if (objectId instanceof TargetId)
      {
        targetSample.getAndPrint((TargetId)objectId);
      }
      else if (objectId instanceof SeedListId)
      {
        seedListSample.getAndPrint((SeedListId)objectId);
      }
      else if (objectId instanceof SuppressionListId)
      {
        suppressionListSample.getAndPrint((SuppressionListId)objectId);
      }
      else if (objectId instanceof AttachmentId)
      {
        attachmentSample.getAndPrint((AttachmentId)objectId);
      }
      else if (objectId instanceof ContentBlockId)
      {
        contentBlockSample.getAndPrint((ContentBlockId)objectId);
      }
      else if (objectId instanceof RuleId)
      {
        ruleSample.getAndPrint((RuleId)objectId);
      }
      else if (objectId instanceof TemplateId)
      {
        templateSample.getAndPrint((TemplateId)objectId);
      }
      else if (objectId instanceof MailingId)
      {
        mailingSample.getAndPrint((MailingId)objectId);
      }
    }
  }

  public static MailingService createService()
  {

    // Read key store, key store password, wsdl file location and endpoint address
    // Endpoint address will replace the one in WSDL file
    System.out.println("Store: " + System.getProperty("javax.net.ssl.trustStore"));
    System.out.println("Password: " + System.getProperty("javax.net.ssl.trustStorePassword"));
    System.out.println("WSDL: " + System.getProperty("wsdl"));
    System.out.println("endpoint: " + System.getProperty("endpoint.address"));

    File wsdlFile = new File(System.getProperty("wsdl"));
    String endpoint = System.getProperty("endpoint.address");

    try
    {
      // Create service
      ClientSetup clientSetup = new ClientSetup(System.getProperty("services.namespace"),
                                                System.getProperty("services.namespace.schema"));
      MailingService service = clientSetup.createService(wsdlFile, endpoint);

      // Apply interceptor to add SOAP header
      // username/password/organization are the same as UI login.
      clientSetup.applyInterceptor(service, "admin", "admin", "admin", null, false);
      /*
        For enabling SSO token authentication, set the last argument in the above method to true
        and specify the token in the password (i.e 2nd) argument. E.g.
      clientSetup.applyInterceptor(service, "admin", "<token>", "admin", null, true);
       */


      return service;
    }
    catch (Exception e)
    {
      e.printStackTrace();
      System.exit(1);
    }

    return null;
  }

  /**
   * Get an ObjectId based on its type.
   *
   * @param objectIds a list of objectIds
   * @param clazz     the Class whose Id will be returned
   * @return an ObjectId that matches the class
   */
  private ObjectId getObjectId(List<ObjectId> objectIds, Class clazz)
  {
    for (ObjectId objectId : objectIds)
    {
      if (clazz.isInstance(objectId))
      {
        return objectId;
      }
    }

    return null;
  }

  /* There are a few operations that are not called here, including
   * EDS: refreshRecords(), cancelRefreshRecords(),
   * Template: importMessagePart(), testTemplate(), validateXsl().
   * Mailing: cancel(), pause(), resume().
   * Please see corresponding sample program.
   */
  public static void main(String[] args) throws Exception
  {
    MailingService svc = SampleProgram.createService();
    List<ObjectId> objectIds = null;
    InternalDataSourceId copiedId = null;
    SampleProgram samples = new SampleProgram(svc);
    StandardMailingId stdMailingId = null;
    TransactionalMailingId txMailingId = null;

    try
    {
      // Create all data
      objectIds = samples.createAllData(args[0]);

      // Print out all data
      samples.getAndPrintAllData(objectIds);

      // Add data to IDS using file
      InternalDataSourceId dataSourceId =
        (InternalDataSourceId)samples.getObjectId(objectIds, InternalDataSourceId.class);

      InternalDataSourceExtnId idxDataSourceId =
        (InternalDataSourceExtnId)samples.getObjectId(objectIds, InternalDataSourceExtnId.class);

      samples.dsSample.addRecordsUsingFile(dataSourceId, new File(IDS_DATA_FILE));

      // Reuse the same file, since the definitions are same, except the primary key, which is different
      samples.dsSample.addRecordsUsingFile(idxDataSourceId, new File(IDS_DATA_FILE));

      // Upsert records into the datasource
      samples.dsSample.upsertRecord(dataSourceId);

      // Upsert records into the IDX datasource
      samples.dsSample.upsertRecord(idxDataSourceId);

      // Get upserted records for the datasource.
      samples.dsSample.getRecord(dataSourceId);

      // Get upserted records for the IDX datasource
      samples.dsSample.getRecord(idxDataSourceId);

      // Export data source records to a file
      File dataSourceExportFile = new File("./data_source_export.txt");
      samples.dsSample.exportRecords(dataSourceId, dataSourceExportFile);

      // Export IDX data source records to a file
      File idxDataSourceExportFile = new File("./idx_data_source_export.txt");
      samples.dsSample.exportRecords(idxDataSourceId, idxDataSourceExportFile);

      // Update the data source with new name
      samples.dsSample.updateDataSourceName(dataSourceId, "ids_cxf_updated");

      // Update the IDX data source with new name
      samples.dsSample.updateDataSourceName(idxDataSourceId, "idx_cxf_updated" + args[0]);

      // Get data source statistics
      samples.dsSample.getAndPrintStatistics(dataSourceId);

      // Get IDX data source statistics
      samples.dsSample.getAndPrintStatistics(idxDataSourceId);

      // Copy the data source
      copiedId = samples.dsSample.copyDataSource(dataSourceId, "ids_cxf_" + args[0] + "_copied");
      System.out.println("\nCopied IDS: " + copiedId.getId());

      // Dedupe data source records and print out new statistics
      samples.dsSample.dedupeRecords(dataSourceId);
      samples.dsSample.getAndPrintStatistics(dataSourceId);

      // Remove data source records and print out new statistics
      int removedDS = samples.dsSample.removeRecordsInline(dataSourceId);
      System.out.println("\nRemoved records: " + removedDS);
      samples.dsSample.getAndPrintStatistics(dataSourceId);

      // Remove data source records and print out new statistics
      int removedDX = samples.dsSample.removeRecordsInline(idxDataSourceId);
      System.out.println("\nRemoved records: " + removedDX);
      samples.dsSample.getAndPrintStatistics(idxDataSourceId);

      // Import new content to attachment using IDS record file
      AttachmentId attachmentId = (AttachmentId)samples.getObjectId(objectIds, AttachmentId.class);
      samples.attachmentSample.importContent(attachmentId, new File(IDS_DATA_FILE));
      samples.attachmentSample.getAndPrint(attachmentId);

      // Remove records from seed list
      SeedListId seedId = (SeedListId)samples.getObjectId(objectIds, SeedListId.class);
      int removedSeed = samples.seedListSample.removeRecordsInline(seedId);
      System.out.println("\nRemoved records: " + removedSeed);
      samples.seedListSample.getAndPrint(seedId);

      // Remove records from suppression list
      SuppressionListId suppressId = (SuppressionListId)samples.getObjectId(objectIds,
                                                                            SuppressionListId.class);
      int removedSuppress = samples.suppressionListSample.removeRecordsInline(suppressId);
      System.out.println("\nRemoved records: " + removedSuppress);
      samples.suppressionListSample.getAndPrint(suppressId);

      // List mailing by name
      samples.mailingSample.listByName("stdmailing_cxf_" + args[0]);

      // List mailing in an organization
      samples.mailingSample.listByOrganization();

      // List mailing by type and status
      samples.mailingSample.listByTypeAndStatus();


      System.out.println();
      stdMailingId =
        (StandardMailingId)samples.getObjectId(objectIds, StandardMailingId.class);
      txMailingId =
        (TransactionalMailingId)samples.getObjectId(objectIds, TransactionalMailingId.class);

      // Schedule a standard mailing
      boolean scheduled = samples.mailingSample.scheduleMailing(stdMailingId);
      System.out.println("Schedule mailing " + stdMailingId.getId() + ": " + scheduled);

      // Load a transactional mailing
      boolean loaded = samples.mailingSample.loadMailing(txMailingId);
      System.out.println("Load mailing " + txMailingId.getId() + ": " + loaded);

      // Wait for the transactional mailing to become ACTIVE.
      samples.mailingSample.waitForStatus(txMailingId, MailingStatus.ACTIVE);

      // txnSend and related APIs.
      samples.mailingSample.printTxnEASInfo(txMailingId);
      String txMailingHandle = samples.mailingSample.getTxnMailingHandle(txMailingId);
      System.out.println("txMailingHandle: " + txMailingHandle );

      boolean sent = samples.mailingSample.txnSend(txMailingHandle);
      System.out.println("Send record using txnSend for " + txMailingId.getId() + ": " + sent);

      // Close mailing
      boolean closed = samples.mailingSample.close(txMailingId);
      System.out.println("Close mailing " + txMailingId.getId() + ": " + closed);

      /*
      // Uncomment this block for running the following code that injects records to a program.
      // Please see the prerequisites (section 5) in the README before enabling this part.
      samples.programSample.injectRecordsIntoProgram();
       */

      samples.programSample.list();

    }
    finally
    {
      if ("true".equals(args[1]))
      {
        // true=delete the data
        // sleep to 5 seconds to let mailing finish.
        try
        {
          Thread.sleep(5000);
        }
        catch (InterruptedException e)
        {
          // ignore
        }

        // Delete all objects
        SampleHelper helper = new SampleHelper(svc);
        if (objectIds != null)
        {
          try
          {
            // Must wait for mailings to complete before they can be deleted
           // samples.mailingSample.waitForStatus(stdMailingId, MailingStatus.COMPLETED);
            samples.mailingSample.waitForStatus(txMailingId, MailingStatus.COMPLETED);
            helper.delete(objectIds);
          }
          catch (Exception e)
          {
            // ignore
          }
        }

        // Delete copied data source
        if (copiedId != null)
        {
          try
          {
            helper.delete(copiedId);
          }
          catch (Exception e)
          {
            // ignore
          }
        }
      }
    }
  }
}
