/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2013 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

package com.strongmail.webserviceclient.samples.cxf;

import com.strongmail.webserviceclient.samples.cxf.generated.MailingService;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.AddSeedListRecordsRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.RemoveSeedListRecordsRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SeedList;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SeedListId;


public class SeedListSample
{
  private MailingService service;
  private SampleHelper helper;

  public SeedListSample(MailingService service)
  {
    this.service = service;
    helper = new SampleHelper(service);
  }


  /**
   * Create a seed list and add some data.
   *
   * @param seedListName the name of the seed list
   * @return seedListId
   * @throws Exception any exception
   */
  public SeedListId createSeedList(String seedListName) throws Exception
  {
    SeedList seedList = new SeedList();

    seedList.setDescription("SeedList from CXF");
    seedList.setIsTestList(true);
    seedList.setName(seedListName);

    // Make call
    SeedListId seedListId = (SeedListId)helper.create(seedList);

    // Add some records
    addRecordsInline(seedListId);

    return seedListId;
  }


  /**
   * Add 2 records to seed list using inline records.
   *
   * @throws Exception any exception
   */
  public void addRecordsInline(SeedListId seedListId) throws Exception
  {
    AddSeedListRecordsRequest request = new AddSeedListRecordsRequest();
    request.setSeedListId(seedListId);
    request.getAddress().add("seed1@seed.cxf.com");
    request.getAddress().add("seed2@seed.cxf.com");

    service.addRecords(request);
  }

  /**
   * Remove one record that was created in addRecordsInline() from the SeedList.
   *
   * @param seedListId the SeedListId whose record will be removed.
   * @return the number of record that was removed.
   * @throws Exception any exception
   */
  public int removeRecordsInline(SeedListId seedListId) throws Exception
  {
    RemoveSeedListRecordsRequest request = new RemoveSeedListRecordsRequest();
    request.setSeedListId(seedListId);
    request.getAddress().add("seed1@seed.cxf.com");

    return service.removeRecords(request).getRecordsRemoved();
  }

  public void getAndPrint(SeedListId seedListId) throws Exception
  {
    SeedList seedList = (SeedList)helper.get(seedListId);

    // Print SeedList to standard out
    System.out.println("\nSeedList:");
    System.out.println("SeedListId: " + seedList.getObjectId().getId());
    System.out.println("Name: " + seedList.getName());
    System.out.println("CampaignId: " + seedList.getCampaignId());
    System.out.println("CreatedTime: " + seedList.getCreatedTime());
    System.out.println("Description: " + seedList.getDescription());
    System.out.println("ModifiedTime: " + seedList.getModifiedTime());
    System.out.println("OrganizationId: " + seedList.getOrganizationId().getId());
    System.out.println("OwnerId: " + seedList.getOwnerId().getId());
    System.out.println("Total Records: " + seedList.getTotalRecords());
  }
}
