/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2013 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

package com.strongmail.webserviceclient.samples.cxf;

import com.strongmail.webserviceclient.samples.cxf.generated.MailingService;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.AddSuppressionListRecordsRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.RemoveSuppressionListRecordsRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SuppressionList;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SuppressionListId;


public class SuppressionListSample
{
  private MailingService service;
  private SampleHelper helper;

  public SuppressionListSample(MailingService service)
  {
    this.service = service;
    helper = new SampleHelper(service);
  }


  /**
   * Create a Suppression list and add some data.
   *
   * @param suppressionListName the name of the suppression list
   * @return SuppressionListId
   * @throws Exception any exception
   */
  public SuppressionListId createSuppressionList(String suppressionListName) throws Exception
  {
    SuppressionList suppressionList = new SuppressionList();

    suppressionList.setDescription("SuppressionList from CXF");
    suppressionList.setIncludeByDefaultOnMailings(true);
    suppressionList.setName(suppressionListName);

    // Make call
    SuppressionListId suppressionListId = (SuppressionListId)helper.create(suppressionList);

    // Add some records
    addRecordsInline(suppressionListId);

    return suppressionListId;
  }


  /**
   * Add 2 records to suppression list using inline records.
   *
   * @param suppressionListId the SuppressionListId that records will be added to.
   * @throws Exception any exception
   */
  public void addRecordsInline(SuppressionListId suppressionListId) throws Exception
  {
    AddSuppressionListRecordsRequest request = new AddSuppressionListRecordsRequest();
    request.setSuppressionListId(suppressionListId);
    request.getAddressMatch().add("suppress.cxf.com");
    request.getAddressMatch().add("suppress@suppress.cxf.com");

    service.addRecords(request);
  }


  /**
   * Remove one record that was created in addRecordsInline() from the SuppressionList.
   *
   * @param suppressionListId the SuppressionListId whose record will be removed.
   * @return the number of record that was removed.
   * @throws Exception any exception
   */
  public int removeRecordsInline(SuppressionListId suppressionListId) throws Exception
  {
    RemoveSuppressionListRecordsRequest request = new RemoveSuppressionListRecordsRequest();
    request.setSuppressionListId(suppressionListId);
    request.getAddressMatch().add("suppress@suppress.cxf.com");

    return service.removeRecords(request).getRecordsRemoved();
  }


  public void getAndPrint(SuppressionListId suppressionListId) throws Exception
  {
    SuppressionList suppressionList = (SuppressionList)helper.get(suppressionListId);

    // Print SuppressionList to standard out
    System.out.println("\nSuppressionList:");
    System.out.println("SuppressionListId: " + suppressionList.getObjectId().getId());
    System.out.println("Name: " + suppressionList.getName());
    System.out.println("CampaignId: " + suppressionList.getCampaignId());
    System.out.println("CreatedTime: " + suppressionList.getCreatedTime());
    System.out.println("Description: " + suppressionList.getDescription());
    System.out.println("ModifiedTime: " + suppressionList.getModifiedTime());
    System.out.println("OrganizationId: " + suppressionList.getOrganizationId().getId());
    System.out.println("OwnerId: " + suppressionList.getOwnerId().getId());
    System.out.println("Total Records: " + suppressionList.getTotalRecords());
  }
}
