/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2014 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */


package com.strongmail.webserviceclient.samples.cxf;

import com.strongmail.webserviceclient.samples.cxf.generated.MailingService;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.FilterStringScalarOperator;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ListRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ObjectId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ScalarStringFilterCondition;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SystemAddress;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SystemAddressFilter;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SystemAddressId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SystemAddressType;

import java.util.List;


public class SystemAddressSample
{
  private MailingService service;
  private SampleHelper helper;

  public SystemAddressSample(MailingService service)
  {
    this.service = service;
    helper = new SampleHelper(service);
  }


  /**
   * List SystemAddress by type (bounce/from/reply) and email address. There is no filter by email
   * address, so we need to get all address for one type and filter them again.
   *
   * @param type         the SystemAddressType to filter on
   * @param emailAddress the email address to match
   * @return the SystemAddress that belongs to the type and matches the email address
   * @throws Exception any exception
   */
  public SystemAddress listByEmailAddress(SystemAddressType type,
                                          String emailAddress) throws Exception
  {

    // Create and set attribute for filter
    SystemAddressFilter filter = new SystemAddressFilter();

    // Create type condition
    ScalarStringFilterCondition typeCondition = new ScalarStringFilterCondition();
    typeCondition.setOperator(FilterStringScalarOperator.EQUAL);

    // Set type condition value
    if (type.equals(SystemAddressType.BOUNCE))
    {
      typeCondition.setValue(SystemAddressType.BOUNCE.value());
    }
    else if (type.equals(SystemAddressType.FROM))
    {
      typeCondition.setValue(SystemAddressType.FROM.value());
    }
    else if (type.equals(SystemAddressType.REPLY))
    {
      typeCondition.setValue(SystemAddressType.REPLY.value());
    }

    filter.setTypeCondition(typeCondition);

    // Create request and make call
    ListRequest request = new ListRequest();
    request.setFilter(filter);
    List<ObjectId> addressIds = service.list(request).getObjectId();

    // Go through all addresses to get the one that matches email address.
    for (ObjectId each : addressIds)
    {
      SystemAddress address = (SystemAddress)helper.get((SystemAddressId)each);
      if (address.getEmailAddress().equals(emailAddress))
      {
        return address;
      }
    }
    SystemAddress systemAddress = new SystemAddress();
    systemAddress.setEmailAddress(emailAddress);
    if (type.equals(SystemAddressType.BOUNCE))
    {
       systemAddress.setIsBounce(true);
    }
    else if (type.equals(SystemAddressType.FROM))
    {
       systemAddress.setIsFrom(true);
       systemAddress.setFromName("From Sample");
    }
    else if (type.equals(SystemAddressType.REPLY))
    {
       systemAddress.setIsReply(true);
    }

      SystemAddressId systemAddressId = (SystemAddressId)helper.create(systemAddress);


    return (SystemAddress)helper.get(systemAddressId);
  }
}
