﻿/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2014 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace ConsoleApplication1
{
    class MailingSample
    {
        private MailingService service;
        private SampleHelper helper;
        DataSourceSample dsSample;
        TargetSample targetSample;
        SeedListSample seedSample;
        SuppressionListSample suppressSample;
        TemplateSample templateSample;
        AttachmentSample attachmentSample;
        ContentBlockSample cbSample;
        SystemAddressSample addressSample;

        public MailingSample(MailingService service)
        {
            this.service = service;
            this.helper = new SampleHelper(service);
            dsSample = new DataSourceSample(service);
            targetSample = new TargetSample(service);
            seedSample = new SeedListSample(service);
            suppressSample = new SuppressionListSample(service);
            attachmentSample = new AttachmentSample(service);
            cbSample = new ContentBlockSample(service);
            addressSample = new SystemAddressSample(service);
            templateSample = new TemplateSample(service);
        }

        /* Create a one mailing with no excluded target. Use only one of instead of list of
         * target, seed list, suppresion list, attachment and content blocks.
         *
         * mailingName is the name of the mailing.
         *
         * return the ID of created standard mailing.
         */
        public StandardMailingId createOneTimeStandardMailing(String mailingName)
        {
            StandardMailing mailing = new StandardMailing();
            setMailingCommonFields(mailing, mailingName);

            mailing.bounceAddressId = (SystemAddressId)addressSample.listByOrCreateEmailAddress("BOUNCE", "bounce@sample.com").objectId;
            mailing.fromAddressId = (SystemAddressId)addressSample.listByOrCreateEmailAddress("FROM", "from@sample.com").objectId;
            mailing.fromName = "From Name";
            mailing.replyAddressId = (SystemAddressId)addressSample.listByOrCreateEmailAddress("REPLY", "reply@sample.com").objectId; ;
            mailing.subject = "mailing subject";
            mailing.header = new String[] { "mailing header" };

            TemplateId templateId = templateSample.createTemplate("Template_" + mailingName);
            mailing.templateId = templateId;
            mailing.description = "StandardMailing from dotnet";
            mailing.eliminateDuplicates = true;
            mailing.type = "ONE_TIME";
            StandardMailingNotification mailingNotification = new StandardMailingNotification();

            mailingNotification.recipientAddresses = new String[] { "test123@strongmail-test.com", "test2@strongmail-test.com" };
            mailingNotification.notifyStateChange = true;
            mailingNotification.notifyTargetingCompletion = false;

            mailing.ccAddress = "cc@strongmail-test.com";
            mailing.bccAddress = "bcc@strongmail-test.com";

            mailing.notificationOption = mailingNotification;

            SchedulableMailingSchedule schedule = new SchedulableMailingSchedule();
            schedule.startDateTime = new DateTime(2020, 01, 01);

            mailing.schedule = schedule;

            // Set attachment
            AttachmentId attachmentId = attachmentSample.createAttachment("Attachment_" + mailingName);
            mailing.attachmentId = new AttachmentId[] { attachmentId };

            // Set ContentBlock
            ContentBlockId contentBlockId = cbSample.createContentBlock("ContentBlock_" + mailingName);            
            mailing.contentBlockId = new ContentBlockId[] {contentBlockId} ;

            // Set Target
            TargetId targetId = targetSample.createTarget("Target_" + mailingName);
            mailing.includedTargetId = new TargetId[] {targetId};

            // Set Seed
            SeedListId seedListId = seedSample.createSeedList("Seed_" + mailingName);
            mailing.seedListId = new SeedListId[] {seedListId};

            // Set Suppression
            SuppressionListId suppressionListId = suppressSample.createSuppressionList("Suppress_" + mailingName);
            mailing.suppressionListId = new SuppressionListId[] {suppressionListId};

            return (StandardMailingId) helper.create(mailing);
        }

        /* Create a recurring mailing with no excluded target. Use only one of instead of list of
         * target, seed list, suppresion list, attachment and content blocks.
         * 
         * mailingName is the name of the mailing.
         * 
         * return the ID of created standard mailing.
         */ 
        public StandardMailingId createStandardMailing(String mailingName)
        {
            StandardMailing mailing = new StandardMailing();
            // set common fields
            setMailingCommonFields(mailing, mailingName);

            // We are appending mailing name to ensure the uniqness of template name
            TemplateId templateId = templateSample.createTemplate("template_" + mailingName);
            mailing.templateId = templateId;

            mailing.bounceAddressId = (SystemAddressId)addressSample.listByOrCreateEmailAddress("BOUNCE", "bounce@sample.com").objectId;
            mailing.fromAddressId = (SystemAddressId)addressSample.listByOrCreateEmailAddress("FROM", "from@sample.com").objectId;
            mailing.fromName = "From Name";
            mailing.replyAddressId = (SystemAddressId)addressSample.listByOrCreateEmailAddress("REPLY", "reply@sample.com").objectId; ;
            mailing.subject = "mailing subject";
            mailing.format = new String[] { "HTML" };
            mailing.header = new String[] { "mailing header" };

            mailing.description = "StandardMailing from dotnet";
            mailing.eliminateDuplicates = true;
            mailing.type = "RECURRING";
            StandardMailingNotification mailingNotification = new StandardMailingNotification();

            mailingNotification.recipientAddresses = new String[] { "test123@strongmail-test.com", "test2@strongmail-test.com" };
            mailingNotification.notifyStateChange = true;
            mailingNotification.notifyTargetingCompletion = false;

            mailing.ccAddress = "cc@strongmail-test.com";
            mailing.bccAddress = "bcc@strongmail-test.com";

            mailing.notificationOption = mailingNotification;
            
            // Set schedule: daily recurrence with every two days
            SchedulableMailingScheduleRecurrenceDailyRecurrence daily = new SchedulableMailingScheduleRecurrenceDailyRecurrence();
            daily.Item = 2; // every two days

            SchedulableMailingScheduleRecurrence recurrence = new SchedulableMailingScheduleRecurrence();
            recurrence.Item1 = daily;
            recurrence.Item = new DateTime(2020, 10, 10);

            SchedulableMailingSchedule schedule = new SchedulableMailingSchedule();
            schedule.startDateTime = new DateTime(2020, 01, 01);
            
            schedule.recurrence = recurrence;
            mailing.schedule = schedule;

            // Set attachment
            AttachmentId attachmentId = attachmentSample.createAttachment("Attachment_" + mailingName);
            mailing.attachmentId = new AttachmentId[] { attachmentId };

            // Set ContentBlock
            ContentBlockId contentBlockId = cbSample.createContentBlock("ContentBlock_" + mailingName);
            mailing.contentBlockId = new ContentBlockId[] { contentBlockId };

            // Set Target
            TargetId targetId = targetSample.createTarget("Target_" + mailingName);
            mailing.includedTargetId = new TargetId[] { targetId };

            // Set Seed
            SeedListId seedListId = seedSample.createSeedList("Seed_" + mailingName);
            mailing.seedListId = new SeedListId[] { seedListId };

            // Set Suppression
            SuppressionListId suppressionListId = suppressSample.createSuppressionList("Suppress_" + mailingName);
            mailing.suppressionListId = new SuppressionListId[] { suppressionListId };
            
            return (StandardMailingId)helper.create(mailing);

        }


        /* Create a transactional mailing.
         * 
         * mailingName is the name of the mailing.
         * 
         * return the ID of created transactional mailing.
         */
        public TransactionalMailingId createTransactionalMailing(string mailingName)
        {
            TransactionalMailing txMailing = new TransactionalMailing();
            setMailingCommonFields(txMailing, mailingName);
            TemplateId templateId = templateSample.createTemplate("Template_" + mailingName);
            txMailing.templateId = templateId;

            txMailing.bounceAddressId = (SystemAddressId)addressSample.listByOrCreateEmailAddress("BOUNCE", "bounce@sample.com").objectId;
            txMailing.fromAddressId = (SystemAddressId)addressSample.listByOrCreateEmailAddress("FROM", "from@sample.com").objectId;
            txMailing.fromName = "From Name";
            txMailing.replyAddressId = (SystemAddressId)addressSample.listByOrCreateEmailAddress("REPLY", "reply@sample.com").objectId; ;
            txMailing.subject = "mailing subject";
            txMailing.header = new String[] { "mailing header" };
            txMailing.format = new String[] { "HTML" };

            txMailing.description = "TransactionalMailing from dotnet";
            txMailing.type = "TRANSACTIONAL";

            TargetId targetId = targetSample.createTarget("Target_"+mailingName);
            txMailing.targetId = targetId;

            return (TransactionalMailingId) helper.create(txMailing);
        }

        /* Create a one mailing with no excluded target. Use only one of instead of list of
         * target, seed list, suppresion list, attachment and content blocks.
         *
         * mailingName is the name of the mailing.
         *
         * return the ID of created standard mailing.
         */
        public ChannelStandardMailingId createChannelStandardMailing(String mailingName)
        {
            ChannelStandardMailing mailing = new ChannelStandardMailing();
            setMailingCommonFields(mailing, mailingName);

            // Get the ChannelId By Name
            ChannelFilter channelFilter = new ChannelFilter();
            ListRequest listRequest = new ListRequest();
            ScalarStringFilterCondition nameCondition = new ScalarStringFilterCondition();
            nameCondition.@operator = FilterStringScalarOperator.EQUAL;
            nameCondition.value = "Push Notification";
            channelFilter.nameCondition = nameCondition;
            listRequest.filter = channelFilter;
            ListResponse listResponse = service.list(new list(listRequest)).listResponse;

            if (listResponse == null || listResponse.objectId.Length < 1)
            {
                Console.WriteLine("Either Message Queue with specified name is not exist OR Message Queue license is not present");
                return null;
            }

            // Set Channel Id
            ChannelId channelId = (ChannelId)listResponse.objectId[0];
            mailing.channelId = channelId;

            ChannelTemplateId templateId = templateSample.createChannelTemplate("ChannelTemplate_" + mailingName);
            mailing.templateId = templateId;
            mailing.description = "ChannelStandardMailing from dotnet";
            mailing.eliminateDuplicates = true;
            mailing.type = "ONE_TIME";
            StandardMailingNotification mailingNotification = new StandardMailingNotification();

            mailingNotification.recipientAddresses = new String[] { "test123@strongmail-test.com", "test2@strongmail-test.com" };
            mailingNotification.notifyStateChange = true;
            mailingNotification.notifyTargetingCompletion = false;

            mailing.notificationOption = mailingNotification;

            SchedulableMailingSchedule schedule = new SchedulableMailingSchedule();
            schedule.startDateTime = new DateTime(2020, 01, 01);

            mailing.schedule = schedule;

            // Set attachment
            AttachmentId attachmentId = attachmentSample.createAttachment("Attachment_" + mailingName);
            mailing.attachmentId = new AttachmentId[] { attachmentId };

            // Set ContentBlock
            ContentBlockId contentBlockId = cbSample.createContentBlock("ContentBlock_" + mailingName);
            mailing.contentBlockId = new ContentBlockId[] { contentBlockId };

            // Set Target
            TargetId targetId = targetSample.createChannelTarget("Target_" + mailingName);
            IncludedTargetField includeTargetField = new IncludedTargetField();
            includeTargetField.targetId = targetId;
            // SuppressionColumnName is mandatory when suppression list is included / remove duplicates is enabled/ both
            includeTargetField.suppressionColumnName = "id";

            // SuppressionColumnName is mandatory when seed list is included
            includeTargetField.seedColumnName = "id";
            mailing.includedTarget = new IncludedTargetField[] { includeTargetField };

            // Set Seed
            SeedListId seedListId = seedSample.createChannelSeedList("channel_seed_" + mailingName);
            mailing.seedListId = new SeedListId[] { seedListId };

            // Set Suppression
            SuppressionListId suppressionListId = suppressSample.createChannelSuppressionList("channel_suppress_" + mailingName);
            mailing.suppressionListId = new SuppressionListId[] { suppressionListId };

            // Add channel headers
            ChannelHeader header1 = new ChannelHeader();
            header1.headerName = "CustomUserHeader1";
            header1.headerValue = "CustomUserHeaderValue1";

            ChannelHeader header2 = new ChannelHeader();
            header2.headerName = "CustomUserHeader2";
            header2.headerValue = "CustomUserHeaderValue2";

            mailing.channelHeader = new ChannelHeader[] { header1, header2 };

            // Add profile attributes - 
            //  value to be specified for it should be equals to column names of the included targets
            mailing.channelProfileAttribute = new String[] { "id", "email_address" };

            return (ChannelStandardMailingId)helper.create(mailing);
        }

        /* Create a Channel transactional mailing.
         * 
         * mailingName is the name of the mailing.
         * 
         * return the ID of created transactional mailing.
         */
        public ChannelTransactionalMailingId createChannelTransactionalMailing(string mailingName)
        {
            ChannelTransactionalMailing txMailing = new ChannelTransactionalMailing();
            setMailingCommonFields(txMailing, mailingName);
            ChannelTemplateId templateId = templateSample.createChannelTemplate("Channel_Template_" + mailingName);
            txMailing.templateId = templateId;
            // Get the ChannelId By Name
            ChannelFilter channelFilter = new ChannelFilter();
            ListRequest listRequest = new ListRequest();
            ScalarStringFilterCondition nameCondition = new ScalarStringFilterCondition();
            nameCondition.@operator = FilterStringScalarOperator.EQUAL;
            nameCondition.value = "Push Notification";
            channelFilter.nameCondition = nameCondition;
            listRequest.filter = channelFilter;
            ListResponse listResponse = service.list(new list(listRequest)).listResponse;

            if (listResponse == null || listResponse.objectId.Length < 1)
            {
                Console.WriteLine("Either Message Queue with specified name is not exist OR Message Queue license is not present");
                return null;
            }

            // Set Channel Id
            ChannelId channelId = (ChannelId)listResponse.objectId[0];
            txMailing.channelId = channelId;

            txMailing.description = "TransactionalMailing from dotnet";
            txMailing.type = "TRANSACTIONAL";
            txMailing.recipientFieldName = "id";

            TargetId targetId = targetSample.createChannelTarget("Target_" + mailingName);
            txMailing.targetId = targetId;

            // Add channel headers
            ChannelHeader header1 = new ChannelHeader();
            header1.headerName = "CustomUserHeader1";
            header1.headerValue = "CustomUserHeaderValue1";

            ChannelHeader header2 = new ChannelHeader();
            header2.headerName = "CustomUserHeader2";
            header2.headerValue = "CustomUserHeaderValue2";

            txMailing.channelHeader = new ChannelHeader[] { header1, header2 };

            return (ChannelTransactionalMailingId)helper.create(txMailing);
        }


        /* Set mailing properties common to both standard and transactional mailings.
         * 
         */
        private void setMailingCommonFields(Mailing mailing,String mailingName)
        {
            mailing.bodyEncoding = "BASE_64";
            
            mailing.fieldDelimiter = "::";            
            mailing.headerEncoding = "EIGHT_BIT";
            mailing.isApproved = true;
            mailing.isCompliant = true;
            mailing.name = mailingName;

            // Use outputBodyCharSet(Token)
            mailing.Item = CharSet.GB2312;

            // Use outputHeaderCharSet(Token)
            mailing.Item1 = CharSet.GB2312;
            mailing.priority = "NORMAL";            
            mailing.rowDelimiter = "\n";

            TagValueId tagValueId = addTagToMailing();
            if(tagValueId != null)
            {
                mailing.tagValueId = new TagValueId[] { tagValueId };
            }
            // Set LETI
            mailing.linkExpiryInterval = "fortyeight_hours";
        }


        /* Get and print out a mailing.
         *
         * mailingId is the ID of the mailing to be printed.
         */
        public void getAndPrint(MailingId mailingId)
        {
            Mailing mailing = (Mailing)helper.get(mailingId);

            if (mailingId is StandardMailingId)
            {
                Console.WriteLine("\nStandardMailing");
            }
            else if (mailingId is TransactionalMailingId)
            {
                Console.WriteLine("\nTransactionalMailing");
            }

            Console.WriteLine("\tID: " + mailing.objectId.id);
            Console.WriteLine("\tName: " + mailing.name);
            Console.WriteLine("\tCampaign: " + mailing.campaignId);
            Console.WriteLine("\tCreatedTime: " + mailing.createdTime);
            Console.WriteLine("\tDescription: " + mailing.description);
            Console.WriteLine("\tModifiedTime: " + mailing.modifiedTime);
            Console.WriteLine("\tOrganization: " + mailing.organizationId.id);
            Console.WriteLine("\tOwenr: " + mailing.ownerId.id);
            Console.WriteLine("\tFieldDelimiter: " + mailing.fieldDelimiter);
            Console.WriteLine("\tRowDelimiter: " + mailing.rowDelimiter);
            Console.WriteLine("\tMessageFormat: " + mailing.format[0]);
            Console.WriteLine("\tTemppate: " + mailing.templateId.id);
            Console.WriteLine("\tBodyEncoding: " + mailing.bodyEncoding);
            Console.WriteLine("\tHeaderEncoding: " + mailing.headerEncoding);
            Console.WriteLine("\tPriority: " + mailing.priority);
            Console.WriteLine("\tServerErrorCode: " + mailing.serverErrorCode);
            Console.WriteLine("\tSererErrorMessage: " + mailing.serverErrorMessage);
            Console.WriteLine("\tStatus: " + mailing.status);
            Console.WriteLine("\tType: " + mailing.type);
            Console.WriteLine("\tLETI : " + mailing.linkExpiryInterval);

            if (mailingId is StandardMailingId)
            {
                StandardMailing m = (StandardMailing) mailing;
                Console.WriteLine("\tIncludedTargets Size: " + m.includedTargetId.Length);
                Console.WriteLine("\tSeedLists Size: " + m.seedListId.Length);
                Console.WriteLine("\tSuppresionLists Size: " + m.suppressionListId.Length);
                Console.WriteLine("\tAttachment Size: " + m.attachmentId.Length);
                Console.WriteLine("\tContentBlock Size: " + m.contentBlockId.Length);
                Console.WriteLine("\tFromName: " + m.fromName);
                Console.WriteLine("\tHeader Size: " + m.header.Length);
                Console.WriteLine("\tBounce: " + m.bounceAddressId.id);
                Console.WriteLine("\tFrom: " + m.fromAddressId.id);
                Console.WriteLine("\tReply: " + m.replyAddressId.id);
                Console.WriteLine("\tMailngClass: " + m.mailingClassId.id);
                Console.WriteLine("\tSubject: " + m.subject);
            }
            else if (mailingId is TransactionalMailingId)
            {
                TransactionalMailing m = (TransactionalMailing)mailing;
                Console.WriteLine("\tFormatField: " + m.formatFieldName);
                Console.WriteLine("\tMailingConfig: " + m.mailingConfig);
                Console.WriteLine("\tRecordStructure: " + m.recordStructure);
                Console.WriteLine("\tSenderNumber: " + m.senderNumber);
                Console.WriteLine("\tTarget: " + m.targetId.id);
                Console.WriteLine("\tFromName: " + m.fromName);
                Console.WriteLine("\tHeader Size: " + m.header.Length);
                Console.WriteLine("\tBounce: " + m.bounceAddressId.id);
                Console.WriteLine("\tFrom: " + m.fromAddressId.id);
                Console.WriteLine("\tReply: " + m.replyAddressId.id);
                Console.WriteLine("\tMailngClass: " + m.mailingClassId.id);
                Console.WriteLine("\tSubject: " + m.subject);
            }
        }


        /* Launch a standard mailing
 */
        public bool launchMailing(StandardMailingId mailingId)
        {
            LaunchRequest request = new LaunchRequest();
            request.mailingId = mailingId;

            return service.launch(new launch(request)).launchResponse.success;
        }

        /* Schedule a standard mailing
         */
        public bool scheduleMailing(StandardMailingId mailingId)
        {
            ScheduleRequest request = new ScheduleRequest();
            request.mailingId = mailingId;

            return service.schedule(new schedule(request)).scheduleResponse.success;
        }

        /* Load a transactional mailing
         */

        public bool loadMailing(TransactionalMailingId txMailingId)
        {
            LoadRequest loadRequest = new LoadRequest();
            loadRequest.mailingId = txMailingId;

            return service.load(new load(loadRequest)).loadResponse.success;
        }

        /**
   * Fetch the handle for a tx mailing, that is to be used in the txnSend API.   
   *
   * @param mailingId
   * @return mailing handle
   * @throws Exception
   */
        public String getTxnMailingHandle(TransactionalMailingId mailingId)
        {
            // Create request and set mailingId
            GetTxnMailingHandleRequest request = new GetTxnMailingHandleRequest();
            request.Item = mailingId;
            return service.getTxnMailingHandle(new getTxnMailingHandle(request)).getTxnMailingHandleResponse.handle;
        }

        public bool txnSend(String mailingHandle)
        {
            // Create request and set mailingId
            TxnSendRequest request = new TxnSendRequest();
            request.handle = mailingHandle;

            // Create record based on the target created in TargetSample   

            NameValuePair pair1 = new NameValuePair();
            pair1.name = "ID";
            pair1.value = "10";

            NameValuePair pair2 = new NameValuePair();
            pair2.name = "EMAIL_ADDRESS";
            pair2.value = "sample1@cxf.sample.com";

            NameValuePair pair3 = new NameValuePair();
            pair3.name = "NAME";
            pair3.value = "First Last";

            NameValuePair[] record1 = new NameValuePair[] { pair1, pair2, pair3 };
            SendRecord sendRecord = new SendRecord();
            sendRecord.field = record1;


            request.sendRecord = new SendRecord[] { sendRecord };

            bool success = false;
            try
            {
                success = service.txnSend(new txnSend(request)).txnSendResponse.success;
                Console.WriteLine("Txsend Request was " + (success ? "successful" : "unsuccessful"));
            }
            catch (Exception e)
            {
                Console.WriteLine("Encountered a BadHandleFault. " + e.Message);
                Console.WriteLine("This indicates that the handle being used to attempt the txnSend call is incorrect.");
                Console.WriteLine("Please ensure that you send the correct handle using the getTxnMailingHandle() API.");
            }
            return success;
        }

        /* Close a transactional mailing
         */
        public bool closeMailing(TransactionalMailingId txMailingId)
        {
            CloseRequest closeRequest = new CloseRequest();
            closeRequest.mailingId = txMailingId;

            return service.close(new close(closeRequest)).closeResponse.success;
        }


        /* 
         * Cancel a mailing
         */
        public bool cancelMailing(MailingId mailingId)
        {
            CancelRequest request = new CancelRequest();
            request.mailingId = mailingId;

            return service.cancel(new cancel(request)).cancelResponse.success;
        }


        /* 
         * Pause a mailing
         */
        public bool pauseMailing(MailingId mailingId)
        {
            PauseRequest request = new PauseRequest();
            request.mailingId = mailingId;

            return service.pause(new pause(request)).pauseResponse.success;
        }


        /*
         * Resume a mailing
         */

        public bool resumeMailing(MailingId mailingId)
        {
            ResumeRequest request = new ResumeRequest();
            request.mailingId = mailingId;

            return service.resume(new resume(request)).resumeResponse.success;
        }

        /* List mailings by attachmentName: this should return one mailnig if attachmentName is matched.
         * 
         * attachmentName is the mailing attachmentName to be listed.
         */
        public void listByName(String name)
        {
            // Create filter and set default values
            MailingFilter filter = new MailingFilter();
            helper.setFilter(filter);

            // Create attachmentName condition
            ScalarStringFilterCondition nameCondition = new ScalarStringFilterCondition();
            nameCondition.@operator = FilterStringScalarOperator.EQUAL;
            nameCondition.value = name;

            // Create filter request and set filter to request
            ListRequest request = new ListRequest();
            request.filter = filter;

            // Make call and print results
            printListResponse(service.list(new list(request)).listResponse);
        }


        /* List mailings in an organization: this can be done by using empty filter.
         * The organization is the one from header.
         */
        public void listByOrganization()
        {
            // Create empty filter: no conditions, and set default attributes
            MailingFilter filter = new MailingFilter();
            helper.setFilter(filter);

            // Create list request and set the filter to request
            ListRequest request = new ListRequest();
            request.filter = filter;

            // Make call and print results
            printListResponse(service.list(new list(request)).listResponse);
        }


        /* List mailings by mailing type and status
         */
        public void listByTypeAndStatus()
        {
            // Create a mailing filer and set default attributes
            MailingFilter filter = new MailingFilter();
            helper.setFilter(filter);

            // Create status condition: status in (ACTIVE, EDITING, COMPLETED)
            ArrayStringFilterCondition statusCondition = new ArrayStringFilterCondition();
            statusCondition.@operator = FilterArrayOperator.IN;
            statusCondition.value = new string[] {
                "ACTIVE", 
                "EDITING", 
                "COMPLETED"};
            filter.statusCondition = statusCondition;

            // Create type condition: type in (RECURRING, TRANSACTIONAL)
            ArrayStringFilterCondition typeCondition = new ArrayStringFilterCondition();
            typeCondition.@operator = FilterArrayOperator.IN;
            typeCondition.value = new string[] { "RECURRING".ToString(), "TRANSACTIONAL".ToString() };
            filter.typeCondition = typeCondition;

            // Create request and set filter to request
            ListRequest request = new ListRequest();
            request.filter = filter;

            printListResponse(service.list(new list(request)).listResponse);
        }

        private TagValueId addTagToMailing()
        {
            TagFilter filter = new TagFilter();
            filter.isAscendingSpecified = false;
            filter.maxRecordsPerPageSpecified = false;
            filter.pageNumberSpecified = false;
            filter.recordsPerPageSpecified = false;

            ListRequest lr = new ListRequest();
            lr.filter = filter;

            ListResponse response = service.list(new list(lr)).listResponse;
            TagValueId tagValueId = null;

            if (response != null && response.objectId.Length > 0)
            {
                TagId tagId = (TagId)response.objectId[0];
                Tag tag = (Tag) helper.get(tagId);
                tagValueId =  tag.tagValue[0].tagValueId;
            }
            return tagValueId;
        }


        /* Print out list response: print ID and attachmentName
         */
        private void printListResponse(ListResponse response)
        {
            Console.WriteLine("\nList Response");
            foreach (ObjectId each in response.objectId)
            {
                Mailing mailing = (Mailing)helper.get(each);
                Console.WriteLine("\t" + each.id + " " + mailing.name);
            }
        } 
    }
}
