﻿/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2014 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */


using System;
using System.Linq;
using System.Text;
using System.Collections.Generic;
using System.ServiceModel;
using System.Xml;
using System.IO;
using System.Threading;
using System.Security.Cryptography.X509Certificates;


namespace ConsoleApplication1
{

    class SampleProgram
    {
        public static string IDS_DATA_FILE = "..\\..\\..\\..\\..\\..\\data\\Shared_DataSource_Records.txt";
        public static string ATTACHMENT_FILE = "..\\..\\..\\..\\..\\..\\data\\Shared_Attachment.gif";
        public static string CONTENT_BLOCK_FILE = "..\\..\\..\\..\\..\\..\\data\\Shared_ContentBlock.txt";

        // Generated MailingService stub
        MailingService svc;
        SampleHelper helper;

        // Instance for each sample: one per module
        DataSourceSample dsSample;
        TargetSample targetSample;
        SeedListSample seedSample;
        SuppressionListSample suppressSample;
        TemplateSample templateSample;
        AttachmentSample attachmentSample;
        ContentBlockSample cbSample;
        RuleSample ruleSample;
        MailingSample mailingSample;


        public SampleProgram(MailingService svc)
        {
            this.svc = svc;
            this.helper = new SampleHelper(svc);

            dsSample = new DataSourceSample(svc);
            targetSample = new TargetSample(svc);
            seedSample = new SeedListSample(svc);
            suppressSample = new SuppressionListSample(svc);
            attachmentSample = new AttachmentSample(svc);
            cbSample = new ContentBlockSample(svc);
            ruleSample = new RuleSample(svc);

            // You need to create these data from MessageStudio UI
            SystemAddressSample addressSample = new SystemAddressSample(svc);
            templateSample = new TemplateSample(svc);
            mailingSample = new MailingSample(svc);             
        }


        /* Create the following objects:
         * Data Sources: IDS, EDS, Target, SeedList, SuppresionList;
         * Content Library: Attachment, ContentBlock, Template, Rule;
         * Mailings: StandardMaiiling, TransactionalMailing.
         *
         * sequence is sequence number to differentiate the attachmentName for each run.
         * return an list of ObjectIds that was created.
         */
        public List<ObjectId> createAllData(String sequence)
        {
            List<ObjectId> objectIds = new List<ObjectId>();  // Use a list to hold all object IDs

            objectIds.Add(dsSample.createIDS("ids_dotnet_" + sequence));

            // User need to use his own external data source.
            //objectIds.Add(dsSample.createEDS("eds_dotnet_" + sequence));

            // Wait for data source finish imporing records
            helper.waitForIdle((InternalDataSourceId) getObjectId(objectIds, typeof(InternalDataSourceId)));

            objectIds.Add(targetSample.createTarget("target_dotnet_" + sequence));

            objectIds.Add(seedSample.createSeedList("seed_dotnet_" + sequence));

            objectIds.Add(suppressSample.createSuppressionList("suppress_donet_" + sequence));

            objectIds.Add(attachmentSample.createAttachment("attachment_dotnet_" + sequence));

            objectIds.Add(cbSample.createContentBlock("cb_dotnet_" + sequence));

            objectIds.Add(ruleSample.createRule("rule_dotnet_" + sequence, (TargetId) getObjectId(objectIds, typeof(TargetId))));
            ObjectId obId = (AttachmentId)getObjectId(objectIds, typeof(AttachmentId));
            ObjectId obId2 = (ContentBlockId)getObjectId(objectIds, typeof(ContentBlockId));

            Console.WriteLine(obId.id);
            Console.WriteLine(obId2.id);

            objectIds.Add(templateSample.createTemplate("template_dotnet_" + sequence));

            objectIds.Add(mailingSample.createStandardMailing("mailing_dotnet_" + sequence));

            objectIds.Add(mailingSample.createTransactionalMailing("txMailing_dotnet_" + sequence));

            // Create Non Email Channel Assets
            seedSample.createChannelSeedList("channel_seed_dotnet_"+sequence);

            suppressSample.createChannelSuppressionList("channel_suppress_dotnet_"+sequence);

            templateSample.createChannelTemplate("channel_template_dotnet_"+sequence);

            mailingSample.createChannelStandardMailing("channel_stdmailing_dotnet_"+sequence);

            mailingSample.createChannelTransactionalMailing("channel_txmailing_dotnet"+sequence);

            return objectIds;
        }


        /* Get and print out the object contnet.
         *
         * objectIds is the list of objectId whose content will be printed.
         */
        public void getAndPrintAllData(List<ObjectId> objectIds)
        {
            foreach (ObjectId objectId in objectIds)
            {
                if (objectId is DataSourceId)
                {
                    dsSample.getAndPrint((DataSourceId) objectId);
                }
                else if (objectId is TargetId)
                {
                    targetSample.getAndPrint((TargetId) objectId);
                }
                else if (objectId is SeedListId)
                {
                    seedSample.getAndPrint((SeedListId)objectId);
                }
                else if (objectId is SuppressionListId)
                {
                    suppressSample.getAndPrint((SuppressionListId)objectId);
                }
                else if (objectId is AttachmentId)
                {
                    attachmentSample.getAndPrint((AttachmentId)objectId);
                }
                else if (objectId is ContentBlockId)
                {
                    cbSample.getAndPrint((ContentBlockId)objectId);
                }
                else if (objectId is TemplateId)
                {
                    templateSample.getAndPrint((TemplateId)objectId);
                }
                else if (objectId is RuleId)
                {
                    ruleSample.getAndPrint((RuleId)objectId);
                }
                else if (objectId is MailingId)
                {
                    mailingSample.getAndPrint((MailingId)objectId);
                }
            }
        }


        /* Get an ObjectId from a list based on its type.
         *
         * objectIds is a list of distinct objectIds, one for each object type.
         * type is the type whose ID will be returned.
         * return the ObjectId that matches the type.
         */
        public ObjectId getObjectId(List<ObjectId> objectIds, Type type)
        {
            foreach (ObjectId each in objectIds)
            {
                if (each.GetType() == type)
                {
                    return each;
                }
            }

            return null;
        }


        /* Create a MailingService client with proper headers.
         */
        private static MailingService createService()
        {
            // Create service
            MailingServiceClient svc = new MailingServiceClient();
            Console.WriteLine("End Point Address "+ svc.Endpoint.Address.Uri.AbsolutePath);

            //// Ignore Certificate by trusting all certificate
            System.Net.ServicePointManager.ServerCertificateValidationCallback +=
                delegate(object sender, System.Security.Cryptography.X509Certificates.X509Certificate certificate,
                        System.Security.Cryptography.X509Certificates.X509Chain chain,
                        System.Net.Security.SslPolicyErrors sslPolicyErrors)
                {
                    return true; // **** Always accept
                };

            return svc;
        }



        public static void Main(string[] args)
        {
            Console.WriteLine("Starting....");

            MailingService svc = createService();
            SampleProgram samples = new SampleProgram(svc);
            SampleHelper helper = new SampleHelper(svc);

            // Use timestamp to avoid duplicate names
            string sequence = DateTime.Now.ToString("yyyy_MMMM_dd_HH_mm_ss");

            // Create all objects: one for each type
            List<ObjectId> objectIds = samples.createAllData(sequence);
            samples.getAndPrintAllData(objectIds);

            // Get the data source that we will work on
            InternalDataSourceId dataSourceId = (InternalDataSourceId)samples.getObjectId(objectIds, typeof(InternalDataSourceId));
            InternalDataSource dataSource = (InternalDataSource) helper.get(dataSourceId);

            // Add data to IDS using file
            FileStream dataFile = File.OpenRead(IDS_DATA_FILE);
            samples.dsSample.addRecordsUsingFile(dataSourceId, dataFile);
            dataFile.Close();

            // Upsert record
            samples.dsSample.upsertRecord(dataSourceId);

            // Get data source records
            samples.dsSample.getRecords(dataSourceId);


            // Export data source records to a file
            FileStream dataSourceExportFile = new FileStream("./data_source_export.txt", FileMode.Create);
            samples.dsSample.exportRecords(dataSourceId, dataSourceExportFile);
            dataSourceExportFile.Close();

            // Update the data source with new attachmentName
            samples.dsSample.updateDataSourceName(dataSourceId, dataSource.name + "_updated");
            dataSource = (InternalDataSource) helper.get(dataSourceId);
            Console.WriteLine("\nNew Data Source Name: " + dataSource.name);

            // Read data source stats
            samples.dsSample.getAndPrintStatistics(dataSourceId);
            samples.dsSample.dedupRecords(dataSourceId);


            // Copy the data source
            InternalDataSourceId copiedId = samples.dsSample.copyDataSource(dataSourceId, dataSource.name + "_copied");
            Console.WriteLine("\nCopied IDS: " + copiedId.id);

            // Remove data source records and print out new statistics
            int removedDS = samples.dsSample.removeRecordsInline(dataSourceId);
            samples.dsSample.getAndPrintStatistics(dataSourceId);

            
            // list DataSource by name
            InternalDataSourceId dataSource2 = samples.dsSample.listInternalDataSourcesByName(dataSource.name);
            Console.WriteLine("Get DS after list by name: " + dataSource2.id);

            // This section was commented out since user need to use his own external data source.
            /*
            ExternalDataSourceId edsId = (ExternalDataSourceId)samples.getObjectId(objectIds, typeof(ExternalDataSourceId));
            bool refreshed = samples.dsSample.refreshRecords(edsId);
            Console.WriteLine("EDS refreshed: " + refreshed);

            bool cancelled = samples.dsSample.cancelRefreshRecords(edsId);
            Console.WriteLine("EDS cancelled: " + cancelled);
            */

            // Global Undeliverable demo
            GlobalUndeliverableSample sample = new GlobalUndeliverableSample(svc);
            sample.getByRecipientAddress("sample", "email", "1");
            sample.getByFailureCode("2002", "email", "1");
            sample.reactivateGlobalUndeliverable("sample@test.com", "email", "1");
            sample.addGlobalUndeliverable();

            // Import new content to attachment using IDS record file
            AttachmentId attachmentId = (AttachmentId) samples.getObjectId(objectIds, typeof(AttachmentId));
            FileStream attachmentFile = new FileStream(IDS_DATA_FILE, FileMode.Open);
            samples.attachmentSample.importContent(attachmentId, attachmentFile);
            attachmentFile.Close();
            samples.attachmentSample.getAndPrint(attachmentId);

            // Remove records from seed list
            SeedListId seedListId = (SeedListId) samples.getObjectId(objectIds, typeof(SeedListId));
            int removedSeed = samples.seedSample.removeRecordsInline(seedListId);
            samples.seedSample.getAndPrint(seedListId);

            // Remove records from suppression list
            SuppressionListId suppressionListId = (SuppressionListId) samples.getObjectId(objectIds, typeof(SuppressionListId));
            int removedSuppres = samples.suppressSample.removeRecordsInline(suppressionListId);
            samples.suppressSample.getAndPrint(suppressionListId);


            // Get the mailings we will work with
            StandardMailingId stdMailingId = (StandardMailingId) samples.getObjectId(objectIds, typeof(StandardMailingId));
            StandardMailing stdMailing = (StandardMailing) helper.get(stdMailingId);
            TransactionalMailingId txMailingId = (TransactionalMailingId) samples.getObjectId(objectIds, typeof(TransactionalMailingId));
            TransactionalMailing txMailing = (TransactionalMailing) helper.get(txMailingId);

            // List mailings by attachmentName
            samples.mailingSample.listByName(stdMailing.name);

            // List mailings in an organization
            samples.mailingSample.listByOrganization();

            // list mailings by type and status
            samples.mailingSample.listByTypeAndStatus();


            TemplateId templateId = new TemplateId();
            templateId = (TemplateId) samples.getObjectId(objectIds, typeof(TemplateId));

            // This section was commented out since user need to create his own media server and zip file.
            /*
            MediaServerId mediaId = new MediaServerId();
            mediaId.id = "100";

            FileStream zipfile = new FileStream("ZIP_FILE", FileMode.Open);
            bool imported = samples.templateSample.importMessagePart(templateId, mediaId, zipfile, "" );
            Console.WriteLine("media imported: " + imported);
            */
            bool validXsl = samples.templateSample.validateXsl(templateId);
            Console.WriteLine("valid xsl: " + validXsl);

            bool tested = samples.templateSample.testTemplate(templateId);
            Console.WriteLine("tested: " + tested);


            // Schedule a standard mailing
            bool scheduled = samples.mailingSample.scheduleMailing(stdMailingId);
            Console.WriteLine("Schedule mailing: " + scheduled);

            // Load a transactional mailing
            bool loaded = samples.mailingSample.loadMailing(txMailingId);
            Console.WriteLine("Load mailing: " + loaded);
            
            Console.WriteLine("\n\nWaiting for 10 seconds ... ");
            Thread.Sleep(10000); // Wait for 10 seconds for Tx Mailing to be active.

            // txnSend and related APIs.            
            // We can use this handle for mutiple request. In other words 
            String txMailingHandle = samples.mailingSample.getTxnMailingHandle(txMailingId);
            Console.WriteLine("txMailingHandle: " + txMailingHandle );

            bool sent = samples.mailingSample.txnSend(txMailingHandle);
            Console.WriteLine("Send record using txnSend for " + txMailingId.id + ": " + sent);
            
            Console.WriteLine("\n\nWaiting for 10 seconds ... ");
            Thread.Sleep(10000); // Wait for 10 seconds before closing Tx Mailing.
            
            bool closed = samples.mailingSample.closeMailing(txMailingId);
            Console.WriteLine("Close mailing: " + closed);
            Console.WriteLine("\n\n\n Completed. Please press any key to Exit");
            Console.Read();
        }

        private List<ObjectId> createSelectiveData(string sequence)
        {
            List<ObjectId> objectIds = new List<ObjectId>();  // Use a list to hold all object IDs

            objectIds.Add(dsSample.createIDS("ids_dotnet_" + sequence));            

            // User need to use his own external data source.
            //objectIds.Add(dsSample.createEDS("eds_dotnet_" + sequence));

            // Wait for data source finish imporing records
            helper.waitForIdle((InternalDataSourceId)getObjectId(objectIds, typeof(InternalDataSourceId)));

            objectIds.Add(targetSample.createTarget(
                "target_dotnet_" + sequence,
                (InternalDataSourceId)getObjectId(objectIds, typeof(InternalDataSourceId))));

            objectIds.Add(ruleSample.createRule("rule_dotnet_" + sequence, (TargetId)getObjectId(objectIds, typeof(TargetId))));

            return objectIds;
            
        }
    }


    // A policy to trust all certificate
    public class TrustAllCertificatePolicy : System.Net.ICertificatePolicy
    {
        public TrustAllCertificatePolicy()
        { }

        public bool CheckValidationResult(System.Net.ServicePoint sp,
         X509Certificate cert, System.Net.WebRequest req, int problem)
        {
            return true;
        }
    }
}
