﻿/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2014 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ConsoleApplication1
{
    class TargetSample
    {
        private MailingService service;
        private SampleHelper helper;

        public TargetSample(MailingService service)
        {
            this.service = service;
            this.helper = new SampleHelper(service);
        }


        /* Create a target based on the data source created in DataSourceSample.
         * 
         * targetName is the name of target.
         * dataSourceId is the data source ID this target will use.
         * 
         * return the ID of creatd target.
         */
        public TargetId createTarget(string targetName, DataSourceId dataSourceId)
        {
            Target target = new Target();
            target.name = targetName;
            target.dataSourceId = dataSourceId;
            target.type = "ADVANCED";
            target.description = "Target from dotnet";
            target.emailAddressFieldName = "email_address";     // From the data source
            target.bounceFieldName = "email_address_status";    // System generated based on email column
            target.excludeBounce = true;
            target.unsubscribeFieldName = "unsub";              // System generated
            target.excludeUnsubscribe = true;

            // Wait for data source status to be idle before creating target
            helper.waitForIdle(dataSourceId);

            return (TargetId) helper.create(target);
        }

         /* Create a target based on the data source created in DataSourceSample.
         *
         * targetName is the name of target.
         * dataSourceId is the data source ID this target will use.
         *
         * return the ID of creatd target.
         */
        public TargetId createChannelTarget(string targetName, DataSourceId dataSourceId)
        {
            Target target = new Target();
            target.name = targetName;
            target.dataSourceId = dataSourceId;
            target.type = "ADVANCED";
            target.description = "Target from dotnet";
            target.emailAddressFieldName = "id";     // set non-email-address column
            // Wait for data source status to be idle before creating target
            helper.waitForIdle(dataSourceId);

            return (TargetId) helper.create(target);
        }

        /* Create a target based on the data source created in DataSourceSample.
         * 
         * targetName is the name of target.
         * 
         * return the ID of creatd target.
         */
        public TargetId createTarget(string targetName)
        {
            DataSourceId dataSourceId = new DataSourceSample(service).createIDS("IDS_" + targetName);
            return createTarget(targetName, dataSourceId);
        }

        /* Create a target based on the data source created in DataSourceSample.
         *
         * targetName is the name of target.
         *
         * return the ID of creatd target.
         */
        public TargetId createChannelTarget(string targetName)
        {
            DataSourceId dataSourceId = new DataSourceSample(service).createIDS("IDS_" + targetName);
            return createChannelTarget(targetName, dataSourceId);
        }

        public void getAndPrint(TargetId targetId)
        {
            Target target = (Target) helper.get(targetId);
            Console.WriteLine("\nTarget:");
            Console.WriteLine("\tID: " + target.objectId.id); 
            Console.WriteLine("\tName: " + target.name);
            Console.WriteLine("\tType: " + target.type);
            Console.WriteLine("\tDataSource: " + target.dataSourceId.id);
            Console.WriteLine("\tEmailField: " + target.emailAddressFieldName);
            Console.WriteLine("\tSMS: " + target.smsAddressFieldName);
            Console.WriteLine("\tBounceFieldName: " + target.bounceFieldName);
            Console.WriteLine("\tUnsubFieldName: " + target.unsubscribeFieldName);
            Console.WriteLine("\tDescription: " + target.description);
            Console.WriteLine("\tQuery: " + target.advancedQuery);
            Console.WriteLine("\tCampaign: " + target.campaignId);
            Console.WriteLine("\tCreatedTime: " + target.createdTime);
            Console.WriteLine("\tModifiedTime: " + target.modifiedTime);
            Console.WriteLine("\tOrganization: " + target.organizationId.id);
            Console.WriteLine("\tOwenr: " + target.ownerId.id);
            Console.WriteLine("\tTotalRecords: " + target.totalRecords);
        }
    }
}
