﻿/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2014 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace ConsoleApplication1
{
    class TemplateSample
    {
        private MailingService service;
        private SampleHelper helper;        
        AttachmentSample attachmentSample;
        ContentBlockSample cbSample;
        SystemAddressSample addressSample;
        DataSourceSample dataSourceSample;



        public TemplateSample(MailingService service)
        {
            addressSample = new SystemAddressSample(service);
            cbSample = new ContentBlockSample(service);
            attachmentSample = new AttachmentSample(service);
            dataSourceSample = new DataSourceSample(service);
            this.service = service;          
            this.helper = new SampleHelper(service);
        }


        /* Create a template.
         * 
         * templateName is the name of template.
         * 
         * return the ID of created template.
         */
        public TemplateId createTemplate(string templateName)
        {
            Template template = new Template();

            template.openTracking = true;
            

            AttachmentId attachmentId = attachmentSample.createAttachment("Attachment_" + templateName);
            template.attachmentId = new AttachmentId[] { attachmentId  };

            ContentBlockId contentBlockId = cbSample.createContentBlock("Content_Block_" + templateName);
            template.contentBlockId = new ContentBlockId[] { contentBlockId };
            template.linkTracking = "Obfuscated";
            template.headerEncoding = "EIGHT_BIT";
            template.bodyEncoding = "BASE_64";
            template.name = templateName;

            // Use outputBodyCharSet(Token)
            template.Item = CharSet.ASCII;

            // Use outputHeaderCharSet(Token)
            template.Item1 = CharSet.UTF8;

            // Set message part
            MessagePart messagePart = new MessagePart();
            messagePart.content = "<html>" +
                     "<body>" +
                     "Hello, This is a simple HTML message with two links<br/>" +
                     "This is <a href=\"http://www.somedomain.com/link1\">Link 1</a>" +
                     "<br/>This is <a href=\"http://www.somedomain.com/link2\">Link 2</a>" +
                     "<br/>This is <a href=\"#\">Link 3</a> " +
                     "</body>" +
                     "</html>"; 

            messagePart.format = "HTML";
            messagePart.isXsl = false;
            template.messagePart = new MessagePart[] { messagePart };

            FetchLinksTemplateRequest linksRequest = new FetchLinksTemplateRequest();
            linksRequest.messageFormat = "HTML";
            linksRequest.template = template;

            FetchLinksResponse linksReponse = service.fetchLinks(new fetchLinks(linksRequest)).fetchLinksResponse;

            FetchLinkResponse[] individualLinkResponses = linksReponse.fetchLinkResponse;

            List<NamedLink> namedLinks = new List<NamedLink>();

            foreach (FetchLinkResponse link in individualLinkResponses)
            {
                NamedLink namedLink = link.namedLink;
                namedLink.tagFormat = "Obfuscated";
                if (namedLink.url != null && namedLink.url.Contains("link1"))
                {
                    namedLink.tagType = "CLICK";
                    namedLinks.Add(namedLink);
                }
                else if (namedLink.url != null && namedLink.url.Contains("link2"))
                {
                    namedLink.tagType = "UNSUB";
                    namedLinks.Add(namedLink);
                }
                else if (namedLink.url != null && namedLink.url.Contains("#"))
                {
                    namedLink.tagType = "FBLIKE";

                    NameValuePair property = new NameValuePair();
                    property.name = "pagettl";
                    property.value = "test123";

                    namedLink.trackingTagProperties = new NameValuePair[] { property };
                    namedLinks.Add(namedLink);
                }
            }

            template.messagePart[0].namedLinks = namedLinks.ToArray<NamedLink>();
            template.bounceAddressId = (SystemAddressId)addressSample.listByOrCreateEmailAddress("BOUNCE", "bounce@sample.com").objectId;
            template.fromAddressId = (SystemAddressId)addressSample.listByOrCreateEmailAddress("FROM", "from@sample.com").objectId; ;
            template.replyAddressId = (SystemAddressId)addressSample.listByOrCreateEmailAddress("REPLY", "reply@sample.com").objectId; ;
            
            template.description = "Template from dotnet";
            template.fromName = "From Name";            
            template.isApproved = true;            
            template.subject = "template subject from dotnet";
            template.header = new String[] { "Template header" };

            String idsName = "IDS_" + templateName;
            String catalogName = "CATALOG_" + templateName;

            dataSourceSample.createIDS(idsName);
            dataSourceSample.createCatalog(catalogName);

            // We could have used the dataSourceId and catalogId from createIDS and createCatalog but
            // the following code is for demonstrating the use of list api

            DataSourceFilter dataSourceFilter = new DataSourceFilter();
            ScalarStringFilterCondition typeCondition = new ScalarStringFilterCondition();
            ScalarStringFilterCondition nameCondition = new ScalarStringFilterCondition();

            nameCondition.@operator = FilterStringScalarOperator.EQUAL;
            typeCondition.@operator = FilterStringScalarOperator.EQUAL;

            typeCondition.value= "INTERNAL";
            nameCondition.value = idsName;
            dataSourceFilter.typeCondition = typeCondition;
            dataSourceFilter.nameCondition = nameCondition;
            ListRequest listRequest = new ListRequest();
            listRequest.filter = dataSourceFilter;

            DataSourceId dataSourceId = (DataSourceId)service.list(new list(listRequest)).listResponse.objectId[0];

            typeCondition.value = "CATALOG";
            nameCondition.value = catalogName;
            dataSourceFilter.typeCondition = typeCondition;
            dataSourceFilter.nameCondition = nameCondition;
            listRequest = new ListRequest();
            listRequest.filter = dataSourceFilter;

            DataSourceId catalogId = (DataSourceId)service.list(new list(listRequest)).listResponse.objectId[0];

            template.Item2 = dataSourceId;

            CatalogField catalogField = new CatalogField();
            catalogField.catalogId = (CatalogId) catalogId;

            // The following field should be present in the underlying datasource/target
            catalogField.mappingFieldName = new String[] {"id"};
            template.catalogField = new CatalogField []{ catalogField };

            return (TemplateId)helper.create(template);
        }

        /* Create a template.
         * 
         * templateName is the name of template.
         * 
         * Please note that it is license dependent feature 
         * return the ID of created template.
         */
        public ChannelTemplateId createChannelTemplate(string templateName)
        {
            ChannelTemplate template = new ChannelTemplate();

            template.bodyEncoding = "BASE_64";
            template.description = "Template from dotnet";
            template.headerEncoding = "EIGHT_BIT";
            template.isApproved = true;
            template.name = templateName;

            // Get the ChannelId By Name
            ChannelFilter channelFilter = new ChannelFilter();
            ListRequest listRequest = new ListRequest();
            ScalarStringFilterCondition nameCondition = new ScalarStringFilterCondition();
            nameCondition.@operator = FilterStringScalarOperator.EQUAL;
            nameCondition.value = "Push Notification";
            channelFilter.nameCondition = nameCondition;
            listRequest.filter = channelFilter;
            ListResponse listResponse = service.list(new list(listRequest)).listResponse;

            if (listResponse == null || listResponse.objectId.Length < 1)
            {
                Console.WriteLine("Either Message Queue with specified name is not exist OR Message Queue license is not present");
                return null;
            }

            ChannelId channelId = (ChannelId)listResponse.objectId[0];
            template.channelId = channelId;

            AttachmentId attachmentId = attachmentSample.createAttachment("Attachment_" + templateName);
            template.attachmentId = new AttachmentId[] { attachmentId };

            ContentBlockId contentBlockId = cbSample.createContentBlock("Content_Block_" + templateName);
            template.contentBlockId = new ContentBlockId[] { contentBlockId };
            template.linkTracking = "Obfuscated";

            // Use outputBodyCharSet(Token)
            template.Item = CharSet.ASCII;

            // Use outputHeaderCharSet(Token)
            template.Item1 = CharSet.UTF8;

            // Set message part
            MessagePart messagePart = new MessagePart();
            // content size should not be greater than 140 characters
            messagePart.content = "This is content for push notification message part";
            messagePart.format = "PUSH_NOTIFICATION";
            messagePart.isXsl = false;
            template.messagePart = new MessagePart[] { messagePart };

            String idsName = "IDS_" + templateName;
            String catalogName = "CATALOG_" + templateName;
            InternalDataSourceId idsId = dataSourceSample.createIDS(idsName);
            CatalogId catalogId = dataSourceSample.createCatalog(catalogName);
                      

            // We can set either DataSourceId or TargetId here
            template.Item2 = idsId;
            CatalogField catalogField = new CatalogField();
            catalogField.catalogId = (CatalogId)catalogId;

            // The following field should be present in the underlying datasource/target
            catalogField.mappingFieldName = new String[] { "id" };
            template.catalogField = new CatalogField[] { catalogField };

            ChannelHeader header1 = new ChannelHeader();
            header1.headerName = "CustomUserHeader1";
            header1.headerValue= "CustomUserHeaderValue1";

            ChannelHeader header2 = new ChannelHeader();
            header2.headerName = "CustomUserHeader2";
            header2.headerValue = "CustomUserHeaderValue2";

            template.channelHeader = new ChannelHeader[] { header1, header2 };

            return (ChannelTemplateId)helper.create(template);
        }

        public void getAndPrint(BaseTemplateId templateId)
        {
            if (templateId is TemplateId)
            {
                getAndPrint((TemplateId)templateId);
            }
            else
            {
                getAndPrint((ChannelTemplateId)templateId);
            }
        }

        private void getAndPrint(TemplateId templateId)
        {
            Template template = (Template)helper.get(templateId);
            Console.WriteLine("\nTemplate:");
            Console.WriteLine("\tID: " + template.objectId.id);
            Console.WriteLine("\tName: " + template.name);
            Console.WriteLine("\tCampaign: " + template.campaignId);
            Console.WriteLine("\tCreatedTime: " + template.createdTime);
            Console.WriteLine("\tDescription: " + template.description);
            Console.WriteLine("\tModifiedTime: " + template.modifiedTime);
            Console.WriteLine("\tOrganization: " + template.organizationId.id);
            Console.WriteLine("\tOwenr: " + template.ownerId.id);
            Console.WriteLine("\tAttchment Size: " + template.attachmentId.Length);
            Console.WriteLine("\tContentBlock Size: " + template.contentBlockId.Length);
            Console.WriteLine("\tHeader Size: " + template.header.Length);
            Console.WriteLine("\tBodyEncoding: " + template.bodyEncoding);
            Console.WriteLine("\tFromName: " + template.fromName);
            Console.WriteLine("\tSubject: " + template.subject);

            Console.WriteLine("\tMessagePart: ");
            for (int i = 0; i < template.messagePart.Length; i++)
            {
                Console.WriteLine("\tMessagePart size: " + template.messagePart[i].content.Length);
            }
        }

        private void getAndPrint(ChannelTemplateId templateId)
        {
            ChannelTemplate template = (ChannelTemplate)helper.get(templateId);
            Console.WriteLine("\nTemplate:");
            Console.WriteLine("\tID: " + template.objectId.id);
            Console.WriteLine("\tName: " + template.name);
            Console.WriteLine("\tCampaign: " + template.campaignId);
            Console.WriteLine("\tCreatedTime: " + template.createdTime);
            Console.WriteLine("\tDescription: " + template.description);
            Console.WriteLine("\tModifiedTime: " + template.modifiedTime);
            Console.WriteLine("\tOrganization: " + template.organizationId.id);
            Console.WriteLine("\tOwenr: " + template.ownerId.id);
            Console.WriteLine("\tAttchment Size: " + template.attachmentId.Length);
            Console.WriteLine("\tContentBlock Size: " + template.contentBlockId.Length);
            Console.WriteLine("\tChannel Id "+ template.channelId.id);                
            Console.WriteLine("\tMessagePart: ");
            for (int i = 0; i < template.messagePart.Length; i++)
            {
                Console.WriteLine("\tMessagePart size: " + template.messagePart[i].content.Length);
            }

            if (template.channelHeader.Length > 0)
            {
                Console.WriteLine("Channel Headers");
                for (int i = 0; i < template.channelHeader.Length; i++)
                {
                    ChannelHeader channelHeader = new ChannelHeader();
                    Console.WriteLine(channelHeader.headerName + " : " + channelHeader.headerValue); 
                }
            }

        }


        /* Import message part using a zip file and media server.
         * 
         * templateId is the ID of template where message part will be imported.
         * mediaServerId is the media server ID used to store the image files.
         * zipFile is the zip file that contains HTML and image files.
         * folderName is the directory to unzip the zip file.
         * 
         * return if the importing operaion is successful.
         * 
         */
        public bool importMessagePart(TemplateId templateId, MediaServerId mediaServerId, FileStream zipFile, String folderName)
        {
            // Create request
            ImportMessagePartRequest request = new ImportMessagePartRequest();
            request.folderName = folderName;
            request.isXsl = false;
            request.mediaServerId = mediaServerId;
            request.templateId = templateId;

            // Set content from zip file
            BinaryReader reader = new BinaryReader(zipFile);
            byte[] data = reader.ReadBytes((int) zipFile.Length);
            request.zipFile = data;

            return service.importMessagePart(new importMessagePart(request)).importMessagePartResponse.success;
        }


        /* Test template for its dynamic content. 
         * Email will be sent and tokens in the template will be replaced wit values.
         * 
         * templateId is the templateId that will be tested.
         * return if the test request is sent successfully.
         */
        public bool testTemplate(TemplateId templateId)
        {
            // Create request
            TestTemplateRequest request = new TestTemplateRequest();
            request.Item = "Subject Prefix: ";
            request.templateId = templateId;
            request.useMultiPart = false;
            request.address = new String[] {"test_address@testtemplate.com"};
            request.format = new String[] {"HTML"};

            // Make sure you have these two tokens in your template
            NameValuePair pair1 = new NameValuePair();
            pair1.name = "EMAIL";
            pair1.value = "testmail@testtemplate.com";

            NameValuePair pair2 = new NameValuePair();
            pair2.name = "NAME";
            pair2.value = "testmail name";

            // Set data to request and make call
            request.tokenValue = new NameValuePair[] {pair1, pair2};
            return service.test(new test(request)).testResponse.success;
        }

        
        /* Validate if a template's xsl content is valid.
         */
        public bool validateXsl(TemplateId templateId)
        {
            ValidateXslRequest request = new ValidateXslRequest();
            request.templateId = templateId;
            request.messageFormat = "HTML";

            return service.validateXsl(new validateXsl(request)).validateXslResponse.valid;
        }
    }


}
