﻿/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2014 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ConsoleApplication1
{
    class GlobalUndeliverableSample
    {
        private MailingService service;
        private SampleHelper helper;

        public GlobalUndeliverableSample(MailingService service)
        {
            this.service = service;
            this.helper = new SampleHelper(service);
        }

        public void getRecipientAddress(String messageDestination, String channelType, String organizationId)
        {
            Console.WriteLine("Start of  getGlobalUndeliverables sample ");

            GlobalUndeliverableRequest request = prepareGlobalUndeliverableRequest(channelType, organizationId);
            request.Item = messageDestination;
            request.ItemElementName = ItemChoiceType1.recipientAddress;

            GlobalUndeliverableInfo[] globalUndeliverableInfo = service.getGlobalUndeliverable(request);

            if (globalUndeliverableInfo != null && globalUndeliverableInfo.Length > 0)
            {
                Console.WriteLine("Number of matching records " + globalUndeliverableInfo.Length);
                foreach (GlobalUndeliverableInfo info in globalUndeliverableInfo)
                {
                    Console.WriteLine(" " + info.recipientAddress + " " + info.channelType+" "+info.failureCode);
                }
            }
            else
            {
                Console.WriteLine("No Matching Records Found");
            }
            Console.WriteLine("End of  getGlobalUndeliverables");
        }

        public void getByFailureCode(String failureCode, String channelType, String organizationId)
        {
            Console.WriteLine("Start of  getGlobalUndeliverables sample ");

            GlobalUndeliverableRequest request = prepareGlobalUndeliverableRequest(channelType, organizationId);
            request.Item = failureCode;
            request.ItemElementName = ItemChoiceType1.failureCode;

            GlobalUndeliverableInfo[] globalUndeliverableInfo = service.getGlobalUndeliverable(request);

            if (globalUndeliverableInfo != null && globalUndeliverableInfo.Length > 0)
            {
                Console.WriteLine("Number of matching records " + globalUndeliverableInfo.Length);
                foreach (GlobalUndeliverableInfo info in globalUndeliverableInfo)
                {
                    Console.WriteLine(" " + info.recipientAddress + " " + info.channelType+" "+info.failureCode);
                }
            }
            else
            {
                Console.WriteLine("No Matching Records Found");
            }
            Console.WriteLine("End of  getGlobalUndeliverables");
        }

        public void reactivateGlobalUndeliverable(String recipientAddress, String channelType, String orgId)
        {
            Console.WriteLine("Start of  reactivateGlobalUndeliverable");

            ReactivateGlobalUndeliverableRequest reactivateGlobalUndeliverableRequest = new ReactivateGlobalUndeliverableRequest();


            GlobalUndeliverableInfo globalUndeliverableInfo = new GlobalUndeliverableInfo();

            globalUndeliverableInfo.recipientAddress = recipientAddress;
            globalUndeliverableInfo.channelType = channelType;

            // Ignore OrgId if config value set as acrross organization
            // OrgId mandatory if config value set as per organization
            OrganizationId organizationId = new OrganizationId();
            organizationId.id = orgId;
            reactivateGlobalUndeliverableRequest.organizationId = organizationId;

            // We can specify multiple addresses to reactivate
            reactivateGlobalUndeliverableRequest.globalUndeliverableInfo = new GlobalUndeliverableInfo[] { globalUndeliverableInfo };


            ReactivateGlobalUndeliverableResponse response = service.reactivateGlobalUndeliverable(reactivateGlobalUndeliverableRequest);

            Console.WriteLine("Is Record Reactivated - " + response.success);

        }

        public void addGlobalUndeliverable()
        {
            Console.WriteLine("Start Of Add GlobalUndeliverable");
            AddGlobalUndeliverableRequest request = new AddGlobalUndeliverableRequest();

            GlobalUndeliverableInfo globalUndeliverableInfo1 = new GlobalUndeliverableInfo();
            globalUndeliverableInfo1.recipientAddress = "test123@undeliverable.com";
            globalUndeliverableInfo1.channelType = "EMAIL";
            globalUndeliverableInfo1.failureCode = 2001;
            globalUndeliverableInfo1.failureCodeSpecified = true;

            GlobalUndeliverableInfo globalUndeliverableInfo2 = new GlobalUndeliverableInfo();
            globalUndeliverableInfo2.recipientAddress = "9876543210";
            globalUndeliverableInfo2.channelType = "SMS";
            globalUndeliverableInfo2.failureCode = 2103;
            globalUndeliverableInfo2.failureCodeSpecified = true;
            
            // OrgId will be ignored if config value set as acrross organization
            OrganizationId organizationId = new OrganizationId();
            organizationId.id = "1";
            request.organizationId = organizationId;

            request.globalUndeliverableInfo = new GlobalUndeliverableInfo[] { globalUndeliverableInfo1, globalUndeliverableInfo2 };

            Console.WriteLine("Adding " + globalUndeliverableInfo1.recipientAddress + " and " + globalUndeliverableInfo2.recipientAddress + " to GU.");
            service.addGlobalUndeliverable(request);
        }

        private GlobalUndeliverableRequest prepareGlobalUndeliverableRequest(String channelType, String organization)
        {
            GlobalUndeliverableRequest request = new GlobalUndeliverableRequest();

            if (!string.IsNullOrWhiteSpace(organization))
            {
                OrganizationId organizationId = new OrganizationId();
                organizationId.id = organization;
                request.organizationId = organizationId;
            }

            request.channelType = channelType;
            request.pageNumber = 1;
            request.recordsPerPage = 25;
            request.pageNumberSpecified = true;
            request.recordsPerPageSpecified = true;

            return request;
        }

    }
}
