﻿/* Copyright 2008, 2009, StrongMail Systems, Inc.  All rights reserved. */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ConsoleApplication1
{
    class RuleSample
    {
        private MailingService service;
        private SampleHelper helper;

        public RuleSample(MailingService service)
        {
            this.service = service;
            this.helper = new SampleHelper(service);
        }


        /* Crate a rule using the target create in TargetSample.
         * 
         * ruleName is the name of the rule to be created.
         * targetId is the target to be used with the rule.
         * 
         * return the ID of the created rule.
         */
        public RuleId createRule(string ruleName, TargetId targetId)
        {
            Rule rule = new Rule();
            rule.description = "Rule from dotnet";
            rule.name = ruleName;

            // Get the target used by the rule
            Target target = (Target)helper.get(targetId);
            rule.targetId = new TargetId[] {targetId} ;

            // Specify IF part
            RuleIfPart ifpart = new RuleIfPart();            

            // Create three conditions useing the target created in TargetSample
            RuleIfPartCondition cond_1 = new RuleIfPartCondition();
            cond_1.column = target.name + ".email_address";
            cond_1.op = ComparisonOperation.EQUALS;
            cond_1.value = "email_for_dotnet@dotnet.com";
            ifpart.condition1 = cond_1;
            

            // IF condition 2
            RuleIfPartCondition cond_2 = new RuleIfPartCondition();
            cond_2.column = target.name + ".name";
            cond_2.op = ComparisonOperation.CONTAINS;
            cond_2.value = "dotnet1";

            RuleIfPartCondition cond_3 = new RuleIfPartCondition();
            cond_3.column = target.name + ".name";
            cond_3.op = ComparisonOperation.CONTAINS;
            cond_3.value = "dotnet2";

            // Set IF part, format for conditions are: Condition1 AND/OR Conditoni AND/OR Condition2 ...            
            ifpart.Items = new Object[] { LogicalOperation.AND, cond_2, LogicalOperation.OR, cond_3};
            rule.ifPart = ifpart;

            // Specify THEN part
            RuleThenPart thenpart = new RuleThenPart();
            TextRuleValue textValue = new TextRuleValue();
            textValue.value = "This is then part";
            thenpart.Item = textValue;
            rule.thenPart = thenpart;

            // Specify ELSE part
            RuleElsePart elsepart = new RuleElsePart();
            ColumnRuleValue colValue = new ColumnRuleValue();
            colValue.value = target.name + ".name";
            elsepart.Item = colValue;
            rule.elsePart = elsepart;

            return (RuleId)helper.create(rule);
        }


        /* Get and print out a rule.
         * 
         * ruleId is the ID of the rule to be printed.
         */
        public void getAndPrint(RuleId ruleId)
        {
            Rule rule = (Rule) helper.get(ruleId);
            Console.WriteLine("\nRule:");
            Console.WriteLine("\tID: " + rule.objectId.id);
            Console.WriteLine("\tName: " + rule.name);
            Console.WriteLine("\tDescription: " + rule.description);
            Console.WriteLine("\tCampaign: " + rule.campaignId);
            Console.WriteLine("\tCreatedTime: " + rule.createdTime);
            Console.WriteLine("\tModitifedTime: " + rule.modifiedTime);
            Console.WriteLine("\tOrganization: " + rule.organizationId.id);
            Console.WriteLine("\tOwenr: " + rule.ownerId.id);

            // Print condition 1
            RuleIfPartCondition cond_1 = rule.ifPart.condition1;
            Console.WriteLine("\tRule-If");
            Console.WriteLine("\t\t" + cond_1.column + " " + cond_1.op + " " + cond_1.value);

            // Print condition 2
            LogicalOperation logicalOp1 = (LogicalOperation) rule.ifPart.Items[0];
            RuleIfPartCondition cond_2 = (RuleIfPartCondition) rule.ifPart.Items[1];

            // Print condition 2
            LogicalOperation logicalOp2 = (LogicalOperation)rule.ifPart.Items[2];
            RuleIfPartCondition cond_3 = (RuleIfPartCondition)rule.ifPart.Items[3];


            Console.WriteLine("\t\t" + logicalOp1.ToString() + " " + cond_2.column + " " + cond_2.op + " " + cond_2.value);
            Console.WriteLine("\t\t" + logicalOp2.ToString() + " " + cond_3.column + " " + cond_3.op + " " + cond_3.value);

            // Print Then part
            Console.WriteLine("\tRule-Then: " + ((TextRuleValue) rule.thenPart.Item).value);

            // Print Else part
            Console.WriteLine("\tRule-Else: " + ((ColumnRuleValue) rule.elsePart.Item).value);
        }
    }
}
