﻿/* Copyright 2008, 2009, StrongMail Systems, Inc.  All rights reserved. */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ConsoleApplication1
{
    class SuppressionListSample
    {
        private MailingService service;
        private SampleHelper helper;

        public SuppressionListSample(MailingService service)
        {
            this.service = service;
            this.helper = new SampleHelper(service);
        }


        /* Create a suppresion list and add some data.
         * 
         * suppresionListName is the name of suppresion list.
         * return the ID of created suppression list.
         */
        public SuppressionListId createSuppressionList(string suppresionListName)
        {
            SuppressionList suppress = new SuppressionList();
            suppress.description = "SupressionList from dotnet";
            suppress.includeByDefaultOnMailings = true;
            suppress.name = suppresionListName;

            SuppressionListId suppressionListId = (SuppressionListId) helper.create(suppress);

            // Add some records
            addRecords(suppressionListId);

            return suppressionListId;
        }

        /* Create a suppresion list and add some data.
        * 
        * suppresionListName is the name of suppresion list.
        * return the ID of created suppression list.
        */
        public SuppressionListId createChannelSuppressionList(string suppresionListName)
        {
            SuppressionList suppress = new SuppressionList();
            suppress.description = "SupressionList from dotnet";
            suppress.includeByDefaultOnMailings = true;
            suppress.name = suppresionListName;

            // Get the ChannelId By Name
            ChannelFilter channelFilter = new ChannelFilter();
            ListRequest listRequest = new ListRequest();
            ScalarStringFilterCondition nameCondition = new ScalarStringFilterCondition();
            nameCondition.@operator = FilterStringScalarOperator.EQUAL;
            nameCondition.value = "Push Notification";
            channelFilter.nameCondition = nameCondition;
            listRequest.filter = channelFilter;
            ListResponse listResponse = service.list(listRequest);

            if (listResponse == null || listResponse.objectId.Length < 1)
            {
                Console.WriteLine("Either Message Queue with specified name is not exist OR Message Queue license is not present");
                return null;
            }

            // Set Channel Id
            ChannelId channelId = (ChannelId)listResponse.objectId[0];
            suppress.channelId = channelId;
            return (SuppressionListId)helper.create(suppress);
        }


        /* Add 2 records to suppresion list using inline records.
         * 
         * suppresionListId is the ID where records will add.
         */
        public void addRecords(SuppressionListId suppressionListId)
        {
            // Craete reuqest
            AddSuppressionListRecordsRequest request = new AddSuppressionListRecordsRequest();
            request.suppressionListId = suppressionListId;

            // Create data
            string[] suppresses = new string[2];
            suppresses[0] = "dotnet.com";
            suppresses[1] = "suppress@dotnet.com";

            // Set data to request and make call.
            request.Items = suppresses;
            service.addRecords(request);
        }


        /* Remove one record that was created in addRecordsInline() from the suppresion list.
         * 
         * suppresionListId is the suppresion list ID whose record will be removed.
         * return the number of records that were removed.
         */
        public int removeRecordsInline(SuppressionListId suppresionListId)
        {
            // Create request
            RemoveSuppressionListRecordsRequest request = new RemoveSuppressionListRecordsRequest();
            request.suppressionListId = suppresionListId;

            // Set reocrd to remove
            String[] addressMatches = new String[1];
            addressMatches[0] = "suppress_from_dotnet_1@dotnet.com";
            request.Items = addressMatches;

            return service.removeRecords(request).recordsRemoved;
        }

        public void getAndPrint(SuppressionListId suppressId)
        {
            SuppressionList suppress = (SuppressionList) helper.get(suppressId);
            Console.WriteLine("\nSuppressionList:");
            Console.WriteLine("\tID: " + suppress.objectId.id);
            Console.WriteLine("\tName: " + suppress.name);
            Console.WriteLine("\tCampaign: " + suppress.campaignId);
            Console.WriteLine("\tCreatedTime: " + suppress.createdTime); 
            Console.WriteLine("\tDescription: " + suppress.description);
            Console.WriteLine("\tModifiedTime: " + suppress.modifiedTime);
            Console.WriteLine("\tOrganization: " + suppress.organizationId.id);
            Console.WriteLine("\tOwenr: " + suppress.ownerId.id);
            Console.WriteLine("\tTotal Records: " + suppress.totalRecords);
        }
    }
}
