/* Copyright 2008, 2009, StrongMail Systems, Inc.  All rights reserved. */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ConsoleApplication1
{
    class SystemAddressSample
    {
        private MailingService service;
        private SampleHelper helper;

        public SystemAddressSample(MailingService service)
        {
            this.service = service;
            this.helper = new SampleHelper(service);
        }


        /* List SystemAddress by type (bounce/from/reply) and email address.
         * There is no filter by email address, so we need to get all address for one type and filter them again.
         * 
         * type is the SystemAddressType with value BOUNCE, FROM and REPLY.
         * emailAddress is the email address to match
         * 
         * return the system address of type matching given email address.
         */
        public SystemAddress listByOrCreateEmailAddress(String type, String emailAddress)
        {
            // Create and set attribute for filter
            SystemAddressFilter filter = new SystemAddressFilter();
            helper.setFilter(filter);

            // Create type condition
            ScalarStringFilterCondition typeCondition = new ScalarStringFilterCondition();
            typeCondition.@operator = FilterStringScalarOperator.EQUAL;
            typeCondition.value = type;

            filter.typeCondition = typeCondition;

            // Create request and make call
            ListRequest request = new ListRequest();
            request.filter = filter;
            ObjectId[] addressIds = service.list(request).objectId;

                        // Go through all addresses to get the one that matches email address
            for (int i = 0; addressIds != null && i < addressIds.Length; i++)
            {
                SystemAddress address = (SystemAddress)helper.get((SystemAddressId) addressIds[i]);
                if (address.emailAddress.Equals(emailAddress))
                {
                    return address;
                }
            }

            SystemAddress systemAddress = new SystemAddress();
            systemAddress.emailAddress = emailAddress;
            if (type.Equals("BOUNCE",StringComparison.OrdinalIgnoreCase))
            {
                systemAddress.Items= new Object[]{true};
                systemAddress.ItemsElementName =  new ItemsChoiceType2[]{ItemsChoiceType2.isBounce};

            }
            else if (type.Equals("FROM",StringComparison.OrdinalIgnoreCase))
            {
                systemAddress.Items = new Object[] {"Sample From", true };
                systemAddress.ItemsElementName = new ItemsChoiceType2[] { ItemsChoiceType2.fromName,ItemsChoiceType2.isFrom };
            }
            else if (type.Equals("REPLY", StringComparison.OrdinalIgnoreCase))
            {
                systemAddress.Items = new Object[] { true };
                systemAddress.ItemsElementName = new ItemsChoiceType2[] { ItemsChoiceType2.isReply };
            }

            SystemAddressId systemAddressId = (SystemAddressId)helper.create(systemAddress);

            return (SystemAddress)helper.get(systemAddressId);
        }
    }
}
