/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2014 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */


package com.strongmail.webserviceclient.samples.cxf;


import org.apache.cxf.binding.soap.SoapMessage;
import org.apache.cxf.configuration.jsse.TLSClientParameters;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.endpoint.ClientImpl;
import org.apache.cxf.endpoint.Endpoint;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.interceptor.Fault;
import org.apache.cxf.phase.Phase;
import org.apache.cxf.phase.PhaseInterceptor;
import org.apache.cxf.transport.http.HTTPConduit;
import org.apache.cxf.ws.security.wss4j.WSS4JOutInterceptor;
import org.apache.ws.security.SOAPConstants;
import org.apache.ws.security.WSConstants;
import org.apache.ws.security.WSPasswordCallback;
import org.apache.ws.security.handler.WSHandlerConstants;
import org.apache.ws.security.util.WSSecurityUtil;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

import com.strongmail.webserviceclient.samples.cxf.generated.MailingService;
import com.strongmail.webserviceclient.samples.cxf.generated.MailingService_Service;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.AddProgramContactRecordsRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.AddProgramContactRecordsResponse;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ProgramContactRecord;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ProgramId;
import com.sun.xml.ws.developer.SchemaValidationFeature;


import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.net.URL;
import java.security.KeyStore;
import java.security.cert.X509Certificate;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;
import javax.net.ssl.X509TrustManager;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.xml.bind.DatatypeConverter;
import javax.xml.namespace.QName;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;
import javax.xml.ws.Binding;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPBinding;


public class ProgramSample
{
  private static Properties CONFIG = null;
  private static final String IPFILE = "./program.conf";
  String programId;
  String contactId;
  String pkey;
  private MailingService service;

  public ProgramSample(MailingService service)
  {
    this.service = service;
    programId = CONFIG.getProperty("ProgramId");
    contactId = CONFIG.getProperty("ContactId");
    pkey = CONFIG.getProperty("Pkey");

  }

  public static MailingService createService()
  {
    // Read key store, key store password, wsdl file location and endpoint address
    // Endpoint address will replace the one in WSDL file
    System.out.println("Store: " + System.getProperty("javax.net.ssl.trustStore"));
    System.out.println("Password: " + System.getProperty("javax.net.ssl.trustStorePassword"));
    System.out.println("WSDL: " + System.getProperty("wsdl"));
    System.out.println("endpoint: " + System.getProperty("endpoint.address"));

    File wsdlFile = new File(System.getProperty("wsdl"));
    String endpoint = System.getProperty("endpoint.address");

    try
    {
      // Create service
      ClientSetup clientSetup = new ClientSetup(System.getProperty("services.namespace"),
                                                System.getProperty("services.namespace.schema"));
      MailingService service = clientSetup.createService(wsdlFile, endpoint);

      // Apply interceptor to add SOAP header
      // username/password/organization are the same as UI login.
      clientSetup.applyInterceptor(service, "admin", "admin", "admin", null, false);
      /*
        For enabling SSO token authentication, set the last argument in the above method to true
        and specify the token in the password (i.e 2nd) argument. E.g.
      clientSetup.applyInterceptor(service, "admin", "<token>", "admin", null, true);
       */

      return service;
    }
    catch (Exception e)
    {
      e.printStackTrace();
      System.exit(1);
    }

    return null;
  }


  private void addToProgram () throws Exception
  {
    int i=0;
    int contactSize=0;
    int pkeySize =0 ;
    String[] primaryKey;
    String[] contact;

    ProgramId progId = new ProgramId();
    progId.setId(programId);
    AddProgramContactRecordsRequest request = new AddProgramContactRecordsRequest();
    request.setProgramId(progId);
    contact = contactId.split(",");
    contactSize= contact.length;
    primaryKey=pkey.split(",");
    pkeySize = primaryKey.length;
    if(contactSize == pkeySize )
    {
          for (i=0;i<contactSize;i++)
          {
            ProgramContactRecord progContactRecord = new ProgramContactRecord();
            progContactRecord.setContactId(contact[i]);
            progContactRecord.setProgramDataSourcePk(primaryKey[i]);
            request.getProgramContactRecord().add(progContactRecord);
          }
    }
    else
    {
        int size = (contactSize < pkeySize) ? contactSize : pkeySize;
        for (i=0;i<size;i++)
        {
             ProgramContactRecord progContactRecord = new ProgramContactRecord();
             progContactRecord.setContactId(contact[i]);
             progContactRecord.setProgramDataSourcePk(primaryKey[i]);
             request.getProgramContactRecord().add(progContactRecord);
        }
        System.out.println("Enter all values for contactId and primaryKey");
    }

    // Make call
    AddProgramContactRecordsResponse response = (AddProgramContactRecordsResponse)service.addRecords(request);
    if (!response.isSuccess())
    {
      System.out.println("Error while adding records to the program.");
    }
    else
    {
      System.out.println("Records added successfully.");
    }
  }


  public static void main(String[] args) throws Exception
  {
    CONFIG = new Properties();
    CONFIG.load(new FileReader(IPFILE));

    ProgramSample sample = new ProgramSample(ProgramSample.createService());
    sample.addToProgram();
  }

  static class AuthenticationInterceptor extends WSS4JOutInterceptor
  {

    /**
     * Schema namespace
     */
    private String schemaNS;

    /**
     * Interceptor to set headers: organizationName and subOrganizationId
     */
    private PhaseInterceptor phaseInterceptor;

    /**
     * organizationName and subOrganizationId to login
     */
    private String organizationName;
    private Long subOrganizationId;

    private boolean isSSO;

    public AuthenticationInterceptor(String schemaNS,
                                     Map<String, Object> properties,
                                     String orgName,
                                     Long subOrganizationId,
                                     boolean isSSO)
    {
      super(properties);
      this.schemaNS = schemaNS;
      this.phaseInterceptor = new WSS4JOutInterceptorInternal();
      this.organizationName = orgName;
      this.subOrganizationId = subOrganizationId;
      this.isSSO = isSSO;
    }

    public void setOrganizationName(String orgName)
    {
      this.organizationName = orgName;
    }

    public void setSubOrganizationId(Long subOrganizationId)
    {
      this.subOrganizationId = subOrganizationId;
    }

    public void setUsername(String username)
    {
      getProperties().put(WSHandlerConstants.USER, username);
    }

    public void setPassword(String password)
    {
      getProperties().put(WSHandlerConstants.PW_CALLBACK_REF,
                          new ClientPasswordCallbackHandler(password));
    }

    public void setSSO(boolean SSO)
    {
      isSSO = SSO;
    }

    public void handleMessage(SoapMessage message) throws Fault
    {
      super.handleMessage(message);
      message.getInterceptorChain().add(phaseInterceptor);
    }


    /**
     * Set SOAP header before sending out message. Header includes username, password,
     * organizationName and optionally subOrganizationId. This message set organizationName and
     * subOrganizationId, and parent class will set username/password.
     */
    class WSS4JOutInterceptorInternal implements PhaseInterceptor<SoapMessage>
    {
      private static final String ORGANIZATION_TOKEN_ELEMENT_NAME = "OrganizationToken";
      private static final String ORGANIZATION_ELEMENT_NAME = "organizationName";
      private static final String SUB_ORGANIZATION_ID_ELEMENT_NAME = "subOrganizationId";
      private static final String ID_ELEMENT_NAME = "id";
      private static final String SSO_ELEMENT_NAME = "IsSSO";

      public WSS4JOutInterceptorInternal()
      {
        super();
      }

      public void handleMessage(SoapMessage message)
      {
        try
        {
          Element securityHeader = getSecurityHeader(message);

          // Create <OrganizationToken> and <organizationName> element,
          // and set <organizationName> as child of <OrganizationToken>
          Element e = securityHeader.getOwnerDocument().createElementNS(schemaNS,
                                                                        ORGANIZATION_TOKEN_ELEMENT_NAME);
          Element e2 = securityHeader.getOwnerDocument().createElementNS(schemaNS,
                                                                         ORGANIZATION_ELEMENT_NAME);
          Text t = securityHeader.getOwnerDocument().createTextNode(
            organizationName);
          e2.appendChild(t);
          e.appendChild(e2);

          // Create <subOrganizationId> if necessary. It is also child of <OrganizationToken>
          if (subOrganizationId != null)
          {
            Element e3 = securityHeader.getOwnerDocument().createElementNS(
              schemaNS, SUB_ORGANIZATION_ID_ELEMENT_NAME);
            Element e4 = securityHeader.getOwnerDocument().createElementNS(schemaNS,
                                                                           ID_ELEMENT_NAME);

            Text t2 = securityHeader.getOwnerDocument().createTextNode(
              subOrganizationId.toString());
            e4.appendChild(t2);
            e3.appendChild(e4);
            e.appendChild(e3);
          }
          securityHeader.appendChild(e);
          if (isSSO)
          {
            Element ssoElement = securityHeader.getOwnerDocument().createElementNS(schemaNS, SSO_ELEMENT_NAME);
            securityHeader.appendChild(ssoElement);
          }
         }
        catch (Throwable e)
        {
          e.printStackTrace();
        }
      }

      private Element getSecurityHeader(SoapMessage message) throws SOAPException
      {
        SOAPMessage doc = message.getContent(SOAPMessage.class);
        String actor = (String)getOption(WSHandlerConstants.ACTOR);

        SOAPConstants sc = WSSecurityUtil.getSOAPConstants(doc.getSOAPPart().getDocumentElement());
        return WSSecurityUtil.getSecurityHeader(doc.getSOAPPart(), actor, sc);
      }

      public Set<String> getAfter()
      {
        return Collections.emptySet();
      }

      public Set<String> getBefore()
      {
        return Collections.emptySet();
      }

      public String getId()
      {
        return WSS4JOutInterceptorInternal.class.getName();
      }

      public String getPhase()
      {
        return Phase.POST_PROTOCOL;
      }

      public void handleFault(SoapMessage message)
      {
        //nothing
      }
    }
  }

  static class ClientPasswordCallbackHandler implements CallbackHandler
  {
    private String password;

    public ClientPasswordCallbackHandler(String password)
    {
      this.password = password;
    }

    public void handle(Callback[] callbacks)
      throws IOException, UnsupportedCallbackException
    {
      WSPasswordCallback pc = (WSPasswordCallback)callbacks[0];
      pc.setPassword(password);
    }
  }

  static class ClientSetup
  {
    /**
     * Service name constant: defined in WSDL
     */
    private  String SERVICE_NAME = "MailingService";

    /**
     * Schema namespace: defined in WSDL
     */
    private String schemaNS;

    /**
     * QName for the service
     */
    private final QName SERVICE_QNAME;

    /**
     * By default do not use MTOM since there is a bug in JAXB library: see:
     * https://jaxb.dev.java.net/issues/show_bug.cgi?id=588
     */
    private boolean useMtom = false;

    public ClientSetup(String serviceNS, String schemaNS)
    {
      this.schemaNS = schemaNS;
      SERVICE_QNAME = new QName(serviceNS, SERVICE_NAME);
    }

    /**
     * Create service using give WSDL file and endpoint. If endpointAddress is null, the one from WSDL
     * will be used.
     *
     * @param wsdlFile        WSDL to be used
     * @param endpointAddress endpoint address if user wants to use something different from WSDL
     * @return StrongMail MailingService
     * @throws Exception any exception
     */
    public MailingService createService(File wsdlFile, String endpointAddress) throws Exception
    {
      URL wsdlURL = wsdlFile.toURI().toURL();

      // Create service with schema validation on
      MailingService_Service mss = new MailingService_Service(wsdlURL, SERVICE_QNAME);
      SchemaValidationFeature feature = new SchemaValidationFeature();
      MailingService svc = mss.getMailingServicePort(feature);

      // Set address either from WSDL or from user input
      String address;
      if (endpointAddress != null)
      {
        address = endpointAddress;
      }
      else
      {
        Endpoint endpoint = ClientProxy.getClient(svc).getEndpoint();
        address = endpoint.getEndpointInfo().getAddress();
      }

      // Set endpoint
      ((BindingProvider)svc).getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY,
                                                     address);

      // Set MTOM on/off
      Binding binding = ((BindingProvider)svc).getBinding();
      ((SOAPBinding)binding).setMTOMEnabled(useMtom);

      return svc;
    }

    /**
     * Use interceptor to set username, password, organizationName, and optionally subOrganizationId.
     *
     * @param svc               MailingService client
     * @param username          username to login
     * @param password          password for the user
     * @param organizationName  organization name to login
     * @param subOrganizationId sub-organization ID to login. Null for default root.
     * @param isSSO
     * @throws Exception any exception
     */
    public void applyInterceptor(MailingService svc,
                                 String username,
                                 String password,
                                 String organizationName,
                                 Long subOrganizationId,
                                 boolean isSSO) throws Exception
    {

      // WSSE standard username token
      Map<String, Object> outprops = new HashMap<String, Object>();
      outprops.put(WSHandlerConstants.ACTION, WSHandlerConstants.USERNAME_TOKEN);

      // Set username and password
      outprops.put(WSHandlerConstants.USER, username);
      outprops.put(WSHandlerConstants.PASSWORD_TYPE, WSConstants.PW_TEXT);
      outprops.put(WSHandlerConstants.PW_CALLBACK_REF, new ClientPasswordCallbackHandler(password));

      // Create client interceptor
      AuthenticationInterceptor authenticationInterceptor =
        new AuthenticationInterceptor(schemaNS, outprops, organizationName, subOrganizationId, isSSO);
      authenticationInterceptor.setAllowMTOM(useMtom);

      // Set client to use HTTPS with/without checking certificate
      ClientImpl cli = (ClientImpl)ClientProxy.getClient(svc);

      // Use HTTPS and validate certificate
      useHttpsWithCertificate(cli);

      // Set interceptor
      Endpoint endpoint = cli.getEndpoint();
      endpoint.getOutInterceptors().add(authenticationInterceptor);
    }

    /**
     * Use HTTPS protocol and check certificate. Read key store, password from system properties.
     *
     * @param client web service client
     * @throws Exception any exception
     */
    private void useHttpsWithCertificate(Client client) throws Exception
    {
      String keystore = System.getProperty("javax.net.ssl.trustStore");
      String keystorePassword = System.getProperty(
        "javax.net.ssl.trustStorePassword");

      KeyStore ks = KeyStore.getInstance("JKS");
      ks.load(new FileInputStream(keystore), keystorePassword.toCharArray());

      TrustManagerFactory tmf = TrustManagerFactory.getInstance(
        TrustManagerFactory.getDefaultAlgorithm());
      tmf.init(ks);
      TrustManager[] tms = tmf.getTrustManagers();

      TLSClientParameters param = new TLSClientParameters();
      param.setSecureSocketProtocol("TLS");
      param.setTrustManagers(tms);

      HTTPConduit https = (HTTPConduit)client.getConduit();
      https.setTlsClientParameters(param);
    }

    /**
     * Use HTTPS protocol and accept all certificates.
     *
     * @param client web service client
     * @throws Exception any exception
     */
    private void useHttpsIgnoreCertificate(Client client) throws Exception
    {
      TrustManager[] trustAllCerts = new TrustManager[]{
        new X509TrustManager()
        {
          public X509Certificate[] getAcceptedIssuers()
          {
            return null;
          }

          public void checkClientTrusted(X509Certificate[] certs,
                                         String authType)
          {
            // No implementation
          }

          public void checkServerTrusted(X509Certificate[] certs,
                                         String authType)
          {
            // No implementation
          }
        }
      };

      TLSClientParameters param = new TLSClientParameters();
      param.setSecureSocketProtocol("TLS");
      param.setTrustManagers(trustAllCerts);
      param.setDisableCNCheck(true);

      HTTPConduit https = (HTTPConduit)client.getConduit();
      https.setTlsClientParameters(param);
    }
  }

}

