<?php
/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2015 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

require_once "SampleHelper.php";
require_once "./generated/MailingService.php";

class ContentBlockSample
{
  private $service;
  private $helper;

  public function __construct($service)
  {
    $this->service = $service;
    $this->helper = new SampleHelper($service);
  }

  /**
   * Create a content block.
   *
   * @param contentBlockName the name of the content block
   * @return contentBlockId
   * @throws Exception any exception
   */
  public function createContentBlock($contentBlockName)
  {
    $contentBlock = new ContentBlock();

    $contentBlock->description = "ContentBlock from PHP";
    $contentBlock->content = "hiking {\n" . "  I love hiking.\n" . "}\n" .
                            "scuba {\n" . "  I love SCUBA diving.\n" . "}";
    $contentBlock->name = $contentBlockName;

    // Make call
    $response = $this->helper->create($contentBlock);
    return $response->objectId;
  }

  /**
   * Get and print contentBlock object to standard out.
   *
   * @param contentBlockId the contentBlock ID that will be retrieved and printed.
   * @throws Exception any exception
   */
  public function getAndPrint($contentBlockId)
  {
    $contentBlocks = $this->helper->get(array(SampleHelper::getSoapEntityIdObject(new ContentBlockId(),
										$contentBlockId->id,
										"ContentBlockId")));
    $contentBlock = $contentBlocks[0];

    // Print ContentBlock to standard out
    print("\nContentBlock:" . "\n");
    print("ContentBlockId: " . $contentBlock->objectId->id . "\n");
    print("Name: " . $contentBlock->name . "\n");
    print("CampaignId: " . $contentBlock->campaignId . "\n");
    print("CreatedTime: " . $contentBlock->createdTime . "\n");
    print("Description: " . $contentBlock->description . "\n");
    print("ModifiedTime: " . $contentBlock->modifiedTime . "\n");
    print("OrganizationId: " . $contentBlock->organizationId->id . "\n");
    print("OwnerId: " . $contentBlock->ownerId->id . "\n");
    print("Content: " . $contentBlock->content . "\n");
    print("Size: " . $contentBlock->size . "\n");
  }
}
?>
