<?php
/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2015 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */


require_once "SampleHelper.php";
require_once "AttachmentSample.php";
require_once "ContentBlockSample.php";
require_once "SeedListSample.php";
require_once "SuppressionListSample.php";
require_once "SystemAddressSample.php";
require_once "TargetSample.php";
require_once "TemplateSample.php";

require_once "./generated/MailingService.php";

class MailingSample
{
  private $service;
  private $helper;
  private $bounceId;
  private $fromId;
  private $replyId;
  private $targetSample;
  private $seedListSample;
  private $suppressionListSample;
  private $templateSample;
  private $attachmentSample;
  private $contentBlockSample;

  public function __construct($service)
  {
    $this->service = $service;
    $this->helper = new SampleHelper($service);
    $this->targetSample = new TargetSample($service);
    $this->seedListSample = new SeedListSample($service);
    $this->suppressionListSample = new SuppressionListSample($service);
    $this->templateSample = new TemplateSample($service);
    $this->attachmentSample = new AttachmentSample($service);
    $this->contentBlockSample = new ContentBlockSample($service);

    $systemAddressSample = new SystemAddressSample($service);
    $bounce = $systemAddressSample->listByEmailAddress("BOUNCE", "bounce@sample.com");
    $from = $systemAddressSample->listByEmailAddress("FROM", "from@sample.com");
    $reply = $systemAddressSample->listByEmailAddress("REPLY", "reply@sample.com");

    $this->bounceId = $bounce->objectId;
    $this->fromId = $from->objectId;
    $this->replyId = $reply->objectId;
  }

  /**
   * Create a recurring mailing with no excluded target.
   *
   * @param mailingName    the name of the mailing to be created
   * @return MailingId
   * @throws Exception any exception
   */
  public function createStandardMailing($mailingName)
  {
    $mailing = new StandardMailing();
    $templateId = $this->templateSample->createTemplate("template_php_" .$mailingName);
    $this->setMailingCommonFields($mailing, $mailingName, $templateId);

    $mailing->bounceAddressId = $this->bounceId;
    $mailing->fromAddressId = $this->fromId;
    $mailing->replyAddressId = $this->replyId;
    $mailing->fromName = "PHP from name";
    $mailing->subject = "mailing subject";
    $mailing->format = array("HTML");
    $mailing->header = array ("mailing header");

    $mailing->description = "StandardMailing from PHP";
    $mailing->eliminateDuplicates = true;
    $mailing->type = "RECURRING";

    // Set schedule: daily recurrence with every two days
    $daily = new dailyRecurrence();
    $daily->interval = 2;

    $recurrence = new recurrence();
    $recurrence->endAfterXMailings = 100; // launch 100 mailing;
    $recurrence->dailyRecurrence = $daily;

    $schedule = new schedule();
    $schedule->recurrence = $recurrence;
    $schedule->startDateTime = time() + 60 * 60;
    $mailing->schedule = $schedule;

    // Set dependencies
    $mailing->attachmentId = array($this->attachmentSample->createAttachment("attachment_php_" . $mailingName));
    $mailing->contentBlockId = array($this->contentBlockSample->createContentBlock("content_block_php_" . $mailingName));
    $mailing->includedTargetId = array($this->targetSample->createTarget("target_php_" . $mailingName));
    $mailing->seedListId = array($this->seedListSample->createSeedList("seed_php_" . $mailingName));
    $mailing->suppressionListId = array($this->suppressionListSample->createSuppressionList("suppress_php_" . $mailingName));

    // Set mailing notifications                                                                
    $mailingNotification = new StandardMailingNotification();
    $mailingNotification->recipientAddresses = array("test123@strongmail-test.com", "test2@strongmail-test.com");
    $mailingNotification->notifyStateChange = true;
    $mailingNotification->notifyTargetingCompletion = false;

    $mailing->notificationOption = $mailingNotification;

    $mailing->ccAddress = "cc@strongmail-test.com";
    $mailing->bccAddress = "bcc@strongmail-test.com";

    // Make call
    $response = $this->helper->create($mailing);
    return $response->objectId;
  }

  public function createTransactionalMailing($mailingName,
                                             $templateId,
                                             $targetId)
  {
    $mailing = new TransactionalMailing();
    $templateId = $this->templateSample->createTemplate("template_php_" .$mailingName);
    $this->setMailingCommonFields($mailing, $mailingName, $templateId);

    $mailing->bounceAddressId = $this->bounceId;
    $mailing->fromAddressId = $this->fromId;
    $mailing->replyAddressId = $this->replyId;
    $mailing->fromName = "PHP from name";
    $mailing->subject = "mailing subject";
    $mailing->format = array("HTML");
    $mailing->header = array ("mailing header");
    $mailing->description = "TransactionalMailing from PHP";
    $mailing->type = "TRANSACTIONAL";
    $mailing->targetId = $targetId;
    $response = $this->helper->create($mailing);
    return $response->objectId;
  }

  private function setMailingCommonFields($mailing,
                                          $mailingName,
                                          $templateId)
  {
    $mailing->bodyEncoding = "BASE_64";
    $mailing->fieldDelimiter = "::";
    $mailing->headerEncoding = "EIGHT_BIT";
    $mailing->isApproved = true;
    $mailing->isCompliant = true;
    $mailing->name = $mailingName;
    $mailing->outputBodyCharSet = "UTF-8";
    $mailing->outputHeaderCharSet = "ASCII";
    $mailing->priority = "NORMAL";
    $mailing->rowDelimiter = "\n";
    $mailing->subject = "mailing subject";
    $mailing->templateId = $templateId;
    // Set LETI interval
    $mailing->linkExpiryInterval = "fortyeight_hours";
    // Add tags to mailing
    $mailing->tagValueId = array($this->addTagToMailing());
  }

  private function addTagToMailing()
  {
    $filter = new TagFilter();
    $filter->isAscendingSpecified = false;
    $filter->maxRecordsPerPageSpecified = false;
    $filter->pageNumberSpecified = false;
    $filter->recordsPerPageSpecified = false;

    $request = new ListRequest();

    //Set the filter to request
    $request = new ListRequest();
    $request->filter = $filter;

    $response = $this->service->_list($request);
    $tagId = $response->objectId[0];
    $tags = $this->helper->get(array($tagId));
    return $tags[0]->tagValue[0]->tagValueId;
  }


  public function getAndPrint($mailingId)
  {
    $mailings = $this->helper->get(array(SampleHelper::getSoapEntityIdObject(new MailingId(),
                                                                                $mailingId->id,
                                                                                "MailingId")));
    $mailing = $mailings[0];

    // Print Mailing to standard out
    if ($mailingId instanceof StandardMailingId)
    {
      print("\nStandard Mailing" . "\n");
    }
    else if ($mailingId instanceof TransactionalMailingId)
    {
      print("\nTransactional Mailing" . "\n");
    }
    print("MailingId: " . $mailing->objectId->id . "\n");
    print("Name: " . $mailing->name . "\n");
    print("CampaignId: " . $mailing->campaignId . "\n");
    print("CreatedTime: " . $mailing->createdTime . "\n");
    print("Description: " . $mailing->description . "\n");
    print("ModifiedTime: " . $mailing->modifiedTime . "\n");
    print("OrganizationId: " . $mailing->organizationId->id . "\n");
    print("OwnerId: " . $mailing->ownerId->id . "\n");
    print("FieldDelimiter: " . $mailing->fieldDelimiter . "\n");
    print("RowDelimiter: " . $mailing->rowDelimiter . "\n");
    print("MessageFormats: " . SampleHelper::formatStringList($mailing->format) . "\n");
    print("FromName: " . $mailing->fromName . "\n");
    print("Headers: " . SampleHelper::formatStringList($mailing->header) . "\n");
    print("Template: " . $mailing->templateId->id . "\n");
    print("Bounce: " . $mailing->bounceAddressId->id . "\n");
    print("From: " . $mailing->fromAddressId->id . "\n");
    print("Reply: " . $mailing->replyAddressId->id . "\n");
    print("BodyEncoding: " . $mailing->bodyEncoding . "\n");
    print("HeaderEncoding: " . $mailing->headerEncoding . "\n");
    print("LastGoodStatus: " . $mailing->lastGoodStatus . "\n");
    print("MailingClass: " . $mailing->mailingClassId->id . "\n");
    print("BodyCharset: " . $mailing->outputBodyCharSet . "\n");
    print("BodyOutputToken: " . $mailing->outputBodyCharSetToken . "\n");
    print("HeaderCharset: " . $mailing->outputHeaderCharSet . "\n");
    print("HeaderOutputToken: " . $mailing->outputHeaderCharSetToken . "\n");
    print("Parent: " . $mailing->parentId . "\n");
    print("PlannedLaunchDate: " . $mailing->plannedLaunchDate . "\n");
    print("Priority: " . $mailing->priority . "\n");
    print("ServerErrorCode: " . $mailing->serverErrorCode . "\n");
    print("ServerErrorMessage: " . $mailing->serverErrorMessage . "\n");
    print("Status: " . $mailing->status . "\n");
    print("Subject: " . $mailing->subject . "\n");
    print("Type: " . $mailing->type . "\n");
    print("LETI: " . $mailing->linkExpiryInterval . "\n") ;

    if ($mailing instanceof StandardMailing)
    {
      print("CC address: " . $mailing->ccAddress . "\n") ;
      print("BCC address: " . $mailing->bccAddress . "\n") ;
      print("TagIds: " . SampleHelper::formatIdList($mailing->tagValueId) . "\n");
      print("IncludedTarget: " . SampleHelper::formatIdList($mailing->includedTargetId) . "\n");
      print("SeedList: " . SampleHelper::formatIdList($mailing->seedListId) . "\n");
      print("SuppressionList: " . SampleHelper::formatIdList($mailing->suppressionListId) . "\n");
      print("AttachmentId: " . SampleHelper::formatIdList($mailing->attachmentId) . "\n");
      print("ContentBlockId: " . SampleHelper::formatIdList($mailing->contentBlockId) . "\n");
    }
    else if ($mailing instanceof TransactionalMailing)
    {
      print("FormatField: " . $mailing->formatFieldName . "\n");
      print("MailingConfig: " . $mailing->mailingConfig . "\n");
      print("RecordStructure: " . SampleHelper::formatStringList($mailing->recordStructure) . "\n");
      print("SenderNumber: " . $mailing->senderNumber . "\n");
      print("Target: " . $mailing->targetId->id . "\n");
    }
  }

  /**
   * Launch a Standard Mailing.
   *
   * @param mailingId the ID of Standard Mailing that will be launched
   * @return if launch operation is successful
   * @throws Exception any exception
   */
  public function launchMailing($mailingId)
  {
    $request = new LaunchRequest();
    $request->mailingId = SampleHelper::getSoapEntityIdObject(new StandardMailingId(),
                                                              $mailingId->id,
                                                              "StandardMailingId");

    $response = $this->service->launch($request);
    return $response->success;
  }

  /**
   * Archive a Standard Mailing.
   *
   * @param mailingId the ID of Standard Mailing that will be archived
   * @return if archive operation is successful
   * @throws Exception any exception
   */
  public function archiveMailing($mailingId)
  {
    $request = new ArchiveRequest();
    $request->mailingId = SampleHelper::getSoapEntityIdObject(new StandardMailingId(),
                                                              $mailingId->id,
                                                              "StandardMailingId");

    $response = $this->service->archive($request);
    return $response->success;
  }

 /**
   * Schedule a Standard Mailing.
   *
   * @param mailingId the ID of Standard Mailing that will be scheduled
   * @return if schedule operation is successful
   * @throws Exception any exception
   */
  public function scheduleMailing($mailingId)
  {
    $request = new ScheduleRequest();
    $request->mailingId = SampleHelper::getSoapEntityIdObject(new StandardMailingId(),
                                                              $mailingId->id,
                                                              "StandardMailingId");

    $response = $this->service->schedule($request);
    return $response->success;
  }

  /**
   * Load a Transactional Mailing.
   *
   * @param mailingId the ID of Transactional Mailing that will be loaded
   * @return if load operation is successful
   * @throws Exception any exception
   */
  public function loadMailing($mailingId)
  {
    $request = new LoadRequest();
    $request->mailingId = SampleHelper::getSoapEntityIdObject(new TransactionalMailingId(),
                                                              $mailingId->id,
                                                              "TransactionalMailingId");

    $response = $this->service->load($request);
    return $response->success;
  }

  /**
   * Send one record for Transactional Mailing.
   *
   * @param mailingId the ID of Transactional Mailing whose data will be sent
   * @return if send operation is successful
   * @throws Exception any exception
   */
  /*public function sendOneRecord($mailingId)
  {
    // Create request and set mailingId
    $request = new SendRequest();
    $request->mailingId = $mailingId;

    // Create record based on the target created in TargetSample
    $record = new SendRecord();

    $pair1 = new NameValuePair();
    $pair1->name = "ID";
    $pair1->value = "10";
    $record->field = array();
    $record->field[] = $pair1;

    $pair2 = new NameValuePair();
    $pair2->name = "EMAIL_ADDRESS";
    $pair2->value = "sample1@php.sample.com";
    $record->field[] = $pair2;

    $pair3 = new NameValuePair();
    $pair3->name = "NAME";
    $pair3->value = "First Last";
    $record->field[] = $pair3;

    // Add record and make call
    $request->sendRecord = array();
    $request->sendRecord[] = $record;
    $response = $this->service->send($request);
    return $response->success;
  }
*/
  /**
   * Send data using strings.
   *
   * @param mailingId the ID of Transactional Mailing whose data will be sent
   * @return if send operation is successful
   * @throws Exception any exception
   */
 /* public function sendData($mailingId)
  {
    $mailings = $this->helper->get($mailingId);
    $mailing = $mailings[0];

    // Get delimiter from mailing and use it for data string
    $fieldDelimiter = $mailing->fieldDelimiter;
    $rowDelimiter = $mailing->rowDelimiter;

    // Create request and set data: data is rows of strings separated by field delimiter.
    $request = new SendRequest();
    $request->mailingId = SampleHelper::getSoapEntityIdObject(new TransactionalMailingId(),
                                                              $mailingId->id,
                                                              "TransactionalMailingId");
    $request->sendData = ("20" . $fieldDelimiter .
                          "sample2@php.sample.com" . $fieldDelimiter .
                          "FirstName LastName" . $rowDelimiter);

    $response = $this->service->send($request);
    return $response->success;
  }
*/

  /**
   * Get info for a loaded Tx Mailing.
   * @param mailingId the ID of Tx Mailing
   * @throws Exception any exception
   */
  public function getTxnEASInfo($mailingId)
  {
    $request = new GetTxnEasInfoRequest();
    $request->mailingId = SampleHelper::getSoapEntityIdObject(new TransactionalMailingId(),
                                                              $mailingId->id,
                                                              "TransactionalMailingId");

    $response = $this->service->getTxnEasInfo($request);
    print ("Hostname: " . $response->hostname . "\n");
    print ("Mailing Name: " . $response->mailingName . "\n");
    print ("Mailing Handle: " . $response->mailingHandle . "\n");
  }

  /**
   * Get mailing handle for a loaded Tx Mailing.
   * This handle is used in send API.
   * @param mailingId the ID of Tx Mailing
   * @throws Exception any exception
   */
  public function getTxnHandle($mailingId)
  {
    $request = new GetTxnMailingHandleRequest();
    $request->mailingId = SampleHelper::getSoapEntityIdObject(new TransactionalMailingId(),
                                                              $mailingId->id,
                                                              "TransactionalMailingId");

    $response = $this->service->getTxnMailingHandle($request);
    return $response->handle;
  }

  /**
   * Send records for Transactional Mailing.
   *
   * @param mailingId the ID of Transactional Mailing whose data will be sent
   * @return if send operation is successful
   * @throws Exception any exception
   */
  public function txnSend($mailingHandle)
  {
    // Create request and set mailingId
    $request = new TxnSendRequest();
    $request->handle = $mailingHandle;

    // Create record based on the target created in TargetSample
    $record = new SendRecord();

    $pair1 = new NameValuePair();
    $pair1->name = "ID";
    $pair1->value = "10";
    $record->field = array();
    $record->field[] = $pair1;

    $pair2 = new NameValuePair();
    $pair2->name = "EMAIL_ADDRESS";
    $pair2->value = "sample1@php.sample.com";
    $record->field[] = $pair2;

    $pair3 = new NameValuePair();
    $pair3->name = "NAME";
    $pair3->value = "First Last";
    $record->field[] = $pair3;

    // Add record and make call
    $request->sendRecord = array();
    $request->sendRecord[] = $record;
    $response = $this->service->txnSend($request);
    return $response->success;
  }



  /**
   * Close a Transactional Mailing.
   *
   * @param mailingId the ID of Transactional Mailing that will be closeed
   * @return if close operation is successful
   * @throws Exception any exception
   */
  public function close($mailingId)
  {
    $request = new CloseRequest();
    $request->mailingId = SampleHelper::getSoapEntityIdObject(new TransactionalMailingId(),
                                                              $mailingId->id,
                                                              "TransactionalMailingId");

    $response = $this->service->close($request);
    return $response->success;
  }

  /**
   * List mailings by name: this should return one mailing if name is matched.
   *
   * @param name the mailing name to be listed
   * @throws Exception any exception
   */
  public function listByName($name)
  {
    $filter = new MailingFilter();
    $helper->filter = $filter;
    $filter->orderBy = array();
    $filter->orderby[] = "MODIFIED_TIME";

    // Create name condition
    $nameCondition = new ScalarStringFilterCondition();
    $nameCondition->operator = "EQUAL";
    $nameCondition->value = $name;
    $filter->nameCondition = $nameCondition;

    // Set the filter to request
    $request = new ListRequest();
    $request->filter = $filter;

    // Make call and print results
    $response = $this->service->_list($request);

    $this->printListResponse($response->objectId);
  }

  /**
   * List mailing in an organization: this can be done using an empty filter. The organization is
   * the one from header.
   *
   * @throws Exception any exception
   */
  public function listByOrganization()
  {
    // Create empty filter: no conditions
    $filter = new MailingFilter();
    $helper->filter = $filter;
    $filter->orderBy = array();
    $filter->orderby[] = "MODIFIED_TIME";

    // Set the filter to request
    $request = new ListRequest();
    $request->filter = $filter;

    // Make call and print results
    $response = $this->service->_list($request);

    $this->printListResponse($response->objectId);
  }

  /**
   * List mailings by mailing type and status.
   *
   * @throws Exception any exception
   */
  public function listByTypeAndStatus()
  {
    // Create a mailing filter and set attribute and order by list
    $filter = new MailingFilter();
    $helper->filter = $filter;
    $filter->orderBy = array();
    $filter->orderby[] = "NAME";
    $filter->orderby[] = "MODIFIED_TIME";

    // Create status condition: status in (ACTIVE, EDITING, COMPLETED)
    $statusCondition = new ArrayStringFilterCondition();
    $statusCondition->operator = "IN";
    $statusCondition->value = array();
    $statusCondition->value[] = "ACTIVE";
    $statusCondition->value[] = "EDITING";
    $statusCondition->value[] = "COMPLETED";
    $filter->statusCondition = $statusCondition;

    // Create Type Condition: type in (RECURRING, TRANSACTIONAL)
    $typeCondition = new ArrayStringFilterCondition();
    $typeCondition->operator = "IN";
    $typeCondition->value = array();
    $typeCondition->value[] = "RECURRING";
    $typeCondition->value[] = "TRANSACTIONAL";
    $filter->typeCondition = $typeCondition;

    // Set the filter to request
    $request = new ListRequest();
    $request->filter = $filter;

    // Make call and print results
    $response = $this->service->_list($request);

    $this->printListResponse($response->objectId);
  }

  private function printListResponse($mailingIds)
  {
    print("\nList Response:" . "\n");
    foreach ($mailingIds as $each)
    {
      $mailings = $this->helper->get(array(SampleHelper::getSoapEntityIdObject(new MailingId(),
                                                                               $each->id,
                                                                               "MailingId")));
      $mailing = $mailings[0];
      print("\t" . $each->id . " " . $mailing->name . "\n");
    }
  }

  public function getMailingStatus($mailingId)
  {
    $mailings = $this->helper->get($mailingId);
    $mailing = $mailings[0];

    return $mailing->status;
  }

  public function waitForStatus($mailingId, $desiredStatus)
  {
    // Return immediately if desired status is received, or if any permanent
    // status is encountered.
    while (($status = $this->getMailingStatus($mailingId)) != $desiredStatus &&
           $status != "ERROR" &&
           $status != "FAILED" &&
           $status != "COMPLETED" &&
           $status != "CANCELED" &&
           $status != "SPEC_CANCELED")
    {
      print("waitForStatus on " . $mailingId->id . ": " . $status . "!=" . $desiredStatus . "\n");
      sleep(10);
    }
    print("waitForStatus on " . $mailingId->id . " final: " . $status .
          ($status == $desiredStatus ? "==" : "!=") . $desiredStatus . "\n");
    return $status;
  }
}

?>
