<?php
/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2015 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */


require_once "SampleHelper.php";
require_once "./generated/MailingService.php";

class OrganizationSample
{
  private $service;
  private $helper;

  public function __construct($service)
  {
    $this->service = $service;
    $this->helper = new SampleHelper($service);
  }

  public function createOrganization($orgName, $logoFile)
  {
    $organization = new Organization();

    $organization->name = $orgName;

    $parentOrgId = new OrganizationId();
    $parentOrgId->id = "1";
    $organization->parentId = $parentOrgId;

    $organization->logo = file_get_contents($logoFile);

    $organization->viewInBrowserExceptionURL = "http://www.strongmail.com/vib/error.html";
    $organization->forward2FriendExceptionURL = "http://www.strongmail.com/ftaf/error.html";
    $organization->socialNotesExceptionURL = "http://www.strongmail.com/share/error.html";
    $organization->forward2FriendOffer = "Some html offer...";
    $organization->forward2FriendOfferTrackingOption = "NORMAL";

    // Make call
    $response = $this->helper->create($organization);
    return $response->objectId;
  }

  /**
   * Get and print organization object to standard out.
   *
   * @param orgId the organization ID that will be retrieved and printed.
   * @throws Exception any exception
   */
  public function getAndPrint($orgId)
  {
    $organizations = $this->helper->get(array(SampleHelper::getSoapEntityIdObject(new OrganizationId(),
										$orgId->id,
										"OrganizationId")));
    $organization = $organizations[0];

    // Print target to standard out
    print("\nOrganization:" . "\n");
    print("OrganizationId: " . $organization->objectId->id . "\n");
    print("Name: " . $organization->name . "\n");
    print("Parent ID: " . $organization->parentId->id . "\n");
    print("VIB Error URL: " . $organization->viewInBrowserExceptionURL . "\n");
    print("FTAF Error URL: " . $organization->forward2FriendExceptionURL . "\n");
    print("Share Error URL: " . $organization->socialNotesExceptionURL . "\n");
    print("FTAF Offer: " . $organization->forward2FriendOffer . "\n");
    print("FTAF Offer Tracking Option: " . $organization->forward2FriendOfferTrackingOption . "\n");
  }
}
?>
