<?php
/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2015 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

require_once "SecurityHeader.php";
require_once "SampleHelper.php";
require_once "AttachmentSample.php";
require_once "ContentBlockSample.php";
require_once "DataSourceSample.php";
require_once "MailingSample.php";
require_once "RuleSample.php";
require_once "SeedListSample.php";
require_once "SuppressionListSample.php";
require_once "SystemAddressSample.php";
require_once "TargetSample.php";
require_once "TemplateSample.php";
require_once "OrganizationSample.php";
require_once "UserSample.php";
require_once "RoleSample.php";
require_once "AssignedRoleSample.php";
require_once "StrongtoolSample.php";
require_once "MediaServerSample.php";
require_once "MailingClassSample.php";
require_once "TagSample.php";

require_once "./generated/MailingService.php";

class SampleProgram
{
  private $orgSample;
  private $userSample;
  private $roleSample;
  private $assignedRoleSample;
  private $strongtoolSample;
  private $mediaServerSample;
  private $mailingClassSample;
  private $dsSample;
  private $targetSample;
  private $seedListSample;
  private $suppressionListSample;
  private $templateSample;
  private $attachmentSample;
  private $contentBlockSample;
  private $ruleSample;
  private $mailingSample;
  private $tagSample;

  public function __construct($svc)
  {
    $this->service = $svc;

    $securityHeader = new SecurityHeader();
    // Use this instead if a sub-organization is being used.  Substitute the id of your sub-org for 101.
    //$securityHeader->setSecurityHeader($this->service, 'admin', 'admin', 'admin', '101');
    // Use this instead for enabling SSO token authentication. The password argument should contain the token, and the last argument must be 'true'.   
    //$securityHeader->setSecurityHeader($this->service, 'admin', <token>, 'admin', null, true);
    $securityHeader->setSecurityHeader($this->service, 'admin', 'admin', 'admin');


    $this->orgSample = new OrganizationSample($svc);
    $this->userSample = new UserSample($svc);
    $this->roleSample = new RoleSample($svc);
    $this->assignedRoleSample = new AssignedRoleSample($svc);
    $this->strongtoolSample = new StrongtoolSample($svc);
    $this->mediaServerSample = new MediaServerSample($svc);
    $this->mailingClassSample = new MailingClassSample($svc);
    $this->dsSample = new DataSourceSample($svc);
    $this->targetSample = new TargetSample($svc);
    $this->seedListSample = new SeedListSample($svc);
    $this->suppressionListSample = new SuppressionListSample($svc);
    $this->templateSample = new TemplateSample($svc);
    $this->attachmentSample = new AttachmentSample($svc);
    $this->contentBlockSample = new ContentBlockSample($svc);
    $this->ruleSample = new RuleSample($svc);
    $this->mailingSample = new MailingSample($svc);
    $this->tagSample = new TagSample($svc);

  }

  public function createAllData($timestamp)
  {
    $objectIds = array();

    $objectIds[] = $this->orgSample->createOrganization("org_php_" . $timestamp, GlobalVariables::$attachment_file);

    $objectIds[] = $this->userSample->createUser("user_php_" . $timestamp);

    $objectIds[] = $this->roleSample->createRole("role_php_" . $timestamp);

    $objectIds[] = $this->assignedRoleSample->createAssignedRole($this->getObjectId($objectIds, UserId),
                                                                 $this->getObjectId($objectIds, OrganizationId),
                                                                 $this->getObjectId($objectIds, RoleId));

    $objectIds[] = $this->strongtoolSample->createStrongtool("strongtool_php_" . $timestamp);

    $objectIds[] = $this->mediaServerSample->createMediaServer("mediaServer_php_" . $timestamp);

    $objectIds[] = $this->mailingClassSample->createMailingClass("mailingClass_php_" . $timestamp,
                                                                 $this->getObjectId($objectIds, OrganizationId));

    $objectIds[] = $this->dsSample->createIDS("ids_php_" . $timestamp);

    // EDS: need to change DataSourceSample to point to your database
    //$objectIds[] = $this->dsSample->createEDS("eds_php_" . $timestamp);

    $objectIds[] = $this->targetSample->createTarget("target_php_" . $timestamp);

    $objectIds[] = $this->seedListSample->createSeedList("seedList_php_" . $timestamp);

    $objectIds[] = $this->suppressionListSample->createSuppressionList("suppressionList_php_" . $timestamp);

    $objectIds[] = $this->attachmentSample->createAttachment("attachment_php_" . $timestamp);

    $objectIds[] = $this->contentBlockSample->createContentBlock("contentBlock_php_" . $timestamp);

    $objectIds[] = $this->ruleSample->createRule("rule_php_" . $timestamp, $this->getObjectId($objectIds, TargetId));

    $objectIds[] = $this->templateSample->createTemplate("template_php_" . $timestamp);

    $objectIds[] = $this->tagSample->createTag("tag_php_" . $timestamp);

    $objectIds[]  = $this->mailingSample->createStandardMailing(
      "stdmailing_php_" . $timestamp,
      $this->getObjectId($objectIds, TargetId),
      $this->getObjectId($objectIds, SeedListId),
      $this->getObjectId($objectIds, SuppressionListId),
      $this->getObjectId($objectIds, AttachmentId),
      $this->getObjectId($objectIds, ContentBlockId),
      $this->getObjectId($objectIds, TemplateId)
    );

    $objectIds[] = $this->mailingSample->createTransactionalMailing(
      "txmailing_php_" . $timestamp,
      $this->getObjectId($objectIds, TemplateId),
      $this->getObjectId($objectIds, TargetId)
    );

    $this->getAndPrintAllData($objectIds);
    return $objectIds;
  }

  public function getAndPrintAllData($objectIds)
  {
    foreach ($objectIds as $objectId)
    {
      if ($objectId instanceof OrganizationId)
      {
        $this->orgSample->getAndPrint($objectId);
      }
      else if ($objectId instanceof UserId)
      {
        $this->userSample->getAndPrint($objectId);
      }
      else if ($objectId instanceof RoleId)
      {
        $this->roleSample->getAndPrint($objectId);
      }
      else if ($objectId instanceof AssignedRoleId)
      {
        $this->assignedRoleSample->getAndPrint($objectId);
      }
      else if ($objectId instanceof StrongtoolId)
      {
        $this->strongtoolSample->getAndPrint($objectId);
      }
      else if ($objectId instanceof MediaServerId)
      {
        $this->mediaServerSample->getAndPrint($objectId);
      }
      else if ($objectId instanceof MailingClassId)
      {
        $this->mailingClassSample->getAndPrint($objectId);
      }
      else if ($objectId instanceof DataSourceId)
      {
        $this->dsSample->getAndPrint($objectId);
      }
      else if ($objectId instanceof TargetId)
      {
        $this->targetSample->getAndPrint($objectId);
      }
      else if ($objectId instanceof SeedListId)
      {
        $this->seedListSample->getAndPrint($objectId);
      }
      else if ($objectId instanceof SuppressionListId)
      {
        $this->suppressionListSample->getAndPrint($objectId);
      }
      else if ($objectId instanceof AttachmentId)
      {
        $this->attachmentSample->getAndPrint($objectId);
      }
      else if ($objectId instanceof ContentBlockId)
      {
        $this->contentBlockSample->getAndPrint($objectId);
      }
      else if ($objectId instanceof RuleId)
      {
        $this->ruleSample->getAndPrint($objectId);
      }
      else if ($objectId instanceof TemplateId)
      {
        $this->templateSample->getAndPrint($objectId);
      }
      else if ($objectId instanceof MailingId)
      {
        $this->mailingSample->getAndPrint($objectId);
      }
      else if ($objectId instanceof TagId)
      {
        $this->tagSample->getAndPrint($objectId);
      }
    }
  }

  private function getObjectId($objectIds, $className)
  {
    foreach ($objectIds as $objectId)
    {
      if ($objectId instanceof $className)
      {
        return $objectId;
      }
    }

    return null;
  }

  public function main($argv)
  {
    $timestamp = $argv[3];
    $deleteAll = $argv[4];

    // Create all data
    $objectIds = $this->createAllData($timestamp);

    // Print out all data
    $this->getAndPrintAllData($objectIds);

    // Add data to IDS using file
    $dataSourceId = $this->getObjectId($objectIds, InternalDataSourceId);
    $this->dsSample->addRecordsUsingFile($dataSourceId);

    // Export data source records to a file
    $dataSourceExportFile = "./data_source_export.txt";
    $this->dsSample->exportRecords($dataSourceId, $dataSourceExportFile);

    // Update the data source with new name
    $this->dsSample->updateDataSourceName($dataSourceId, "ids_php_updated");

    // Get data source statistics
    $this->dsSample->getAndPrintStatistics($dataSourceId);

    // Copy the data source
    $copiedId = $this->dsSample->copyDataSource($dataSourceId, "ids_php_" . $timestamp . "_copied");
    print("\nCopied IDS: " . $copiedId->id . "\n");

    // Dedupe data source records and print out new statistics
    $this->dsSample->dedupeRecords($dataSourceId);
    $this->dsSample->getAndPrintStatistics($dataSourceId);

    // Remove data source records and print out new statistics
    $removedDS = $this->dsSample->removeRecordsInline($dataSourceId);
    print("\nRemoved records: " . $removedDS . "\n");
    $this->dsSample->getAndPrintStatistics($dataSourceId);

    // Import new content to attachment using IDS record file
    $attachmentId = $this->getObjectId($objectIds, AttachmentId);
    $this->attachmentSample->importContent($attachmentId, GlobalVariables::$ids_data_file);
    $this->attachmentSample->getAndPrint($attachmentId);

    // Remove records from seed list
    $seedId = $this->getObjectId($objectIds, SeedListId);
    $removedSeed = $this->seedListSample->removeRecordsInline($seedId);
    print("\nRemoved records: " . $removedSeed . "\n");
    $this->seedListSample->getAndPrint($seedId);

    // Remove records from suppression list
    $suppressId = $this->getObjectId($objectIds, SuppressionListId);
    $removedSuppress = $this->suppressionListSample->removeRecordsInline($suppressId);
    print("\nRemoved records: " . $removedSuppress . "\n");
    $this->suppressionListSample->getAndPrint($suppressId);

    // List mailing by name
    $this->mailingSample->listByName("stdmailing_php_" . $timestamp);

    // List mailing in an organization
    $this->mailingSample->listByOrganization();

    // List mailing by type and status
    $this->mailingSample->listByTypeAndStatus();


    print("\n");
    $stdMailingId = $this->getObjectId($objectIds, StandardMailingId);
    $txMailingId = $this->getObjectId($objectIds, TransactionalMailingId);

    // Schedule a standard mailing
    $scheduled = $this->mailingSample->scheduleMailing($stdMailingId);
    print("Schedule mailing " . $stdMailingId->id . ": " . ($scheduled? "true" : "false") . "\n");

    // Load a transactional mailing
    $loaded = $this->mailingSample->loadMailing($txMailingId);
    print("Load mailing " . $txMailingId->id . ": " . ($loaded? "true" : "false") . "\n");

    $this->mailingSample->waitForStatus($txMailingId, "ACTIVE");

    print("\n");
    // Get Tx mailing EAS Info.
    $this->mailingSample->getTxnEASInfo($txMailingId);

    // Get Tx mailing handle to be used in the send API. 
    $txMailingHandle = $this->mailingSample->getTxnHandle($txMailingId);
    $sent = $this->mailingSample->txnSend($txMailingHandle);
    print("Send record for " . $txMailingId->id . ": " . ($sent? "true" : "false") . "\n");
    
    // Commenting out send requests since they are deprecated
    // Send data using records
    // $sent = $this->mailingSample->sendOneRecord($txMailingId);
    // print("Send record for " . $txMailingId->id . ": " . ($sent? "true" : "false") . "\n");

    // Send data using string
    // $sent = $this->mailingSample->sendData($txMailingId);
    // print("Send data for " . $txMailingId->id . ": " . ($sent? "true" : "false") . "\n");

    // Close mailing
    $closed = $this->mailingSample->close($txMailingId);
    print("Close mailing " . $txMailingId->id . ": " . ($closed? "true" : "false") . "\n");

    // Teardown
    {
      if ("true" == $deleteAll)
      {
        // true=delete the data
        // sleep to 5 seconds to let mailing finish.
        try
        {
          sleep(5);
        }
        catch (Exception $e)
        {
          // ignore
        }

        // Delete all objects
        $helper = new SampleHelper($this->service);
        try
        {
          // Must wait for mailings to complete before they can be deleted
          //$this->mailingSample->waitForStatus($stdMailingId, "COMPLETED");
          $this->mailingSample->waitForStatus($txMailingId, "COMPLETED");
          // All assets which are created during sample run will be deleted.
          $helper->delete();
        }
        catch (Exception $e)
        {
            // ignore
        }


        // Delete copied data source
        if ($copiedId != null)
        {
          try
          {
            $helper->delete($copiedId);
          }
          catch (Exception $e)
          {
            // ignore
          }
        }
      }
    }
  }
}

GlobalVariables::$wsdl_url = $argv[1];
GlobalVariables::$schema_version = $argv[2];
GlobalVariables::$attachment_file = "../data/Shared_Attachment.gif";
GlobalVariables::$ids_data_file = "../data/Shared_DataSource_Records.txt";
$service = SampleHelper::getMailingService();
$samples = new SampleProgram($service);

$samples->main($argv);
?>
