<?php
/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2015 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

class SuppressionListSample
{
  private $service;
  private $helper;

  public function __construct($service)
  {
    $this->service = $service;
    $this->helper = new SampleHelper($service);
  }

  /**
   * Create a suppression list and add some data.
   *
   * @param suppressionListName the name of the suppression list
   * @return suppressionListId
   * @throws Exception any exception
   */
  public function createSuppressionList($suppressionListName)
  {
    $suppressionList = new SuppressionList();

    $suppressionList->description = "SuppressionList from PHP";
    $suppressionList->includeByDefaultOnMailings = true;
    $suppressionList->name = $suppressionListName;

    // Make call
    $response = $this->helper->create($suppressionList);
    $suppressionListId =  $response->objectId;

    // Add some records
    $this->addRecordsInline($suppressionListId);

    return $suppressionListId;
  }

  /**
   * Add 2 records to suppression list using inline records.
   *
   * @param suppressionListId the SuppressionListId that records will be added to.
   * @throws Exception any exception
   */
  public function addRecordsInline($suppressionListId)
  {
    $request = new AddSuppressionListRecordsRequest();
    $request->suppressionListId = SampleHelper::getSoapEntityIdObject(new SuppressionListId(),
                                                               $suppressionListId->id,
                                                               "SuppressionListId");
    $request->addressMatch = array();
    $request->addressMatch[] = "suppress.php.com";
    $request->addressMatch[] = "suppress@suppress.php.com";

    $this->service->addRecords($request);
  }

  /**
   * Remove one record that was created in addRecordsInline() from the SuppressionList.
   *
   * @param suppressionListId the SuppressionListId whose record will be removed.
   * @return the number of record that was removed.
   * @throws Exception any exception
   */
  public function removeRecordsInline($suppressionListId)
  {
    $request = new RemoveSuppressionListRecordsRequest();
    $request->suppressionListId = SampleHelper::getSoapEntityIdObject(new SuppressionListId(),
                                                               $suppressionListId->id,
                                                               "SuppressionListId");
    $request->addressMatch = array();
    $request->addressMatch[] = "suppress@suppress.php.com";

    $response = $this->service->removeRecords($request);
    return $response->recordsRemoved;
  }

  /**
   * Get and print suppressionList object to standard out.
   *
   * @param suppressionListId the suppressionList ID that will be retrieved and printed.
   * @throws Exception any exception
   */
  public function getAndPrint($suppressionListId)
  {
    $suppressionLists = $this->helper->get(array(SampleHelper::getSoapEntityIdObject(new SuppressionListId(),
                                                                                $suppressionListId->id,
                                                                                "SuppressionListId")));
    $suppressionList = $suppressionLists[0];

    // Print SuppressionList to standard out
    print("\nSuppressionList:" . "\n");
    print("SuppressionListId: " . $suppressionList->objectId->id . "\n");
    print("Name: " . $suppressionList->name . "\n");
    print("CampaignId: " . $suppressionList->campaignId . "\n");
    print("CreatedTime: " . $suppressionList->createdTime . "\n");
    print("Description: " . $suppressionList->description . "\n");
    print("ModifiedTime: " . $suppressionList->modifiedTime . "\n");
    print("OrganizationId: " . $suppressionList->organizationId->id . "\n");
    print("OwnerId: " . $suppressionList->ownerId->id . "\n");
    print("Total Records: " . $suppressionList->totalRecords . "\n");
  }
}

?>
