<?php
/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2015 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */


require_once "SampleHelper.php";
require_once "./generated/MailingService.php";

class SystemAddressSample
{
  private $service;
  private $helper;

  public function __construct($service)
  {
    $this->service = $service;
    $this->helper = new SampleHelper($service);
  }

  /**
   * List SystemAddress by type (bounce/from/reply) and email address. There is no filter by email
   * address, so we need to get all address for one type and filter them again.
   *
   * @param type         the SystemAddressType to filter on
   * @param emailAddress the email address to match
   * @return the SystemAddress that belongs to the type and matches the email address
   * @throws Exception any exception
   */
  public function listByEmailAddress($type, $emailAddress)
  {
    // Create and set attribute for filter
    $filter = new SystemAddressFilter();
    $this->helper->filter = $filter;

    // Create type condition
    $typeCondition = new ScalarStringFilterCondition();
    $typeCondition->operator = "EQUAL";

    // Set type condition value
    if ($type == "BOUNCE")
    {
      $typeCondition->value = "BOUNCE";
    }
    else if ($type == "FROM")
    {
      $typeCondition->value = "FROM";
    }
    else if ($type == "REPLY")
    {
      $typeCondition->value = "REPLY";
    }

    $filter->typeCondition = $typeCondition;

    // Create request and make call
    $request = new ListRequest();
    $request->filter = $filter;

    $response = $this->service->_list($request);
    $addressIds = $response->objectId;

    // Go through all addresses to get the one that matches email address.
    foreach ($addressIds as $each)
    {
      $addresses = $this->helper->get(array(SampleHelper::getSoapEntityIdObject(new SystemAddressId(),
                                                                                $each->id,
                                                                                "SystemAddressId")));
      $address = $addresses[0];

      if ($address->emailAddress == $emailAddress)
      {
        return $address;
      }
    }

    return null;
  }

  public function createAddress($email, $isBounce, $isFrom, $isReply)
  {
    $address = new SystemAddress();

    $address->emailAddress = $email;

    if ($isReply)
    {
      $address->isReply = true;
      $address->isBounce = false;
      $address->isFrom = false;
    }
    else if ($isFrom)
    {
      $address->isFrom = true;
      $address->fromName = "From Name";
      $address->isReply = false;
      $address->isBounce = false;
    }
    else if ($isBounce)
    {
      $address->isBounce = true;
      $address->isReply = false;
      $address->isFrom = false;
    }

    // Make call
    $response = $this->helper->create($address);
    return $response->objectId;
  }

  /**
   * Get and print system address object to standard out.
   *
   * @param addressId the address ID that will be retrieved and printed.
   * @throws Exception any exception
   */
  public function getAndPrint($addressId)
  {
    $addresses = $this->helper->get(array(SampleHelper::getSoapEntityIdObject(new SystemAddressId(),
										$addressId->id,
										"SystemAddressId")));
    $address = $addresses[0];

    // Print address to standard out
    print("\nAddress:" . "\n");
    print("AddressId: " . $address->objectId->id . "\n");
    print("Email: " . $address->emailAddress . "\n");
    print("Bounce flag: " . $address->isBounce . "\n");
    print("From flag: " . $address->isFrom . "\n");
    print("Reply flag: " . $address->isReply . "\n");
    print("From Name: " . $address->fromName . "\n");
    print("Organization ID: " . $address->organizationId->id . "\n");
  }
}

?>
