<?php
/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2015 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */


require_once "SampleHelper.php";
require_once "DataSourceSample.php";
require_once "./generated/MailingService.php";

class TargetSample
{
  private $service;
  private $helper;
  private $dataSourceSample;

  public function __construct($service)
  {
    $this->service = $service;
    $this->helper = new SampleHelper($service);
    $this->dataSourceSample = new DataSourceSample($service);
  }

  public function createTarget($targetName)
  {
    $target = new Target();

    // Fill in the target
    $target->name = $targetName;
    $dataSourceId = $this->dataSourceSample->createIDS("ids_php_" . $targetName);
    $target->dataSourceId = $dataSourceId;
    $target->type = "ADVANCED";
    $target->description = "Target from PHP";
    $target->emailAddressFieldName = "email_address";   // From the data source
    $target->bounceFieldName = "email_address_status";  // system generated based on email column
    $target->excludeBounce = true;
    $target->unsubscribeFieldName = "unsub";  // system generated
    $target->excludeUnsubscribe = true;

    // Wait for data source status to be idle before adding
    $this->helper->waitForIdle($dataSourceId);

    // Make call
    $response = $this->helper->create($target);
    return $response->objectId;
  }

  /**
   * Get and print target object to standard out.
   *
   * @param targetId the target ID that will be retrieved and printed.
   * @throws Exception any exception
   */
  public function getAndPrint($targetId)
  {
    $targets = $this->helper->get(array(SampleHelper::getSoapEntityIdObject(new TargetId(),
										$targetId->id,
										"TargetId")));
    $target = $targets[0];

    // Print target to standard out
    print("\nTarget:" . "\n");
    print("TargetId: " . $target->objectId->id . "\n");
    print("Name: " . $target->name . "\n");
    print("Type: " . $target->type . "\n");
    print("DataSource: " . $target->dataSourceId->id . "\n");
    print("EmailField: " . $target->emailAddressFieldName . "\n");
    print("SMSField: " . $target->smsAddressFieldName . "\n");
    print("BounceField: " . $target->bounceFieldName . "\n");
    print("UnsubField: " . $target->unsubscribeFieldName . "\n");
    print("Description: " . $target->description . "\n");
    print("Query: " . $target->advancedQuery . "\n");
    print("CampaignId: " . $target->campaignId . "\n");
    print("CreatedTime: " . $target->createdTime . "\n");
    print("ModifiedTime: " . $target->modifiedTime . "\n");
    print("OrganizationId: " . $target->organizationId->id . "\n");
    print("OwnerId: " . $target->ownerId->id . "\n");
    print("Total Records: " . $target->totalRecords . "\n");
  }
}
?>
