# Copyright 2014, StrongMail Systems, Inc.  All rights reserved.

class GlobalUndeliverableSample
  def initialize(svc)
    @svc = svc
  end

  # Get global undeliverable records based on recipient address search criteria
  # messageDestination is the search string of recipient address
  # channelType possible values are EMAIL or SMS
  # organizationId to get records for this organization
  def getByRecipientAddress(messageDestination, channelType, organizationId)
    # Create request
    request = GlobalUndeliverableRequest.new

    request.recipientAddress = messageDestination
    request.channelType = channelType
    request.pageNumber = 1
    request.recordsPerPage = 25

    organizationIdObj = OrganizationId.new
    organizationIdObj.id = organizationId
    request.organizationId = organizationIdObj

    # Make call
    printListResponse(@svc.getGlobalUndeliverable(request))
  end

  # Get global undeliverable records based on
  # failure code - search criteria
  # channelType possible values are EMAIL or SMS
  # organizationId to get records for this organization (optional)
  def getByFailureCode(failureCode, channelType, organizationId)
    # Create request
    request = GlobalUndeliverableRequest.new

    request.failureCode = failureCode
    request.channelType = channelType
    request.pageNumber = 1
    request.recordsPerPage = 25

    organizationIdObj = OrganizationId.new
    organizationIdObj.id = organizationId
    request.organizationId = organizationIdObj

    # Make call
    printListResponse(@svc.getGlobalUndeliverable(request))
  end

    # reactivate global undeliverable records
    # messageDestination is the search string of recipient address
    # channelType possible values are EMAIL or SMS
    # organizationId to get records for this organization
    # pageNumber to get the records in pagination
    # recordPerPage represent no. of records per page
    def reactivateGlobalUndeliverable(messageDestination, channelType)
      # Create request
      request = ReactivateGlobalUndeliverableRequest.new

      globalUndeliverableInfoObj = GlobalUndeliverableInfo.new
      globalUndeliverableInfoObj.recipientAddress = messageDestination
      globalUndeliverableInfoObj.channelType = channelType

      request.globalUndeliverableInfo = [globalUndeliverableInfoObj]

      # Make call
      return @svc.reactivateGlobalUndeliverable(request)
    end

    # Add global undeliverable example
    def addGlobalUndeliverables()
      # Add Global Undeliverable Request
      request = AddGlobalUndeliverableRequest.new

      infoObj1 = GlobalUndeliverableInfo.new
      infoObj1.recipientAddress = 'demo_email@undeliverable.com'
      infoObj1.channelType = 'EMAIL'
      infoObj1.failureCode = 2001

      infoObj2 = GlobalUndeliverableInfo.new
      infoObj2.recipientAddress = '9876543210'
      infoObj2.channelType = 'SMS'
      infoObj2.failureCode = 2103

      request.globalUndeliverableInfo = [infoObj2,infoObj2]

      # Make call
      puts "Adding " + infoObj1.recipientAddress + " and " + infoObj2.recipientAddress + " to global undeliverable."
      return @svc.addGlobalUndeliverable(request)
    end

   # Print out get response: recipient address and channelType
    def printListResponse(globalUndeliverables)
      puts "\Global Undeliverable Response: "
      if globalUndeliverables.length > 0
        puts "\tMessage Destination\tType\tFailure Code"
        for globalUndeliverableInfo in globalUndeliverables
            puts "\t" +globalUndeliverableInfo.recipientAddress + "\t" + globalUndeliverableInfo.channelType + "\t" +  globalUndeliverableInfo.failureCode.to_s
        end
      else
        puts "No matching records found"
      end
    end
end