# Copyright 2014, StrongMail Systems, Inc.  All rights reserved.

require 'SampleHelper'
require 'SampleHelper.rb'
require 'DataSourceSample'
require 'TargetSample'
require 'SeedListSample'
require 'SuppressionListSample'
require 'AttachmentSample'
require 'ContentBlockSample'
require 'RuleSample'
require 'TemplateSample'
require 'SystemAddressSample'

class MailingSample
  def initialize(svc)
    @svc = svc
  end


  # Create a recurring mailing with no excluded target. Use only one of instead of list of
  #  target, seed list, suppresion list, attachment and content blocks.
  #  mailingName is the name of the mailing.
  #  return the ID of created standard mailing.
  def createStandardMailing(mailingName)
    mailing = StandardMailing.new
    setMailingCommonFields(mailing, mailingName)

    # Set dependencies
    $contentBlockId = ContentBlockSample.new(@svc).createContentBlock("content_block_"+mailingName);
    mailing.contentBlockId = [$contentBlockId]

    $attachmentId = AttachmentSample.new(@svc).createAttachment("attachment_"+mailingName);
    mailing.attachmentId = [$attachmentId]

    $templateId = TemplateSample.new(@svc).createTemplate("template_"+mailingName)
    mailing.templateId = $templateId;

    $targetId = TargetSample.new(@svc).createTarget("target_"+mailingName)
    mailing.includedTargetId = [$targetId]
    mailing.subject = "mailing subject"
    mailing.format = ["HTML"]
    mailing.header = ["mailing header"]
    mailing.fromName = "From Name"
    mailing.forward2FriendOffer = ""
    mailing.forward2FriendOfferTrackingOption = "NONE"

    $seedListId = SeedListSample.new(@svc).createSeedList("seed_"+mailingName)
    mailing.seedListId = [$seedListId]

    $suppressionListId = SuppressionListSample.new(@svc).createSuppressionList("supp_"+mailingName)
    mailing.suppressionListId = [$suppressionListId]

    $bounceId = SystemAddressSample.new(@svc).listByEmailAddress("BOUNCE", "bounce@sample.com").objectId;
    $fromId = SystemAddressSample.new(@svc).listByEmailAddress("FROM", "from@sample.com").objectId
    $replyId = SystemAddressSample.new(@svc).listByEmailAddress("REPLY", "reply@sample.com").objectId

    mailing.bounceAddressId = $bounceId
    mailing.replyAddressId = $replyId
    mailing.fromAddressId = $fromId

    mailing.description = "StandardMailing from Ruby"
    mailing.eliminateDuplicates = true
    mailing.type = "RECURRING"
    mailing.useHeaderFromTemplate = true

    # Set schedule: daily recurrence with every two days
    daily = SchedulableMailing::Schedule::Recurrence::DailyRecurrence.new
    daily.interval = 2

    recurrence = SchedulableMailing::Schedule::Recurrence.new
    recurrence.endDate = Date.new
    recurrence.dailyRecurrence = daily

    schedule = SchedulableMailing::Schedule.new
    schedule.startDateTime = DateTime::civil(2020, 1, 11, 1, 1, 1)
    schedule.recurrence = recurrence
    mailing.schedule = schedule

    return SampleHelper::create(@svc, mailing)
  rescue => e
    puts "Error was occurred while creating a recurring standard mailing mailing #{e.message}"
    return nil
  end

# Create a one_time mailing with no excluded target.
  #  mailingName is the name of the mailing.
  #  return the ID of created standard mailing.
  def createOneTimeStandardMailing(mailingName)
    mailing = StandardMailing.new
    setMailingCommonFields(mailing, mailingName)

    # Set dependencies
    $contentBlockId = ContentBlockSample.new(@svc).createContentBlock("content_block_"+mailingName);
    mailing.contentBlockId = [$contentBlockId]

    $attachmentId = AttachmentSample.new(@svc).createAttachment("attachment_"+mailingName);
    mailing.attachmentId = [$attachmentId]

    $templateId = TemplateSample.new(@svc).createTemplate("template_"+mailingName)
    mailing.templateId = $templateId;

    $targetId = TargetSample.new(@svc).createTarget("target_"+mailingName)
    mailing.includedTargetId = [$targetId]
    mailing.subject = "mailing subject"
    mailing.format = ["HTML"]
    mailing.header = ["mailing header"]
    mailing.fromName = "From Name"
    mailing.forward2FriendOffer = ""
    mailing.forward2FriendOfferTrackingOption = "NONE"

    $seedListId = SeedListSample.new(@svc).createSeedList("seed_"+mailingName)
    mailing.seedListId = [$seedListId]

    $suppressionListId = SuppressionListSample.new(@svc).createSuppressionList("supp_"+mailingName)
    mailing.suppressionListId = [$suppressionListId]

    $bounceId = SystemAddressSample.new(@svc).listByEmailAddress("BOUNCE", "bounce@sample.com").objectId;
    $fromId = SystemAddressSample.new(@svc).listByEmailAddress("FROM", "from@sample.com").objectId
    $replyId = SystemAddressSample.new(@svc).listByEmailAddress("REPLY", "reply@sample.com").objectId

    mailing.bounceAddressId = $bounceId
    mailing.replyAddressId = $replyId
    mailing.fromAddressId = $fromId

    mailing.description = "StandardMailing from Ruby"
    mailing.eliminateDuplicates = true
    mailing.type = "ONE_TIME"
    mailing.useHeaderFromTemplate = true

    # Set start time
    schedule = SchedulableMailing::Schedule.new
    schedule.startDateTime = DateTime::civil(2020, 1, 11, 1, 1, 1)
    mailing.schedule = schedule

    return SampleHelper::create(@svc, mailing)
  rescue => e
    puts "Error was occurred while creating a standard mailing mailing #{e.message}"
    return nil
  end

# Create a channel mailing
  #  mailingName is the name of the mailing.
  #  return the ID of created standard mailing.
  def createChannelStdMailing(mailingName)
    mailing = ChannelStandardMailing.new
    mailing.description = "Channel StandardMailing from Ruby"
    mailing.eliminateDuplicates = true
    mailing.type = "ONE_TIME"
    mailing.useHeaderFromTemplate = false

    # Add Include Targets to channel mailing
    $targetId = TargetSample.new(@svc).createChannelTarget("target_"+mailingName)
    $targetField = IncludedTargetField.new()
    $targetField.targetId = $targetId
    # This field is mandatory if eliminateDuplicates = true or suppression list added to mailing
    $targetField.suppressionColumnName = "id"
    # This field is mandatory if seed list added to mailing
    $targetField.seedColumnName = "id"
    mailing.includedTarget = [$targetField]

    # This value will be ignored and formats associated with Channel will be set.
    mailing.format = ["PUSH_NOTIFICATION"]

    schedule = SchedulableMailing::Schedule.new
    schedule.startDateTime = DateTime::civil(2020, 1, 11, 1, 1, 1)
    mailing.schedule = schedule

    setMailingCommonFields(mailing, mailingName)

    # Set dependencies
    $contentBlockId = ContentBlockSample.new(@svc).createContentBlock("content_block_"+mailingName);
    mailing.contentBlockId = [$contentBlockId]

    $attachmentId = AttachmentSample.new(@svc).createAttachment("attachment_"+mailingName);
    mailing.attachmentId = [$attachmentId]

    $templateId = TemplateSample.new(@svc).createChannelTemplate("channel_template_"+mailingName)
    mailing.templateId = $templateId

    $seedListId = SeedListSample.new(@svc).createChannelSeedList("seed_"+mailingName)
    mailing.seedListId = [$seedListId]

    $suppressionListId = SuppressionListSample.new(@svc).createChannelSuppressionList("supp_"+mailingName)
    mailing.suppressionListId = [$suppressionListId]

    # Get Channel by name
    filter = ChannelFilter.new
    SampleHelper::setFilter(filter)
    nameCondition = ScalarStringFilterCondition.new
    nameCondition.operator = "EQUAL"
    nameCondition.value = "Push Notification"
    filter.nameCondition = nameCondition
    $channelId = SampleHelper::list(@svc, filter)

    raise "Channel with specified name is not exist or licensed." if $channelId == nil

    # Set Channel Id
    mailing.channelId = $channelId[0]

    # Add Channel Headers
    $channelHeader1 = ChannelHeader.new()
    $channelHeader1.headerName = "Custom Header1"
    $channelHeader1.headerValue = "Custom value1"

    $channelHeader2 = ChannelHeader.new()
    $channelHeader2.headerName = "Custom header2"
    $channelHeader2.headerValue = "Custom value2"

    mailing.channelHeader = [ $channelHeader1, $channelHeader2 ]

    # The values of this field should be the column names in the all included targets
    mailing.channelProfileAttribute = ["id", "email_address"]

    return SampleHelper::create(@svc, mailing)
 rescue => e
    puts "Error occurred while creating a channel mailing #{e.message}"
    return nil
 end


  # Create a transactional mailing.
  #  mailingName is the name of the mailing.
  #  return the ID of created transactional mailing.
  def createTransactionalMailing(mailingName)
    txMailing = TransactionalMailing.new
    setMailingCommonFields(txMailing, mailingName)
    txMailing.description = "TransactionalMailing from Ruby"
    txMailing.type = "TRANSACTIONAL"
    txMailing.fromName = "From Name"
    txMailing.forward2FriendOffer = ""
    txMailing.forward2FriendOfferTrackingOption = "NONE"
    $targetId = TargetSample.new(@svc).createTarget("target_"+mailingName)

    $templateId = TemplateSample.new(@svc).createTemplate("template_"+mailingName)
    txMailing.templateId = $templateId

    $bounceId = SystemAddressSample.new(@svc).listByEmailAddress("BOUNCE", "bounce@sample.com").objectId;
    $fromId = SystemAddressSample.new(@svc).listByEmailAddress("FROM", "from@sample.com").objectId
    $replyId = SystemAddressSample.new(@svc).listByEmailAddress("REPLY", "reply@sample.com").objectId

    txMailing.bounceAddressId = $bounceId
    txMailing.replyAddressId = $replyId
    txMailing.fromAddressId = $fromId

    txMailing.targetId = $targetId
    txMailing.subject = "mailing subject"
    txMailing.format = ["HTML"]
    txMailing.header = ["mailing header"]

    return SampleHelper::create(@svc, txMailing)
  rescue => e
      puts "Error occurred while tx-mailing -  #{e.message}"
  end

  # Create a  channel tx-mailing.
  #  mailingName is the name of the mailing.
  #  return the ID of created transactional mailing.
  def createChannelTxMailing(mailingName)
    txMailing = ChannelTransactionalMailing.new
    setMailingCommonFields(txMailing, mailingName)
    txMailing.description = "Channel TxMailing from Ruby"
    txMailing.type = "TRANSACTIONAL"
    $targetId = TargetSample.new(@svc).createChannelTarget("target_"+mailingName)
    txMailing.targetId = $targetId

    txMailing.format = ["HTML"] # It will be ignored.
    txMailing.recipientFieldName = "id"

    $templateId = TemplateSample.new(@svc).createChannelTemplate("template_"+mailingName)
    txMailing.templateId = $templateId

    # Get Channel by name
    filter = ChannelFilter.new
    SampleHelper::setFilter(filter)
    nameCondition = ScalarStringFilterCondition.new
    nameCondition.operator = "EQUAL"
    nameCondition.value = "Push Notification"
    filter.nameCondition = nameCondition
    $channelId = SampleHelper::list(@svc, filter)

    raise "Channel with specified name is not exist or licensed." if $channelId == nil

    # Set Channel Id
    txMailing.channelId = $channelId[0]

    # Add Channel Headers
    $channelHeader1 = ChannelHeader.new()
    $channelHeader1.headerName = "Custom Header1"
    $channelHeader1.headerValue = "Custom value1"

    $channelHeader2 = ChannelHeader.new()
    $channelHeader2.headerName = "Custom header2"
    $channelHeader2.headerValue = "Custom value2"

    txMailing.channelHeader = [ $channelHeader1, $channelHeader2 ]

    return SampleHelper::create(@svc, txMailing)
  rescue => e
      puts "Error occurred while creating channel tx-mailing -  #{e.message}"
  end


  # Set mailing properties common to both standard and transactional mailings.
  def setMailingCommonFields(mailing, mailingName)
    mailing.bodyEncoding = "BASE_64"
    mailing.fieldDelimiter = "::"
    mailing.headerEncoding = "EIGHT_BIT"
    mailing.isApproved = true
    mailing.isCompliant = true
    mailing.name = mailingName
    mailing.outputBodyCharSet = "ASCII"
    mailing.outputHeaderCharSet = "UTF-8"
    mailing.priority = "NORMAL"
    mailing.rowDelimiter = "\n"
    mailing.useHeaderFromTemplate = false; 
  end


  # Get and print out a mailing.
  #  mailingId is the ID of the mailing to be printed.
  def getAndPrint(mailingId)
    mailing = SampleHelper::get(@svc, mailingId)
    puts mailing.class
    puts "\tID: " + mailing.objectId.id
    puts "\tName: " + mailing.name
    puts "\tCreated Time: " + mailing.createdTime.to_s()
    puts "\tDescription: " + mailing.description
    puts "\tModified Time: " + mailing.modifiedTime.to_s()
    puts "\tOrganization: " + mailing.organizationId.id
    puts "\tUserId: " + mailing.ownerId.id
    puts "\tFieldDelimiter: " + mailing.fieldDelimiter
    puts "\tRowDelimiter: " + mailing.rowDelimiter
    puts "\tMessageFormat: " + mailing.format[0].to_s
    puts "\tTemppate: " + mailing.templateId.id
    puts "\tBodyEncoding: " + mailing.bodyEncoding.to_s
    puts "\tHeaderEncoding: " + mailing.headerEncoding.to_s
    puts "\tPriority: " + mailing.priority.to_s
    puts "\tServerErrorCode: " + mailing.serverErrorCode.to_s
    puts "\tSererErrorMessage: " + mailing.serverErrorMessage.to_s
    puts "\tStatus: " + mailing.status.to_s
    puts "\tType: " + mailing.type.to_s

    if (mailingId.instance_of?(StandardMailingId))
      puts "\tBounce: " + mailing.bounceAddressId.id
      puts "\tFrom: " + mailing.fromAddressId.id
      puts "\tReply: " + mailing.replyAddressId.id
      puts "\tSubject: " + mailing.subject
      puts "\tFromName: " + mailing.fromName
      puts "\tMailngClass: " + mailing.mailingClassId.id
      puts "\tHeader Size: " + mailing.header[0]
      puts "\tIncludedTargets: " + mailing.includedTargetId[0].id
      puts "\tSeedLists: " + mailing.seedListId[0].id
      puts "\tSuppresionLists: " + mailing.suppressionListId[0].id
      puts "\tAttachment: " + mailing.attachmentId[0].id
      puts "\tContentBlock: " + mailing.contentBlockId[0].id
    elsif (mailingId.instance_of?(TransactionalMailingId))
      puts "\tBounce: " + mailing.bounceAddressId.id
      puts "\tFrom: " + mailing.fromAddressId.id
      puts "\tReply: " + mailing.replyAddressId.id
      puts "\tSubject: " + mailing.subject
      puts "\tFromName: " + mailing.fromName
      puts "\tMailngClass: " + mailing.mailingClassId.id
      puts "\tHeader Size: " + mailing.header[0]
      puts "\tFormatField: " + mailing.formatFieldName.to_s
      puts "\tRecipientField: " + mailing.recipientFieldName.to_s
      puts "\tMailingConfig:" + mailing.mailingConfig.to_s
      puts "\tRecordStructure: " + mailing.recordStructure.to_s
      puts "\tSenderNumber: " + mailing.senderNumber.to_s
      puts "\tTarget: " + mailing.targetId.id
    elsif (mailingId.instance_of?(ChannelStandardMailingId))
      puts "\tIncludedTarget: " + mailing.includedTarget[0].targetId.id
      puts "\tSuppression Column: " + mailing.includedTarget[0].suppressionColumnName
      puts "\tSeed Column: " + mailing.includedTarget[0].seedColumnName
      puts "\tSeedLists: " + mailing.seedListId[0].id
      puts "\tSuppresionLists: " + mailing.suppressionListId[0].id
      puts "\tAttachment: " + mailing.attachmentId[0].id
      puts "\tContentBlock: " + mailing.contentBlockId[0].id
      printChannelHeaders(mailing.channelHeader);
      puts "\tProfile Attributes :"
      for each in mailing.channelProfileAttribute
        puts "\t\t - "+each
      end
    elsif (mailingId.instance_of?(ChannelTransactionalMailingId))
      puts "\tRecipientField: " + mailing.recipientFieldName.to_s
      puts "\tMailingConfig:" + mailing.mailingConfig.to_s
      puts "\tRecordStructure: " + mailing.recordStructure.to_s
      puts "\tTarget: " + mailing.targetId.id
      printChannelHeaders(mailing.channelHeader);
    end
  rescue => e
    puts "Error occurred while printing the mailing info #{e.message}"
  end

  def printChannelHeaders(channelHeaders)
    puts "\tChannelHeaders - "
    for channelHeader in channelHeaders
        puts "\t\t - " + channelHeader.headerName + " = " + channelHeader.headerValue
    end
  rescue => e
      puts "#{e.message}"
  end


  # Launch a standard mailing
  def launchMailing(mailingId)
    request = LaunchRequest.new
    request.mailingId = mailingId

    return @svc.launch(request).success
  end

 # Schedule a standard mailing
  def scheduleMailing(mailingId)
    request = ScheduleRequest.new
    request.mailingId = mailingId

    return @svc.schedule(request).success
  end

  # Load a transactional mailing
  def loadMailing( txMailingId)
    request = LoadRequest.new
    request.mailingId = txMailingId

    return @svc.load(request).success
  end

  def getTxnMailingHandle( txMailingId)
     # Create request and set mailingId
     request = GetTxnMailingHandleRequest.new
     request.mailingId = txMailingId

     return @svc.getTxnMailingHandle(request).handle
  end

  def txnSend(mailingHandle)
     # Create request and set mailingId
     request = TxnSendRequest.new
     request.handle = mailingHandle;

     # Create record based on the target created in TargetSample
     record = SendRecord.new;

     field_id = NameValuePair.new
     field_id.name = "ID"
     field_id.value = "10"

     field_email = NameValuePair.new
     field_email.name = "email_address"
     field_email.value = "sample1@ruby.sample.com"

     field_name = NameValuePair.new
     field_name.name = "attachmentName"
     field_name.value = "First Last"

     # Set records
     record = [field_id, field_email, field_name]
     request.sendRecord = [record]

     return @svc.txnSend(request).success;
  end

#Commenting out send functions since they are deprecated
  # Send one record for transactional mailing.
  #  txMailingId is the ID of transactional mailing whose data will be sent
  #  return if send operation is successfule.
#  def sendOneRecord(txMailingId)
#    # Create request
#    request = SendRequest.new
#    request.mailingId = txMailingId
#
#    # Create one record based on the target created in TargetSample
#    field_id = NameValuePair.new
#    field_id.name = "ID"
#    field_id.value = "10"
#
#    field_email = NameValuePair.new
#    field_email.name = "email_address"
#    field_email.value = "sample1@ruby.sample.com"
#
#    field_name = NameValuePair.new
#    field_name.name = "attachmentName"
#    field_name.value = "First Last"
#
#    # Set records
#    record = SendRecord.new
#    record = [field_id, field_email, field_name]
#    request.sendRecord = [record]
#
#    return @svc.send(request).success
#  end
#
#
#  # Send data using strings.
#  #  txMailingId is the ID of transactional mailing whose data will be sent.
#  #  return if send operation is successful
#  def sendData(txMailingId)
#    # Get delimiter from mailing and use it for data string
#    mailing = SampleHelper::get(@svc, txMailingId)
#    fieldDelimiter = mailing.fieldDelimiter
#    rowDelimiter = mailing.rowDelimiter
#
#    # Create request
#    request = SendRequest.new
#    request.mailingId = txMailingId
#
#    # Set data: data is rows of strings separated by delimiters
#    data = "20" + fieldDelimiter + "sample2@dotnet.sample.com" + fieldDelimiter + "FirstName LastName" + rowDelimiter
#    request.sendData = data
#
#    return @svc.send(request).success;
#  end



  # Close a transactional mailing
  def closeMailing(txMailingId)
    request = CloseRequest.new
    request.mailingId = txMailingId

    return @svc.close(request).success
  end


  # Cancel a mailing
  def cancelMailing(mailingId)
    request = CancelRequest.new
    request.mailingId = mailingId

    return @svc.resume(request).success
  end


  # Pause a mailing
  def pauseMailing(mailingId)
    request = PauseRequest.new
    request.mailingId = mailingId

    return @svc.pause(request).success
  end


  # Resume a mailing
  def resumeMailing(mailingId)
    request = ResumeRequest.new
    request.mailingId = mailingId

    return @svc.resume(request).success
  end

  
  # List mailings by attachmentName: this should return one mailnig if name is matched.
  #  name is the mailing name to be listed.
  def listByName(name)
    # Create list filter
    filter = MailingFilter.new
    SampleHelper::setFilter(filter)

    # Set name condition
    nameCondition = ScalarStringFilterCondition.new
    nameCondition.operator = "EQUAL"
    nameCondition.value = name

    filter.nameCondition = nameCondition
    printListResponse(SampleHelper::list(@svc, filter))
  end

  # List Channel mailings
  def listChannelMailings()
    # Create list filter
    filter = ChannelMailingFilter.new

    # Create type condition: type in (RECURRING, TRANSACTIONAL)
    typeCondition = ArrayStringFilterCondition.new
    typeCondition.operator = "IN"
    typeCondition.value = ["ONE_TIME","RECURRING", "TRANSACTIONAL"]
    filter.typeCondition = typeCondition

    SampleHelper::setFilter(filter)
    printListResponse(SampleHelper::list(@svc, filter))
  end

  # List mailings in an organization: this can be done by using empty filter.
  # The organization is the one from header.
  def listByOrganization()
    # Create list filter
    filter = MailingFilter.new
    SampleHelper::setFilter(filter)

    printListResponse(SampleHelper::list(@svc, filter))
  end


  # List mailings by mailing type and status
  def listByTypeAndStatus()
    # Create list filter
    filter = MailingFilter.new
    SampleHelper::setFilter(filter)

    # Create status condition: status in (ACTIVE, EDITING, COMPLETED)
    statusCondition = ArrayStringFilterCondition.new
    statusCondition.operator = "IN"
    statusCondition.value = ["ACTIVE", "EDITING", "COMPLETED"]
    filter.statusCondition = statusCondition

    # Create type condition: type in (RECURRING, TRANSACTIONAL)
    typeCondition = ArrayStringFilterCondition.new
    typeCondition.operator = "IN"
    typeCondition.value = ["RECURRING", "TRANSACTIONAL"]
    filter.typeCondition = typeCondition

    printListResponse(SampleHelper::list(@svc, filter))
  end

  
  # Print out list response: print ID and attachmentName
  def printListResponse(objectIds)
    puts "\nList Response: "
    for id in objectIds
      mailing = SampleHelper::get(@svc, id)
      puts "\t" + id.id + " " + mailing.name
    end
  rescue
    puts "Nothing to print."
  end


  # Wait for mailing to become a desired status
  def waitForStatus(mailingId, desiredStatus)
    status = SampleHelper::get(@svc, mailingId).status

    # Return immediately if desired status is received, or if any permanent
    # status is encountered.
    while status != desiredStatus &&
          status != "ERROR" &&
          status != "FAILED" &&
          status != "COMPLETED" &&
          status != "COMPLETED_DEFERRED" &&
          status != "CANCELED" &&
          status != "EDITING" &&
          status != "SPEC_CANCELED"

      puts "waitForStatus on " + mailingId.id + ": " + status + "!=" + desiredStatus
      sleep(10);
      status = SampleHelper::get(@svc, mailingId).status
    end

    puts "waitForStatus on " + mailingId.id + " final: " + status
    return status;
  end
end
