# Copyright 2014, StrongMail Systems, Inc.  All rights reserved.


class SampleHelper
  # Get one object based on its ID.
  #  objectId is the object ID to retrieve.
  #  returan an object whose ID is objectId.
  def self.get(svc, objectId)
    request = GetRequest.new
    request << objectId

    # Go through each batchCreateResponse inside batch batchCreateResponse and get the object if it is successful.
    # Print out error message if any object is not retrieved successfully and set the object to null.
    batchGetResponse = svc.get(request)
    if (batchGetResponse.success[0])
      return batchGetResponse.getResponse[0].baseObject
    else
      faultDetail = batchGetResponse.fault[0]
      puts faultDetail.faultCode + " " + faultDetail.faultMessage
    end
  end


  # Create an object.
  #  anObject is the object that will be created.
  #  return the objectId.
  def self.create(svc, anObject)
    request = CreateRequest.new
    request << anObject

    # Go through each response inside batch response and get the objectId if it is successful.
    # Print out error message if any object is not created successfully and set the objectId to null.
    batchCreateResponse = svc.create(request)
    if (batchCreateResponse.success[0])
      $CREATED_OBJECT_IDS.push(batchCreateResponse.createResponse[0].objectId);
      return batchCreateResponse.createResponse[0].objectId
    else
      faultDetail = batchCreateResponse.fault[0]
      puts faultDetail.faultCode + " " + faultDetail.faultMessage
    end
  end


  # Update an object.
  #  anObject is the object that will be updated.
  def self.update(svc, anObject)
    request = UpdateRequest.new
    request << anObject
    batchUpdateResponse = svc.update(request)

    # Go through each request and print out error message if update failed.
    if (!batchUpdateResponse.success[0])
      faultDetail = batchUpdateResponse.fault[0]
      puts faultDetail.faultCode + " " + faultDetail.faultMessage
    end
  end

  def self.delete(svc, objectId)
    request = DeleteRequest.new
    request << objectId

    return svc.delete(request)
  end


  # Set values to a filter:
  # get the first page, 10 records per page, and using ascending order.
  def self.setFilter(filter)
    filter.isAscending = true
    filter.pageNumber = 0
    filter.recordsPerPage = 10
  end

  # List objects in an organization based on a filter
  #  return list of object IDs
  def self.list(svc, filter)
    request = ListRequest.new
    request.filter = filter

    return svc.list(request).objectId
  end
end

