# Copyright 2014, StrongMail Systems, Inc.  All rights reserved.

require 'WsseAuthHeaderHandler'
require 'SampleHelper.rb'
require 'DataSourceSample'
require 'TargetSample'
require 'SeedListSample'
require 'SuppressionListSample'
require 'AttachmentSample'
require 'ContentBlockSample'
require 'RuleSample'
require 'TemplateSample'
require 'MailingSample'
require 'SystemAddressSample'
require 'GlobalUndeliverableSample'

$IDS_DATA_FILE = "../data/Shared_DataSource_Records.txt"
$ATTACHMENT_FILE = "../data/Shared_Attachment.gif"
$CONTENT_BLOCK_FILE = "../data/Shared_ContentBlock.txt"
$CREATED_OBJECT_IDS=[]


# Create each sample in global scope for convenience
def createSamples(svc)

  $dsSample = DataSourceSample.new(svc)
  $targetSample = TargetSample.new(svc)
  $seedSample = SeedListSample.new(svc)
  $suppressSample = SuppressionListSample.new(svc)
  $attachmentSample = AttachmentSample.new(svc)
  $cbSample = ContentBlockSample.new(svc)
  $ruleSample = RuleSample.new(svc)
  $addressSample = SystemAddressSample.new(svc)
  $templateSample = TemplateSample.new(svc)
  $mailingSample = MailingSample.new(svc)
  $guSample = GlobalUndeliverableSample.new(svc)
end


# Create the following objects:
# Data Sources: IDS, EDS, Target, SeedList, SuppresionList;
# Content Library: Attachment, ContentBlock, Template, Rule;
# Mailings: StandardMaiiling, TransactionalMailing.
#  sequence is sequence number to differentiate the attachmentName for each run.
#  return an list of ObjectIds that was created.
def createAllData(sequence)
  
  objectIds = []

  dataSourceId = $dsSample.createIDS("ids_ruby_" + sequence)
  objectIds.push(dataSourceId)

  objectIds.push($dsSample.createCatalog("catalog_ruby_" + sequence))

  #User need to use his own external data source.
# objectIds.push($dsSample.createEDS("eds_ruby_" + sequence))

  targetId = $targetSample.createTarget("target_ruby_" + sequence)
  objectIds.push(targetId)

  seedListId = $seedSample.createSeedList("seed_ruby_" + sequence)
  objectIds.push(seedListId)

  suppressId = $suppressSample.createSuppressionList("suppress_ruby_" + sequence)
  objectIds.push(suppressId)

  attachmentId = $attachmentSample.createAttachment("attachment_ruby_" + sequence)
  objectIds.push(attachmentId)

  contentBlockId = $cbSample.createContentBlock("cb_ruby_" + sequence)
  objectIds.push(contentBlockId)
  
  objectIds.push($ruleSample.createRule("rule_ruby_" + sequence, targetId))

  templateId = $templateSample.createTemplate("template_ruby_"+sequence)
  objectIds.push(templateId)
  objectIds.push($mailingSample.createStandardMailing("mailing_ruby_" + sequence))
  objectIds.push($mailingSample.createTransactionalMailing("tx_Mailing_ruby_" + sequence))

  # Creating NonEmail Channel Assets (You need to have license for it)

  objectIds.push($seedSample.createChannelSeedList("channel_seed_"+sequence))

  objectIds.push($suppressSample.createChannelSuppressionList("channel_suppress_"+sequence))

  objectIds.push($templateSample.createChannelTemplate("channel_template_"+sequence))

  objectIds.push($mailingSample.createChannelStdMailing("channel_std_mailing_"+sequence))

  objectIds.push($mailingSample.createChannelTxMailing("channel_tx_mailing_"+sequence))

  return objectIds
end


# Get and print out the object contnet.
#  objectIds is the list of objectId whose content will be printed.
def getAndPrintAllData(objectIds)
  for id in objectIds
    return if id == nil
    if id.instance_of?(InternalDataSourceId)
      $dsSample.getAndPrint(id)
    elsif id.instance_of?(ExternalDataSourceId)
      $dsSample.getAndPrint(id)
    elsif id.instance_of?(TargetId)
      $targetSample.getAndPrint(id)
    elsif id.instance_of?(SeedListId)
      $seedSample.getAndPrint(id)
    elsif id.instance_of?(SuppressionListId)
      $suppressSample.getAndPrint(id)
    elsif id.instance_of?(AttachmentId)
      $attachmentSample.getAndPrint(id)
    elsif id.instance_of?(ContentBlockId)
      $cbSample.getAndPrint(id)
    elsif id.instance_of?(RuleId)
      $ruleSample.getAndPrint(id)
    elsif id.instance_of?(TemplateId)
      $templateSample.getAndPrint(id)
    elsif id.instance_of?(StandardMailingId)
      $mailingSample.getAndPrint(id)
    elsif id.instance_of?(TransactionalMailingId)
      $mailingSample.getAndPrint(id)
    elsif id.instance_of?(ChannelStandardMailingId)
      $mailingSample.getAndPrint(id)
    elsif id.instance_of?(ChannelTransactionalMailingId)
      $mailingSample.getAndPrint(id)
    elsif id.instance_of?(CatalogId)
      $dsSample.getAndPrint(id)
    else
      puts "unknow type: ", id.class
    end

  end
end


# Delete all objects created in createAllData()
def deleteAllData(svc, objectIds)
  #stdMailingId = getObjectId(objectIds, StandardMailingId)
  txMailingId = getObjectId(objectIds, TransactionalMailingId)

  # Must wait for mailings to complete before they can be deletedS
  #$mailingSample.waitForStatus(stdMailingId, "COMPLETED")
  $mailingSample.waitForStatus(txMailingId, "COMPLETED")

  for id in $CREATED_OBJECT_IDS
    SampleHelper::delete(svc, id)
  end
end


# Get an ObjectId from a list based on its type.
#  objectIds is a list of distinct objectIds, one for each object type.
#  type is the type whose ID will be returned.
#  return the ObjectId that matches the type.
def getObjectId(objectIds, type)
  for id in objectIds
    if id.instance_of?(type)
      return id
    end
  end
end



# Main
begin

  # username/password/organization and service end point.
  # Point to your MessageStudio server hostname with port 4443.
  opts = {"endpoint_url" => "https://localhost:4443/sm/services/mailing/v2",
          "username" => "admin",
          "password" => "admin",
          "organization" => "admin"}
  puts "User input:\n" + opts.inspect

  svc = WsseAuthHeader::sm_service(opts)
	puts svc

  # Use timestamp to avoid duplicate names
  sequence = DateTime.now.strftime("%Y_%m_%d_%H_%M_%S")

  # Create all objects: one for each type
  createSamples(svc)
  objectIds = createAllData(sequence)

  # Read and print out the objects
  getAndPrintAllData(objectIds)

  # Get Global Undeliverable records by Recipient Address
  $guSample.getByRecipientAddress("my41222", "EMAIL", "1")

  # Get Global Undeliverable records by FailureCode
  $guSample.getByFailureCode("2002", "EMAIL","1")

  # Reactivate Global Undeliverable
  $guSample.reactivateGlobalUndeliverable("my360.957386903465test@undeliverable.com", "EMAIL")

  # Add Global Undeliverable
  $guSample.addGlobalUndeliverables()

  # Get the data source that we will work on
  dataSourceId = getObjectId(objectIds, InternalDataSourceId)
  dataSource = SampleHelper::get(svc, dataSourceId)

  # Add data to IDS using file
  $dsSample.addRecordsUsingFile(dataSourceId, $IDS_DATA_FILE)

  # Upsert datasource records
  $dsSample.upsertRecords(dataSourceId)

  # Get datasource records
  $dsSample.getRecords(dataSourceId)

  # Export data source records to a file
  $dsSample.exportRecords(dataSourceId, "./data_source_export.txt")

  #  Update the data source with new name
  $dsSample.updateDataSourceName(dataSourceId, dataSource.name + "_updated")
  dataSource = SampleHelper::get(svc, dataSourceId)
  puts "\nNew Data Source Name: " + dataSource.name

  # Read data source stats
  $dsSample.getAndPrintStatistics(dataSourceId)

  # Copy the data source
  copiedId = $dsSample.copyDataSource(dataSourceId, dataSource.name + "_copied")
  puts "\nCopied IDS: " + copiedId.id

  # Remove data source records and print out new statistics
  removedDS = $dsSample.removeRecordsInline(dataSourceId)
  $dsSample.getAndPrintStatistics(dataSourceId);

  # Dedupe
  $dsSample.dedupeRecords(dataSourceId)
  $dsSample.getAndPrintStatistics(dataSourceId)

  # list DataSource by name
  dataSourceId2 = $dsSample.listInternalDataSourcesByName(dataSource.name)
  puts "Get DS after list by name: " + dataSourceId2.id

  # This section was commented out since user needs to use his own external data source.
  #edsId = getObjectId(objectIds, ExternalDataSourceId)
  #refreshed = $dsSample.refreshRecords(edsId)
  #puts "EDS refreshed: " + refreshed.to_s
  #
  #cancelled = $dsSample.cancelRefreshRecords(edsId)
  #puts "EDS cancelled: " + cancelled.to_s


  # Import new content to attachment using IDS record file
  attachmentId = getObjectId(objectIds, AttachmentId)
  $attachmentSample.importContent(attachmentId, $IDS_DATA_FILE)
  $attachmentSample.getAndPrint(attachmentId)

  # Remove records from seed list
  seedListId = getObjectId(objectIds, SeedListId)
  removedSeed = $seedSample.removeRecordsInline(seedListId)
  $seedSample.getAndPrint(seedListId);

  # Remove records from suppression list
  suppressionListId = getObjectId(objectIds, SuppressionListId)
  removedSuppres = $suppressSample.removeRecordsInline(suppressionListId)
  $suppressSample.getAndPrint(suppressionListId)


  # Get the mailings we will work with
  stdMailingId = getObjectId(objectIds, StandardMailingId)
  stdMailing = SampleHelper::get(svc, stdMailingId);
  txMailingId = getObjectId(objectIds, TransactionalMailingId)
  txMailing = SampleHelper::get(svc, txMailingId)

  # List mailings by attachmentName
  $mailingSample.listByName(stdMailing.name)

  # List mailings in an organization
  $mailingSample.listByOrganization()

  # list mailings by type and status
  $mailingSample.listByTypeAndStatus()

  # List channel mailing
  $mailingSample.listChannelMailings()

  # Get the templates we will work with
  templateId = getObjectId(objectIds, TemplateId)

  # This section was commented out since user need to create his own media server and zip file.
  #mediaId = MediaServerId.new
  #mediaId.id = "200"
  #
  #imported = $templateSample.importMessagePart(templateId, mediaId, "ZIP_FILE", "")
  #puts "Media imported: " + imported.to_s

  validXsl = $templateSample.validateXsl(templateId)
  puts "Valid xsl: " + validXsl.to_s

  tested = $templateSample.testTemplate(templateId)
  puts "Tested: " + tested.to_s


  # Schedule a standard mailing
  scheduled = $mailingSample.scheduleMailing(stdMailingId)
  puts "Schedule mailing: " + scheduled.to_s

  # Load a transactional mailing
  loaded = $mailingSample.loadMailing(txMailingId)
  puts  "Load mailing: " + loaded.to_s
  sleep(10); # Wait for 10 seconds for Tx Mailing to be active.

  # Get handle (You need not to get handle before each txnSend())
    handle =  $mailingSample.getTxnMailingHandle(txMailingId)

  # Send TxMailing using handle
  sent = $mailingSample.txnSend(handle)
  puts "Send records to " + txMailingId.id + ": " + sent.to_s

  # Send data using records
  # sent = $mailingSample.sendOneRecord(txMailingId)
  # puts "Mailing sent: " + sent.to_s

  # Send data using string
  # sent = $mailingSample.sendData(txMailingId);
  # puts "Send data for " + txMailingId.id + ": " + sent.to_s

  # Close mailing
  sleep(10); # Wait for 10 seconds before closing Tx Mailing.
  closed = $mailingSample.closeMailing(txMailingId);
  puts "Close mailing: " + closed.to_s


  # Delete all assets
  deleteAllData(svc, objectIds)
  SampleHelper::delete(svc, copiedId)

  puts "\nEnd of Ruby Samples"
rescue => bang
  $stderr << bang.to_s
  exit 1
end

