# Copyright 2014, StrongMail Systems, Inc.  All rights reserved.

require 'SampleHelper'

class SeedListSample
  def initialize(svc)
    @svc = svc
  end


  # Create a seed list and add some data.
  #  seedListName is the name of the seed list.
  #  return the ID of create seed list.
  def createSeedList(seedListName)
    seed = SeedList.new
    seed.name = seedListName
    seed.description = "SeedList from Ruby"
    seed.isTestList = true

    seedListId = SampleHelper::create(@svc, seed)

    # Add some records
    addRecords(seedListId);
    return seedListId;

  rescue => e
    puts "Error was occurred while creating a seed: #{e.message}"
    return nil
  end

# Create a channel seed list and add some data.
  #  seedListName is the name of the seed list.
  #  return the ID of create seed list.
  def createChannelSeedList(seedListName)
    seed = SeedList.new
    seed.name = seedListName
    seed.description = "SeedList from Ruby"
    seed.isTestList = true

    # Get Channel by name
    filter = ChannelFilter.new
    SampleHelper::setFilter(filter)

    # Set name condition
    nameCondition = ScalarStringFilterCondition.new
    nameCondition.operator = "EQUAL"
    nameCondition.value = "Push Notification"

    filter.nameCondition = nameCondition
    $channelId = SampleHelper::list(@svc, filter)

    raise "Channel with specified name is not exist or licensed." if $channelId == nil

    # Set Channel Id
    seed.channelId = $channelId[0]

    seedListId = SampleHelper::create(@svc, seed)

    # Add some records
    addRecords(seedListId);

    return seedListId;
  rescue => e
    puts "Error was occurred while creating a channel seed: #{e.message}"
    return nil
  end

  # Add 2 records to seed list using inline records.
  #  seedListId is the ID of the seed list where records will add.
  def addRecords(seedListId)
    # Create request
    request = AddSeedListRecordsRequest.new
    request.seedListId = seedListId

    # Create and set data
    request.address = ["seed1@ruby.com", "seed2@ruby.com"]

    # Make call
    @svc.addRecords(request)
  rescue => e
    puts "Error was occurred while adding records to seed: #{e.message}"
    return nil
  end


  # Remove one record that was created in addRecordsInine() from the seed list.
  #  seedListId is the seed list ID whose record will be removed.
  #  return the number of records that was removed.
  def removeRecordsInline(seedListId)
    request = RemoveSeedListRecordsRequest.new
    request.seedListId = seedListId
    request.address = ["seed_1@ruby.com"]

    return @svc.removeRecords(request)
  rescue => e
      puts "Error was occurred while removing records from seed: #{e.message}"
      return nil
  end

  
  def getAndPrint(seedListId)
    seed = SampleHelper::get(@svc, seedListId)
    puts seed.class
    puts "\tID: " + seed.objectId.id
    puts "\tName: " + seed.name
    puts "\tCreated Time: " + seed.createdTime.to_s
    puts "\tDescription: " + seed.description
    puts "\tModified Time: " + seed.modifiedTime.to_s
    puts "\tOrganization: " + seed.organizationId.id
    puts "\tUserId: " + seed.ownerId.id
    puts "\tIsTestList: " + seed.isTestList.to_s
    puts "\tTotal Records: " + seed.totalRecords.to_s
  end
end
