# Copyright 2014, StrongMail Systems, Inc.  All rights reserved.

require 'SampleHelper'

class SuppressionListSample
  def initialize(svc)
    @svc = svc
  end


  # Create a suppresion list and add some data.
  #  suppresionListName is the name of suppresion list.
  #  return the ID of created suppression list.
  def createSuppressionList(suppresionListName)
    suppress = SuppressionList.new
    suppress.name = suppresionListName
    suppress.description = "SupressionList from Ruby"
    suppress.includeByDefaultOnMailings = true

    suppressionListId = SampleHelper::create(@svc, suppress)

    # Add some records
    addRecords(suppressionListId);

    return suppressionListId
  rescue => e
    puts "Error occurred while creating a suppression #{e.message}"
    return nil
  end

  # Create a channel suppresion list and add some data.
  #  suppresionListName is the name of suppresion list.
  #  return the ID of created suppression list.
  def createChannelSuppressionList(suppresionListName)
    suppress = SuppressionList.new
    suppress.name = suppresionListName
    suppress.description = "SupressionList from Ruby"
    suppress.includeByDefaultOnMailings = true

    # Get Channel by name
    filter = ChannelFilter.new
    SampleHelper::setFilter(filter)

    # Set name condition
    nameCondition = ScalarStringFilterCondition.new
    nameCondition.operator = "EQUAL"
    nameCondition.value = "Push Notification"

    filter.nameCondition = nameCondition
    $channelId = SampleHelper::list(@svc, filter)

    raise "Channel with specified name is not exist or licensed." if $channelId == nil

    # Set Channel Id
    suppress.channelId = $channelId[0]

    suppressionListId = SampleHelper::create(@svc, suppress)

    # Add some records
    addRecords(suppressionListId);
    return suppressionListId
  rescue => e
    puts "Error occurred while creating channel suppression : #{e.message}"
    return nil
  end


  # Add 2 records to suppresion list using inline records.
  #  suppresionListId is the ID where records will add.
  def addRecords(suppressionListId)
    request = AddSuppressionListRecordsRequest.new
    request.suppressionListId = suppressionListId

    # Create and set data
    request.addressMatch = ["ruby.com", "suppress@ruby.com"] 

    # Make call
    @svc.addRecords(request)
  end


  # Remove one record that was created in addRecordsInline() from the suppresion list.
  #  suppresionListId is the suppresion list ID whose record will be removed.
  #  return the number of records that were removed.
  def removeRecordsInline(suppresionListId)
    request = RemoveSuppressionListRecordsRequest.new
    request.suppressionListId = suppresionListId
    request.addressMatch = ["suppress@ruby.com"]

    return @svc.removeRecords(request)
  end


  def getAndPrint(suppressionListId)
    suppress = SampleHelper::get(@svc, suppressionListId)
    puts suppress.class
    puts "\tID: " + suppress.objectId.id
    puts "\tName: " + suppress.name
    puts "\tCreated Time: " + suppress.createdTime.to_s
    puts "\tDescription: " + suppress.description
    puts "\tModified Time: " + suppress.modifiedTime.to_s
    puts "\tOrganization: " + suppress.organizationId.id
    puts "\tUserId: " + suppress.ownerId.id
    puts "\tIncludeByDefaultOnMailings: " + suppress.includeByDefaultOnMailings.to_s
    puts "\tTotal Records: " + suppress.totalRecords.to_s
  end
end
