# Copyright 2014, StrongMail Systems, Inc.  All rights reserved.

require 'SampleHelper'

class TargetSample
  def initialize(svc)
    @svc = svc
  end


  # Create a target based on the data source created in DataSourceSample.
  #  targetName is the name of target.
  #  dataSourceId is the data source ID this target will use.
  #  return the ID of creatd target.
  def createTargetByDataSource(targetName, dataSourceId)
    # Create Target and fill in properties.
    target = Target.new
    target.name = targetName
    target.dataSourceId = dataSourceId
    target.type = "ADVANCED"
    target.description = "Target from Ruby"
    target.emailAddressFieldName = "email_address"  # From the data source
    target.bounceFieldName = "email_address_status" # System generated based on email column
    target.excludeBounce = true
    target.unsubscribeFieldName = "unsub"           # System generated
    target.excludeUnsubscribe = true

    return SampleHelper::create(@svc, target)
  rescue => e
    puts "Error was occurred while creating a target : #{e.message}"
    return nil
  end

  # Create a non-email channel target
  # based on the data source created in DataSourceSample.
  #  targetName is the name of target.
  #  dataSourceId is the data source ID this target will use.
  #  return the ID of creatd target.
  def createChannelTargetByDataSource(targetName, dataSourceId)
    # Create Target and fill in properties.
    target = Target.new
    target.name = targetName
    target.dataSourceId = dataSourceId
    target.type = "ADVANCED"
    target.description = "Target from Ruby"
    target.emailAddressFieldName = "id"  # From the data source (non-email column)
    target.excludeBounce = false
    target.excludeUnsubscribe = false

    return SampleHelper::create(@svc, target)
  rescue => e
    puts "Error was occurred while creating a channel target : #{e.message}"
    return nil
  end

  # Create a target
  #  targetName is the name of target.
  #  return the ID of creatd target.
  def createChannelTarget(targetName)
    $dataSourceId = DataSourceSample.new(@svc)::createIDS("ids_"+targetName)
    return createChannelTargetByDataSource(targetName,$dataSourceId);
  end

  # Create a channel target
  #  targetName is the name of target.
  #  return the ID of creatd target.
  def createTarget(targetName)
    $dataSourceId = DataSourceSample.new(@svc)::createIDS("ids_"+targetName)
    return createTargetByDataSource(targetName,$dataSourceId);
  end


  def getAndPrint(targetId)
    target = SampleHelper::get(@svc, targetId)
    puts target.class
    puts "\tID: " + target.objectId.id
    puts "\tName: " + target.name
    puts "\tType: " + target.type
    puts "\tDataSource: " + target.dataSourceId.id
    puts "\tEmailField: " + target.emailAddressFieldName
    puts "\tSMS: " + target.smsAddressFieldName.to_s()
    puts "\tBounceFieldName: " + target.bounceFieldName
    puts "\tUnsubFieldName: " + target.unsubscribeFieldName
    puts "\tDescription: " + target.description
    puts "\tQuery: " + target.advancedQuery.to_s()
    puts "\tCreated Time: " + target.createdTime.to_s()
    puts "\tModified Time: " + target.modifiedTime.to_s()
    puts "\tOrganization: " + target.organizationId.id
    puts "\tUserId: " + target.ownerId.id
    puts "\tTotalRecords: " + target.totalRecords.to_s()
  end
end
