# Copyright 2014, StrongMail Systems, Inc.  All rights reserved.

require 'SampleHelper'
require 'SampleHelper.rb'
require 'DataSourceSample'
require 'TargetSample'
require 'SeedListSample'
require 'SuppressionListSample'
require 'AttachmentSample'
require 'ContentBlockSample'
require 'SystemAddressSample'


class TemplateSample
  def initialize(svc)
    @svc = svc
  end

  # Create a template.
  #  templateName is the name of template.
  #  attachmentId is the ID of attachment used by this template.
  #  contentBlockId is the ID of content block used by this tempalte.
  #  return the ID of created template.
  def createTemplate(templateName)
    template = Template.new

    template.name = templateName
    template.description = "Template from ruby"
    template.bodyEncoding = "BASE_64"
    template.fromName = "From Name"
    template.headerEncoding = "EIGHT_BIT"
    template.isApproved = true

    $bounceId = SystemAddressSample.new(@svc).listByEmailAddress("BOUNCE", "bounce@sample.com").objectId;
    template.bounceAddressId = $bounceId;
    $fromId = SystemAddressSample.new(@svc).listByEmailAddress("FROM", "from@sample.com").objectId
    template.fromAddressId = $fromId
    $replyId = SystemAddressSample.new(@svc).listByEmailAddress("REPLY", "reply@sample.com").objectId
    template.replyAddressId = $replyId
    template.subject = "template subject from Ruby"
    template.header = ["Template header"]

    $attachmentId = AttachmentSample.new(@svc).createAttachment("attachment_"+templateName);
    template.attachmentId = $attachmentId

    $contentBlockId = ContentBlockSample.new(@svc).createContentBlock("content_block_"+templateName);
    template.contentBlockId = $contentBlockId
    template.openTracking = true

    template.forward2FriendOffer = ""
    template.forward2FriendOfferTrackingOption = "NONE"

    # Use outputBodyCharSet(Token)
    template.outputBodyCharSet = "ASCII"

    # We need to specify datasource/target for add catalog info mapping
    $dataSourceId = DataSourceSample.new(@svc).createIDS("ids_"+templateName);
    template.dataSourceId = $dataSourceId

    $catalogId = DataSourceSample.new(@svc).createCatalog("catalog_"+templateName)

    $catalogField = CatalogField.new()
    $catalogField.catalogId = $catalogId
    # The type of the mapping field should be matched to type of pk in catalog
    $catalogField.mappingFieldName = ["id"]

    template.catalogField = [ $catalogField ]

    # Use outputHeaderCharSet(Token)
    template.outputHeaderCharSet = "UTF-8"

    # Set message part
    messagePart = MessagePart.new
    messagePart.content = "<html><body>HTML: hello, HTML message</body></html>"
    messagePart.format = "HTML"
    messagePart.isXsl = false
    template.messagePart = [messagePart]

    return SampleHelper::create(@svc, template)
  rescue => e
    puts "Error was occurred while creating a template: #{e.message}"
    return nil
  end

  
  # Get ant print a template.
  # templateId is the ID of the template to be printed out.
  def getAndPrint(templateId)
    template = SampleHelper::get(@svc, templateId)
    puts template.class
    puts "\tID: " + template.objectId.id
    puts "\tName: " + template.name
    puts "\tCreated Time: " + template.createdTime.to_s()
    puts "\tDescription: " + template.description
    puts "\tModified Time: " + template.modifiedTime.to_s()
    puts "\tOrganization: " + template.organizationId.id
    puts "\tUserId: " + template.ownerId.id
    puts "\tAttchment: " + template.attachmentId[0].id
    puts "\tContentBlock: " + template.contentBlockId[0].id
    puts "\tHeader: " + template.header.to_s
    puts "\tBodyEncoding: " + template.bodyEncoding.to_s
    puts "\tFromName: " + template.fromName
    puts "\tSubject: " + template.subject
    puts "\tMessagePart: " + template.messagePart[0].content
  end

  
  # Import message part using a zip file and media server.
  #  templateId is the ID of template where message part will be imported.
  #  mediaServerId is the media server ID used to store the image files.
  #  zipFile is the zip file that contains HTML and image files.
  #  folderName is the directory to unzip the zip file.
  #  return if the importing operaion is successful.
  def importMessagePart(templateId, mediaServerId, zipFile, folderName)
    # Create request
    request = ImportMessagePartRequest.new
    request.templateId = templateId
    request.folderName = folderName
    request.isXsl = false
    request.mediaServerId = mediaServerId

    # Set content from zip file
    request.zipFile = Base64.encode64(File.open(zipFile, "r") {|io| io.read})

    return @svc.importMessagePart(request).success
  rescue => e
     puts "Error was occurred  while importing message part : #{e.message}"
     return false
  end

# Create a channel template.
  #  templateName is the name of template.
  #  return the ID of created template.
  def createChannelTemplate(templateName)

    template = ChannelTemplate.new

    template.name = templateName
    template.description = "Channel Template from Ruby"
    template.bodyEncoding = "BASE_64"
    template.headerEncoding = "EIGHT_BIT"
    template.isApproved = true

    # Get Channel by name
    filter = ChannelFilter.new
    SampleHelper::setFilter(filter)

    # Set name condition
    nameCondition = ScalarStringFilterCondition.new
    nameCondition.operator = "EQUAL"
    nameCondition.value = "Push Notification"

    filter.nameCondition = nameCondition
    $channelId = SampleHelper::list(@svc, filter)

    raise "Channel with specified name is not exist or licensed." if $channelId == nil

    # Set Channel Id
    template.channelId = $channelId[0]

    $attachmentId = AttachmentSample.new(@svc).createAttachment("attachment_"+templateName);
    template.attachmentId = $attachmentId

    $contentBlockId = ContentBlockSample.new(@svc).createContentBlock("content_block_"+templateName);
    template.contentBlockId = $contentBlockId

    # Use outputBodyCharSet(Token)
    template.outputBodyCharSet = "ASCII"

    # Use outputHeaderCharSet(Token)
    template.outputHeaderCharSet = "UTF-8"

    # Set message part
    messagePart = MessagePart.new
    messagePart.content = "Content of push notification"
    messagePart.format = "Push_Notification"
    messagePart.isXsl = false
    template.messagePart = [messagePart]

    # We need to specify datasource/target for add catalog info mapping
    $dataSourceId = DataSourceSample.new(@svc).createIDS("ids_"+templateName);
    template.dataSourceId = $dataSourceId

    $catalogId = DataSourceSample.new(@svc).createCatalog("catalog_"+templateName)
    $catalogField = CatalogField.new()
    $catalogField.catalogId = $catalogId
    # The type of the mapping field should be matched to type of pk in catalog
    $catalogField.mappingFieldName = ["id"]

    template.catalogField = [ $catalogField ]

    # Add channel headers
    $channelHeader1 = ChannelHeader.new()
    $channelHeader1.headerName = "Custom Header1"
    $channelHeader1.headerValue = "Custom value1"

    $channelHeader2 = ChannelHeader.new()
    $channelHeader2.headerName = "Custom header2"
    $channelHeader2.headerValue = "Custom value2"

    template.channelHeader = [ $channelHeader1, $channelHeader2 ]
    return SampleHelper::create(@svc, template)
  rescue => e
    puts "Error was occurred while creating a channel template : #{e.message}"
    return nil
  end


  # Test template for its dynamic content.
  #  Email will be sent and tokens in the template will be replaced wit values.
  #  templateId is the templateId that will be tested.
  #  return if the test request is sent successfully.
  def testTemplate(templateId)
    # Create request
    request = TestTemplateRequest.new
    request.subjectPrefix = "Subject Prefix: "
    request.templateId = templateId
    request.useMultiPart = false
    request.address = ["test_address@testtemplate.com"]
    request.format = "HTML"

    # Make sure you have these two tokens in your template
    pair1 = NameValuePair.new
    pair1.name = "EMAIL"
    pair1.value = "testmail@testtemplate.com"
    pair2 = NameValuePair.new
    pair2.name = "NAME"
    pair2.value = "testmail name"

    # Set data to request and make call
    request.tokenValue = [pair1, pair2]
    return @svc.test(request).success
  rescue => e
    puts "Error was occurred while testing a template : #{e.message}"
    return nil
  end


  # Validate if a template's xsl content is valid.
  def validateXsl(templateId)
    request = ValidateXslRequest.new
    request.templateId = templateId
    request.messageFormat = "HTML"

    return @svc.validateXsl(request).valid
  end
end
