/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2012 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

package com.strongmail.webserviceclient.samples.cxf;

import com.strongmail.webserviceclient.samples.cxf.generated.MailingService;
import com.strongmail.webserviceclient.samples.cxf.generated.UnexpectedFault;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.*;
import org.apache.commons.lang.StringUtils;
import transactionalmailing.*;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class EASService {

    Map<Integer, SmTmailingPortType> _serviceHolder;
    AuthDetails _authCredentials;

    public EASService() {
        int eas_cnt = 0;
        try {
            try {
                eas_cnt = Integer.valueOf(System.getProperty("eas.count"));

                if (eas_cnt <= 0) {
                    throw new Exception("eas.count in build.xml should be a non-zero value.");
                }

            } catch (NumberFormatException n) {
                throw new Exception("eas.count not defined in build.xml or is 0. Please define the # of EASs as -Deas.count=n");
            }


            _serviceHolder = new HashMap<Integer, SmTmailingPortType>();

            for (int i = 1; i <= eas_cnt; i++) {
                String wsdl_location = System.getProperty("eas" + i + ".wsdl");
                System.out.println("Processing WSDL: " + wsdl_location);
                _serviceHolder.put(i, getService(wsdl_location));
            }

            _authCredentials = new AuthDetails();
            _authCredentials.setUserName(System.getProperty("eas.username"));
            _authCredentials.setPassword(System.getProperty("eas.password"));
        } catch (java.lang.Exception e) {
            System.out.println("Error in service creation: " + e.getMessage());
            //e.printStackTrace();
            System.exit(1);
        }
    }

    private static SmTmailingPortType getService(String wsdl_location) {
        SmTmailing stub = null;
        try {

            stub = new SmTmailing(new URL("file:" + wsdl_location));
        } catch (MalformedURLException e) {
            e.printStackTrace();
            System.exit(1);
        }
        return stub.getSmTmailing();
    }

    public void roundRobinEASSend(MailingService msService, TransactionalMailingId txMailingId) {
        GetAllEasByMailingIdRequest request = new GetAllEasByMailingIdRequest();
        request.setMailingId(txMailingId);
        try {
            GetAllEasListByMailingIdResponse response = msService.getAllEasByMailingId(request);
            List<GetAllEasByMailingIdResponse> easList = response.getGetAllEasByMailingIdResponse();


            for (int rounds = 0; rounds < 2; rounds++) {
                Iterator<GetAllEasByMailingIdResponse> itr = easList.iterator();
                while (itr.hasNext()) {
                    GetAllEasByMailingIdResponse g = itr.next();
                    this.send(Integer.valueOf(g.getEasId()), g.getMailingConfigName());
                }
            }
        } catch (UnexpectedFault unexpectedFault) {
            unexpectedFault.printStackTrace();
        }

    }

    private int send(int easid, String mailingConfigName) {
        try {

            FileDetails fileInfo = new FileDetails();
            fileInfo.setFormat(FormatType.DATA);
            fileInfo.setData("1::test@local-test.strongmailsystems.com::test");

            javax.xml.ws.Holder<Long> cnt = new javax.xml.ws.Holder<Long>();
            javax.xml.ws.Holder<String> dbid = new javax.xml.ws.Holder<String>();

            _serviceHolder.get(easid).send(_authCredentials, mailingConfigName, fileInfo, dbid, cnt);
            System.out.println("Sent " + cnt.value + " record to EAS" + easid + " using EAS API.");


        } catch (AuthenticationErrorFault authenticationErrorFault) {
            authenticationErrorFault.printStackTrace();
        } catch (FeatureUnavailabilityErrorFault featureUnavailabilityErrorFault) {
            featureUnavailabilityErrorFault.printStackTrace();
        } catch (VSGErrorFault vsgErrorFault) {
            vsgErrorFault.printStackTrace();
        } catch (ConnectionErrorFault connectionErrorFault) {
            connectionErrorFault.printStackTrace();
        } catch (MailingCreationErrorFault mailingCreationErrorFault) {
            mailingCreationErrorFault.printStackTrace();
        } catch (MailingOperationErrorFault mailingOperationErrorFault) {
            mailingOperationErrorFault.printStackTrace();
        } catch (ExceptionFault exceptionFault) {
            exceptionFault.printStackTrace();
        } catch (DatabaseSpecificationErrorFault databaseSpecificationErrorFault) {
            databaseSpecificationErrorFault.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("ERROR:" + e.getMessage());
            return -1;
        }

        return 0;
    }
}
