/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2014 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

package com.strongmail.webserviceclient.samples.cxf;

import com.strongmail.webserviceclient.samples.cxf.generated.MailingService;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.*;
import org.apache.commons.lang.StringUtils;

import java.util.ArrayList;
import java.util.List;

public class GlobalUndeliverableSample {

    private MailingService service;

    public GlobalUndeliverableSample(MailingService service) {
        this.service = service;
    }

    public void getByRecipientAddress(String messageDestination, String channelType, String organizationId) throws Exception{
        System.out.println("Start of  getGlobalUndeliverables");

        GlobalUndeliverableRequest request = prepareGlobalUndeliverableRequest(channelType, organizationId);
        request.setRecipientAddress(messageDestination);

        GlobalUndeliverableResponse response = service.getGlobalUndeliverable(request);

        if (response != null && response.getGlobalUndeliverableInfo() != null && response.getGlobalUndeliverableInfo().size()>0) {
            printRecords(response.getGlobalUndeliverableInfo());
        } else {
            System.out.println("No Records found");
        }
        System.out.println("End of getGlobalUndeliverables");
    }

    public void getByFailureCode(String failureCode, String channelType, String organizationId) throws Exception{
        System.out.println("Start of  getGlobalUndeliverables");

        GlobalUndeliverableRequest request = prepareGlobalUndeliverableRequest(channelType, organizationId);
        request.setFailureCode(failureCode);

        GlobalUndeliverableResponse response = service.getGlobalUndeliverable(request);

        if (response != null && response.getGlobalUndeliverableInfo() != null && response.getGlobalUndeliverableInfo().size()>0) {
          printRecords(response.getGlobalUndeliverableInfo());            
        } else {
            System.out.println("No Records found");
        }
        System.out.println("End of  getGlobalUndeliverables");
    }

    public void reactivateGlobalUndeliverable(String recipientAddress, String channelType, String orgId) throws Exception {
        System.out.println("Start of  reactivateGlobalUndeliverable");

        ReactivateGlobalUndeliverableRequest reactivateGlobalUndeliverableRequest = new ReactivateGlobalUndeliverableRequest();

        List<GlobalUndeliverableInfo> globalUndeliverableInfoList = new ArrayList<GlobalUndeliverableInfo>();
        GlobalUndeliverableInfo globalUndeliverableInfo = new GlobalUndeliverableInfo();

        globalUndeliverableInfo.setRecipientAddress(recipientAddress);
        globalUndeliverableInfo.setChannelType(channelType);

        // Ignore OrgId if config value set as acrross organization
        // OrgId mandatory if config value set as per organization
        OrganizationId organizationId = new OrganizationId();
        organizationId.setId(orgId);
        reactivateGlobalUndeliverableRequest.setOrganizationId(organizationId);

        globalUndeliverableInfoList.add(globalUndeliverableInfo);
        reactivateGlobalUndeliverableRequest.getGlobalUndeliverableInfo().addAll(globalUndeliverableInfoList);

        ReactivateGlobalUndeliverableResponse response = service.reactivateGlobalUndeliverable(reactivateGlobalUndeliverableRequest);

        System.out.println("End of  reactivateGlobalUndeliverable");

    }

    public void addGlobalUndeliverable() throws Exception {
        System.out.println("Start of addGlobalUndeliverable");

        AddGlobalUndeliverableRequest request = new AddGlobalUndeliverableRequest();

        OrganizationId orgId = new OrganizationId();
        orgId.setId("1");
        request.setOrganizationId(orgId);

        GlobalUndeliverableInfo info = new GlobalUndeliverableInfo();
        info.setFailureCode(2001);
        info.setRecipientAddress("dummayEmail@undeliverable.com");
        info.setChannelType("EMAIL");

        GlobalUndeliverableInfo info1 = new GlobalUndeliverableInfo();
        info1.setFailureCode(2101);
        info1.setRecipientAddress("0000000000");
        info1.setChannelType("SMS");

        request.getGlobalUndeliverableInfo().add(info);
        request.getGlobalUndeliverableInfo().add(info1);

        AddGlobalUndeliverableResponse response = service.addGlobalUndeliverable(request);

        if(response != null) {
            if(response.getFault().size() > 0) {
                for(FaultDetail faultDetail: response.getFault()) {
                    System.out.print("Fault code - "+faultDetail.getFaultCode());
                    System.out.println("\t Fault Description - " + faultDetail.getFaultMessage());
                }
            }
        }
        System.out.println("End of addGlobalUndeliverable");
    }

    private GlobalUndeliverableRequest prepareGlobalUndeliverableRequest(String channelType, String organizationId) throws Exception {
        GlobalUndeliverableRequest request = new GlobalUndeliverableRequest();

        if(StringUtils.isNotBlank(organizationId)) {
            OrganizationId organizationIdObj = new OrganizationId();
            organizationIdObj.setId(organizationId);
            request.setOrganizationId(organizationIdObj);
        }
        request.setChannelType(channelType);
        request.setPageNumber(1);
        request.setRecordsPerPage(25);

        return request;
    }
  
    private void printRecords(List<GlobalUndeliverableInfo> undeliverables){
      if(undeliverables.size() > 0){
        for(GlobalUndeliverableInfo undeliverable:undeliverables){
          System.out.println(" "+undeliverable.getRecipientAddress()+"\t"+undeliverable.getChannelType()+"\t"+undeliverable.getFailureCode());
        }
      }else{
        System.out.println("No Records Found");
      }
    }
}
