/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2014 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

package com.strongmail.webserviceclient.samples.cxf;

import com.strongmail.webserviceclient.samples.cxf.generated.MailingService;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.AddProgramContactRecordsRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.AddProgramContactRecordsResponse;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.GetProgramNodesRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.GetProgramNodesResponse;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ListRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ListResponse;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ObjectId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.Program;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ProgramContactRecord;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ProgramFilter;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ProgramId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ProgramNode;

import java.util.List;


public class ProgramSample
{
  private MailingService service;
  private SampleHelper helper;

  public ProgramSample(MailingService service)
  {
    this.service = service;
    helper = new SampleHelper();
  }


  /**
   * Inject records into an active program
   *
   * The program should be in active state. The contact ID and PDS primary key values need to be
   * specified for each record to be injected.
   *
   * @throws Exception
   */
  public void injectRecordsIntoProgram() throws Exception
  {
    ProgramId programId = new ProgramId();
    programId.setId("100"); // Please update this with the id of the program that you have launched from the UI.

    AddProgramContactRecordsRequest addToProgramRequest = new AddProgramContactRecordsRequest();
    addToProgramRequest.setProgramId(programId);

    ProgramContactRecord record1 = new ProgramContactRecord();
    record1.setContactId("1"); // Please update this with the value that you wish to use.
    record1.setProgramDataSourcePk("1"); // Please update this with the value that you wish to use.
    addToProgramRequest.getProgramContactRecord().add(record1);

    ProgramContactRecord record2 = new ProgramContactRecord();
    record2.setContactId("2"); // Please update this with the value that you wish to use.
    record2.setProgramDataSourcePk("2"); // Please update this with the value that you wish to use.
    addToProgramRequest.getProgramContactRecord().add(record2);

    AddProgramContactRecordsResponse response = (AddProgramContactRecordsResponse)this.service.addRecords(addToProgramRequest);
    if (response.getSuccessCount() == addToProgramRequest.getProgramContactRecord().size())
    {
      System.out.println("Successfully injected records into the specified program");
    }
    else
    {
      System.out.println("Could not inject records into the specified program");
      if (response.getDuplicateRecord() != null && response.getDuplicateRecord().size() > 0)
      {
        System.out.println("Duplicate records: ");
        for (ProgramContactRecord record : response.getDuplicateRecord())
        {
          System.out.println("ContactId: " + record.getContactId() + ", PDS PK: " + record.getProgramDataSourcePk());
        }
      }
      if (response.getFailureRecord() != null && response.getFailureRecord().size() > 0)
      {
        System.out.println("Failed records: ");
        for (ProgramContactRecord record : response.getFailureRecord())
        {
          System.out.println("ContactId: " + record.getContactId() + ", PDS PK: " + record.getProgramDataSourcePk());
        }
      }
    }
  }

  public void list() throws Exception
  {
    ProgramFilter filter = new ProgramFilter();
    helper.setFilter(filter);

    // Set the filter to request
    ListRequest request = new ListRequest();
    request.setFilter(filter);

    // Make call and print results
    printListResponse(service.list(request));
  }

  private void printListResponse(ListResponse response) throws Exception
  {
    List<ObjectId> programIds = response.getObjectId();

    System.out.println("\nProgram List Response:\n");
    for (ObjectId each : programIds)
    {
      Program program = (Program)helper.get(each);
      System.out.println("ProgramId:" + each.getId() + " Name:" + program.getName() + " Modified:" + program.getModifiedTime() + " Version:" + program.getVersion());
      GetProgramNodesRequest nodesRequest = new GetProgramNodesRequest();
      ProgramId id = new ProgramId();
      id.setId(each.getId());
      nodesRequest.setProgramId(id);
      GetProgramNodesResponse nodes = service.getProgramNodes(nodesRequest);
      System.out.println("Program Nodes:");
      for ( ProgramNode node: nodes.getProgramNodes())
      {
    	  System.out.println("\t" + node.getObjectId().getId() + " " + node.getName() + " Modified:" + node.getModifiedTime() + " Version:" + node.getVersion());
      }

    }
  }

}
