/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2014 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

package com.strongmail.webserviceclient.samples.cxf;

import com.strongmail.webserviceclient.samples.cxf.generated.MailingService;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.AttachmentId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.DataSourceId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.InternalDataSourceId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.InternalDataSourceExtnId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.MailingId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SeedListId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.StandardMailingId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SuppressionListId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.Tag;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TagId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TargetId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TransactionalMailingId;

import java.io.File;


public class SampleProgram
{
  public static final String IDS_DATA_FILE = "../data/Shared_DataSource_Records.txt";
  public static final String OMNI_IDS_DATA_FILE = "../data/Shared_OmniDataSource_Records.txt";
  public static final String ATTACHMENT_FILE = "../data/Shared_Attachment.gif";

  private DataSourceSample dsSample;
  private TargetSample targetSample;
  private SeedListSample seedListSample;
  private SuppressionListSample suppressionListSample;
  private TemplateSample templateSample;
  private AttachmentSample attachmentSample;
  private ContentBlockSample contentBlockSample;
  private RuleSample ruleSample;
  private MailingSample mailingSample;
  private ProgramSample programSample;
  private GlobalUndeliverableSample globalUndeliverableSample;
  private TagSample tagSample;

  private MailingId stdMailingId;
  private MailingId channelStdMailingId;
  private MailingId txMailingId;
  private MailingId channelTxMailingId;
  private DataSourceId idsId;
  private DataSourceId idxIdsId;
  private DataSourceId omniIdsId;

  private String tagName;

  public SampleProgram(MailingService svc) throws Exception
  {
    dsSample = new DataSourceSample(svc);
    targetSample = new TargetSample(svc);
    seedListSample = new SeedListSample(svc);
    suppressionListSample = new SuppressionListSample(svc);
    templateSample = new TemplateSample(svc);
    attachmentSample = new AttachmentSample(svc);
    contentBlockSample = new ContentBlockSample(svc);
    ruleSample = new RuleSample(svc);
    programSample = new ProgramSample(svc);
    globalUndeliverableSample = new GlobalUndeliverableSample(svc);
    tagSample = new TagSample(svc);

    SystemAddressSample addressSample;
    addressSample = new SystemAddressSample(svc);

    mailingSample = new MailingSample(svc);
  }

  /**
   * Create the objects:
   * Data Sources:  IDS, Omni-IDS, EDS, Target, SeedList, SuppressionList.
   *
   * Content Library: Attachment, ContentBlock, Template, Rule.
   *
   * Mailings: StandardMailing, TransactionalMailing, ChannelStandardMailing,
   * ChannelTransactionalMailing.
   *
   * @param sequence a sequence number to differentiate the name for each run
   * @return list of ObjectIds that was created
   * @throws Exception any exception
   */
  public boolean createAllData(String sequence) throws Exception
  {
    /****************************************************************************
     * Data Source Assets
     ****************************************************************************/

    // Create IDS
    idsId =dsSample.createIDS("ids_cxf"+sequence);

    // Create Omni IDS
    omniIdsId = dsSample.createIDSWithSync("omni_ids_cxf_"+sequence);

    // create IDX
    idxIdsId = dsSample.createIDX("idx_cxf_" + sequence);

    // EDS: need to change DataSourceSample to point to your database
    //objectIds.add(dsSample.createEDS("eds_cxf_" + sequence));

    // create Target
    TargetId targetId = targetSample.createTarget("target_cxf_" + sequence,idsId);


    /****************************************************************************
     * Common Assets
     ****************************************************************************/
     TagId tagId = tagSample.createTag("tag_name" + sequence);
     tagSample.getAndPrintTag(tagId);
     tagSample.listTagsByName("tag_name" + sequence);

    /****************************************************************************
     * EMail Channel Assets                                                   *
     ****************************************************************************/

    // Create a Seed
    seedListSample.createSeedList("seedList_cxf_" + sequence);

    //Create a Suppression
    suppressionListSample.createSuppressionList("suppressionList_cxf_" + sequence);

    //Create Attachement
    attachmentSample.createAttachment("attachment_cxf_"+sequence);

    // Create a Content_Block
    contentBlockSample.createContentBlock("contentBlock_cxf_" + sequence);

    // Create a Rule
    ruleSample.createRule("rule_cxf_" + sequence, targetId);

    // Create a Template
    templateSample.createTemplate("template_cxf_" + sequence);

    stdMailingId = mailingSample. createStandardMailing("std_mailing_cxf_" + sequence);

    txMailingId = mailingSample.createTransactionalMailing("tx_mailing_cxf_"+ sequence);

    /****************************************************************************
    * Non-EMail Channel Assets - License Dependent                          *
    ****************************************************************************/

    // Create a Channel Seed
    seedListSample.createChannelSeedList("ChannelSeed_"+sequence);

    // Create a Channel Suppression List
    suppressionListSample.createChannelSuppressionList("channel_suppression_"+sequence);

    // Create a Channel Template
    templateSample.createChannelTemplate("channel_std_mailing_cxf"+sequence);

    // Create a Channel Standard Mailing
    channelStdMailingId = mailingSample.createChannelStdMailing("channel_standard_mailing_"+sequence);

    // Create a Channel Tx Mailing
    channelTxMailingId = mailingSample.createChannelTxMailing("channel_tx_mailing_"+sequence);

    return true;
  }

  /* There are a few operations that are not called here, including
   * EDS: refreshRecords(), cancelRefreshRecords(),
   * Template: importMessagePart(), testTemplate(), validateXsl().
   * Mailing: cancel(), pause(), resume().
   * Please see corresponding sample program.
   */
  public static void main(String[] args) throws Exception
  {
    MailingService svc = SampleHelper.createService();
    InternalDataSourceId copiedId = null;
    SampleProgram samples = new SampleProgram(svc);

    if(args.length == 3)
    {
      SampleHelper.setLogRequests("true".equals(args[2]));
    }

    try
    {
      // Create all data
      if(samples.createAllData(args[0]))
      {
        // Add data to IDS using file
        InternalDataSourceId dataSourceId = (InternalDataSourceId) samples.idsId;

        InternalDataSourceExtnId idxDataSourceId = (InternalDataSourceExtnId) samples.idxIdsId;

        samples.dsSample.addRecordsUsingFile(dataSourceId, new File(IDS_DATA_FILE));

        samples.dsSample.addRecordsUsingFile(samples.omniIdsId, new File(OMNI_IDS_DATA_FILE));

        // Reuse the same file, since the definitions are same, except the primary key, which is different
        samples.dsSample.addRecordsUsingFile(idxDataSourceId, new File(IDS_DATA_FILE));

        // Upsert records into the datasource
        samples.dsSample.upsertRecord(dataSourceId);

        // Upsert records into the IDX datasource
        samples.dsSample.upsertRecord(idxDataSourceId);

        // Get upserted records for the datasource.
        samples.dsSample.getRecord(dataSourceId);

        // Get upserted records for the IDX datasource
        samples.dsSample.getRecord(idxDataSourceId);

        // Export data source records to a file
        File dataSourceExportFile = new File("./data_source_export.txt");
        samples.dsSample.exportRecords(dataSourceId, dataSourceExportFile);

        // Export IDX data source records to a file
        File idxDataSourceExportFile = new File("./idx_data_source_export.txt");
        samples.dsSample.exportRecords(idxDataSourceId, idxDataSourceExportFile);

        // Update the data source with new name
        samples.dsSample.updateDataSourceName(dataSourceId, "ids_cxf_updated");

        // Update the IDX data source with new name
        samples.dsSample.updateDataSourceName(idxDataSourceId, "idx_cxf_updated" + args[0]);

        // Get data source statistics
        samples.dsSample.getAndPrintStatistics(dataSourceId);

        // Get IDX data source statistics
        samples.dsSample.getAndPrintStatistics(idxDataSourceId);

        // Copy the data source
        copiedId = samples.dsSample.copyDataSource(dataSourceId, "ids_cxf_" + args[0] + "_copied");
        System.out.println("\nCopied IDS: " + copiedId.getId());

        // Dedupe data source records and print out new statistics
        samples.dsSample.dedupeRecords(dataSourceId);
        samples.dsSample.getAndPrintStatistics(dataSourceId);

        // Remove data source records and print out new statistics
        int removedDS = samples.dsSample.removeRecordsInline(dataSourceId);
        System.out.println("\nRemoved records: " + removedDS);
        samples.dsSample.getAndPrintStatistics(dataSourceId);

        // Remove data source records and print out new statistics
        int removedDX = samples.dsSample.removeRecordsInline(idxDataSourceId);
        System.out.println("\nRemoved records: " + removedDX);
        samples.dsSample.getAndPrintStatistics(idxDataSourceId);

        // Import new content to attachment using IDS record file
        AttachmentId attachmentId = (AttachmentId)SampleHelper.getObjectId(AttachmentId.class);
        samples.attachmentSample.importContent(attachmentId, new File(IDS_DATA_FILE));
        samples.attachmentSample.getAndPrint(attachmentId);

        // Remove records from seed list
        SeedListId seedId = (SeedListId)SampleHelper.getObjectId(SeedListId.class);
        int removedSeed = samples.seedListSample.removeRecordsInline(seedId);
        System.out.println("\nRemoved records: " + removedSeed);
        samples.seedListSample.getAndPrint(seedId);

        // Remove records from suppression list
        SuppressionListId suppressId = (SuppressionListId)SampleHelper.getObjectId(SuppressionListId.class);
        int removedSuppress = samples.suppressionListSample.removeRecordsInline(suppressId);
        System.out.println("\nRemoved records: " + removedSuppress);
        samples.suppressionListSample.getAndPrint(suppressId);

        // List mailing by name
        samples.mailingSample.listByName("std_mailing_cxf_" + args[0],true);

        // List channel mailing by name
        samples.mailingSample.listByName("std_mailing_cxf_" + args[0], false);

        // List mailing in an organization
        samples.mailingSample.listByOrganization(true);

        // List channel mailing in an organization
        samples.mailingSample.listByOrganization(false);

        // List mailing by type and status
        samples.mailingSample.listByTypeAndStatus(true);

        // List channel mailing by type and status
        samples.mailingSample.listByTypeAndStatus(false);

        System.out.println();
        StandardMailingId stdMailingId = (StandardMailingId)samples.stdMailingId;
        TransactionalMailingId txMailingId = (TransactionalMailingId)samples.txMailingId;

        // Schedule a standard mailing
        System.out.println("MailingID: " + stdMailingId.getId());

        boolean scheduled = samples.mailingSample.scheduleMailing(stdMailingId);
        System.out.println("Schedule mailing " + stdMailingId.getId() + ": " + scheduled);

        // Load a transactional mailing
        boolean loaded = samples.mailingSample.loadMailing(txMailingId);
        System.out.println("Load mailing " + txMailingId.getId() + ": " + loaded);

        // Wait for the transactional mailing to become ACTIVE.
        samples.mailingSample.waitForStatus(txMailingId, "ACTIVE");

        // txnSend and related APIs.
        samples.mailingSample.printTxnEASInfo(txMailingId);
        String txMailingHandle = samples.mailingSample.getTxnMailingHandle(txMailingId);
        System.out.println("txMailingHandle: " + txMailingHandle);

        boolean sent = samples.mailingSample.txnSend(txMailingHandle);
        System.out.println("Send record using txnSend for " + txMailingId.getId() + ": " + sent);

        // Close mailing
        boolean closed = samples.mailingSample.close(txMailingId);
        System.out.println("Close mailing " + txMailingId.getId() + ": " + closed);

      // Get the Global Undeliverable records based on search criteria
      samples.globalUndeliverableSample.getByRecipientAddress("dummyRecipientAddress", "EMAIL", "1");
      samples.globalUndeliverableSample.getByFailureCode("2002", "EMAIL", "1");

      // Reactivate recipient from global undeliverable to active
      samples.globalUndeliverableSample.reactivateGlobalUndeliverable("dummyRecipientAddress", "EMAIL", "100");

      // Add Global Undeliverable records
      samples.globalUndeliverableSample.addGlobalUndeliverable();

      /*
      // Uncomment this block for running the following code that injects records to a program.
      // Please see the prerequisites (section 5) in the README before enabling this part.
      samples.programSample.injectRecordsIntoProgram();
       */

        samples.programSample.list();
      }

    }
    finally
    {
      if ("true".equals(args[1]))
      {
        // true=delete the data
        // sleep to 5 seconds to let mailing finish.
        try
        {
          Thread.sleep(5000);
        }
        catch (InterruptedException e)
        {
          // ignore
        }

        // Delete the generated assets
        try
        {
          // Must wait for mailings to complete before they can be deleted
          // samples.mailingSample.waitForStatus(stdMailingId, MailingStatus.COMPLETED);
          samples.mailingSample.waitForStatus(samples.txMailingId, "COMPLETED");
          SampleHelper.deleleGeneratedAssests();
        }
        catch(Exception e)
        {
          // ignore
        }


        // Delete copied data source
        if (copiedId != null)
        {
          try
          {
            new SampleHelper().delete(copiedId);
          }
          catch (Exception e)
          {
            // ignore
          }
        }
      }
    }
  }
}
