/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2014 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

package com.strongmail.webserviceclient.samples.cxf;





import com.strongmail.webserviceclient.samples.cxf.generated.MailingService;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.FilterStringScalarOperator;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ListRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ListResponse;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ObjectId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.OrganizationId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ScalarStringFilterCondition;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.Tag;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TagFilter;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TagId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TagValue;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.UserId;


public class TagSample{
  MailingService service;

  public TagSample(MailingService svc){
    service = svc;
  }

  public TagId createTag(String tagName){
    Tag tag = new Tag();
    tag.setName(tagName);
    tag.setDescription("Created Using CXF samples");

    OrganizationId organizationId = new OrganizationId();
    organizationId.setId("1");
    tag.setOrganizationId(organizationId);

    UserId userId = new UserId();
    userId.setId("1");
    tag.setOwnerId(userId);

    // Specifiy the tag values
    TagValue tagValue1 = new TagValue();
    tagValue1.setValue("demo-value1");
    tag.getTagValue().add(tagValue1);

    TagValue tagValue2 = new TagValue();
    tagValue2.setValue("demo-value2");
    tag.getTagValue().add(tagValue2);

    TagValue tagValue3 = new TagValue();
    tagValue3.setValue("demo-value3");
    tag.getTagValue().add(tagValue3);

    TagId tagId = null;
    try{
      tagId = (TagId)new SampleHelper().create(tag);
      getAndPrintTag(tagId);

    }catch(Exception e){
      System.out.println("Error Occurred while creating Tag "+tagName);
      e.printStackTrace();
    }
    return tagId;
  }

  public void getAndPrintTag(TagId tagId) throws Exception
  {
    if(tagId == null){
      System.out.println("Null value is specified for tag.");
      return;
    }
    Tag tag = (Tag) new SampleHelper().get(tagId);

    System.out.println("Tag:\n");
    System.out.println("\tName:"+tag.getName());
    System.out.println("\tDescription:"+tag.getDescription());
    System.out.println("\tTagValues:");
    for(TagValue tagValue:tag.getTagValue()){
      System.out.println("\t\t-"+tagValue.getValue());
    }
  }

  public void listTagsByName(String tagName){
    try{
      TagFilter tagFilter = new TagFilter();


      ScalarStringFilterCondition nameCondition = new ScalarStringFilterCondition();
      nameCondition.setOperator(FilterStringScalarOperator.EQUAL);
      nameCondition.setValue(tagName);
      tagFilter.setTagName(nameCondition);

      ListRequest listRequest = new ListRequest();
      listRequest.setFilter(tagFilter);

      ListResponse listResponse = service.list(listRequest);
      if(listResponse.getObjectId().size() > 0)
      {
        System.out.println("Tag Ids-");
        for(ObjectId objectId:listResponse.getObjectId()){
          System.out.println("\nTagId "+objectId.getId());
        }

      }
    }catch(Exception e){
      System.out.println("Error occurred in list operation: "+e.getMessage());
    }
  }

}
