/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2014 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

package com.strongmail.webserviceclient.samples.cxf;

import com.strongmail.webserviceclient.samples.cxf.generated.MailingService;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.DataSourceId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.Target;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TargetId;


public class TargetSample
{
  private MailingService service;
  private SampleHelper helper;

  public TargetSample(MailingService service)
  {
    this.service = service;
    helper = new SampleHelper();
  }


  /**
   * Create a target based on the data source created in DataSourceSample.
   *
   * @param targetName   the name of the target
   * @param dataSourceId the data source ID this target will use
   * @return targetId
   * @throws Exception any exception
   */
  public TargetId createTarget(String targetName, DataSourceId dataSourceId) throws Exception
  {
    Target target = new Target();

    // Fill in the target
    target.setName(targetName);
    target.setDataSourceId(dataSourceId);
    target.setType("ADVANCED");
    target.setDescription("Target from CXF");
    target.setEmailAddressFieldName("email_address");   // From the data source
    target.setBounceFieldName("email_address_status");  // system generated based on email column
    target.setExcludeBounce(true);
    target.setUnsubscribeFieldName("unsub");  // system generated
    target.setExcludeUnsubscribe(true);

    // Wait for data source status to be idle before exporting
    helper.waitForIdle(dataSourceId);
    TargetId targetId = (TargetId)helper.create(target);
    return targetId;
  }

  /**
   * Create a channel target based on the data source created in DataSourceSample.
   *
   * @param targetName   the name of the target
   * @param dataSourceId the data source ID this target will use
   * @return targetId
   * @throws Exception any exception
   */
  public TargetId createChannelTarget(String targetName, DataSourceId dataSourceId) throws Exception
  {
    Target target = new Target();

    // Fill in the target
    target.setName(targetName);
    target.setDataSourceId(dataSourceId);
    target.setType("ADVANCED");
    target.setDescription("Target from CXF");
    target.setEmailAddressFieldName("id");   // Select to non-email column here

    // Wait for data source status to be idle before exporting
    helper.waitForIdle(dataSourceId);
    TargetId targetId = (TargetId)helper.create(target);
    return targetId;
  }

  public TargetId createTarget(String targetName) throws Exception
  {
    DataSourceId dataSourceId = new DataSourceSample(service).createIDS("IDS_"+targetName);
    return createTarget(targetName,dataSourceId);
  }

  public TargetId createChannelTarget(String targetName) throws Exception
  {
    DataSourceId dataSourceId = new DataSourceSample(service).createIDS("IDS_"+targetName);
    return createChannelTarget(targetName, dataSourceId);
  }

  /**
   * Get and print target object to standard out.
   *
   * @param targetId the target ID that will be retrieved and printed.
   * @throws Exception any exception
   */
  public void getAndPrint(TargetId targetId) throws Exception
  {
    Target target = (Target)helper.get(targetId);

    // Print target to standard out
    System.out.println("\nTarget:");
    System.out.println("TargetId: " + target.getObjectId().getId());
    System.out.println("Name: " + target.getName());
    System.out.println("Type: " + target.getType());
    System.out.println("DataSource: " + target.getDataSourceId().getId());
    System.out.println("EmailField: " + target.getEmailAddressFieldName());
    System.out.println("SMSField: " + target.getSmsAddressFieldName());
    System.out.println("BounceField: " + target.getBounceFieldName());
    System.out.println("UnsubField: " + target.getUnsubscribeFieldName());
    System.out.println("Description: " + target.getDescription());
    System.out.println("Query: " + target.getAdvancedQuery());
    System.out.println("CampaignId: " + target.getCampaignId());
    System.out.println("CreatedTime: " + target.getCreatedTime());
    System.out.println("ModifiedTime: " + target.getModifiedTime());
    System.out.println("OrganizationId: " + target.getOrganizationId().getId());
    System.out.println("OwnerId: " + target.getOwnerId().getId());
    System.out.println("Total Records: " + target.getTotalRecords());
  }
}
