/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2014 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

package com.strongmail.webserviceclient.samples.cxf;




import com.strongmail.webserviceclient.samples.cxf.generated.AuthorizationFault;
import com.strongmail.webserviceclient.samples.cxf.generated.InvalidObjectFault;
import com.strongmail.webserviceclient.samples.cxf.generated.MailingService;
import com.strongmail.webserviceclient.samples.cxf.generated.ObjectNotFoundFault;
import com.strongmail.webserviceclient.samples.cxf.generated.UnexpectedFault;
import com.strongmail.webserviceclient.samples.cxf.generated.UnrecognizedObjectTypeFault;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.AttachmentId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.BaseTemplateId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.CatalogField;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.CatalogId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ChannelFilter;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ChannelHeader;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ChannelId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ChannelTemplate;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ChannelTemplateId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.CharSet;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ContentBlockId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.CopyResponse;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.CopyTemplateRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.DataSourceFilter;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.DataSourceId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.FilterStringScalarOperator;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ImportMessagePartRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.InternalDataSourceId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ListRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ListResponse;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.MediaServerId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.MessagePart;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.NameValuePair;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.OrganizationId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ScalarStringFilterCondition;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.Template;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TemplateId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TestTemplateRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ValidateXslRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.NamedLink;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.FetchLinksTemplateRequest;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.FetchLinksResponse;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.FetchLinkResponse;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.DeepLink;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import javax.activation.DataHandler;
import javax.mail.util.ByteArrayDataSource;


public class TemplateSample
{
  private com.strongmail.webserviceclient.samples.cxf.generated.MailingService service;
  private SampleHelper helper;
  private AttachmentSample attachmentSample;
  private ContentBlockSample contentBlockSample;

  public TemplateSample(MailingService service)
  {
    this.service = service;
    attachmentSample = new AttachmentSample(service);
    contentBlockSample = new ContentBlockSample(service);
    helper = new SampleHelper();
  }


  /**
   * Create a template.
   *
   * @param templateName   the name of the template
   * @return TemplateId
   * @throws Exception any exception
   */
  public TemplateId createTemplate(String templateName) throws Exception
  {
    Template template = new Template();

    template.setBodyEncoding("BASE_64");
    template.setDescription("Template from CXF");
    template.setFromName("From Name");
    template.setHeaderEncoding("EIGHT_BIT");
    template.setIsApproved(true);
    template.setName(templateName);
    template.setOutputBodyCharSet(CharSet.ASCII);
    template.setOutputHeaderCharSet(CharSet.UTF_8);
    template.setSubject("CXF subject");
    template.setLinkTracking("Full");
    template.setOpenTracking(true);


    AttachmentId attachmentId = attachmentSample.createAttachment("Attachment_"+templateName);
    template.getAttachmentId().add(attachmentId);

    ContentBlockId contentBlockId = contentBlockSample.createContentBlock("ContentBlock_"+templateName);
    template.getContentBlockId().add(contentBlockId);


    template.getHeader().add("CXF template header");

    // Create a message part
    MessagePart messagePart = new MessagePart();
    String content = "<html>" +
                     "<body>" +
                     "Hello, This is a simple HTML message with two links<br/>" +
                     "This is <a href=\"http://www.somedomain.com/link1\">Link 1</a>" +
                     "<br/>This is <a href=\"http://www.somedomain.com/link2\">Link 2</a>" +
                     "<br/>This is <a href=\"#\">Link 3</a> "+
                     "</body>" +
                     "</html>"; 
    messagePart.setContent(content);
    messagePart.setFormat("HTML");
    messagePart.setIsXsl(false);

    template.getMessagePart().add(messagePart);

    // Make a call to fetchLinks.
    // This will return the list of urls found in the message part, encapsulated as named link objects.
    List<NamedLink> links = fetchLinks(template, "HTML");


    Map<String, String> urlToTagMap = new HashMap<String, String>();
    urlToTagMap.put("http://www.somedomain.com/link1", "CLICK");
    urlToTagMap.put("http://www.somedomain.com/link2", "UNSUB");
    urlToTagMap.put("#","FBRECOMMEND");

    for (int i = 0 ; i < links.size() ; i ++)
    {
      System.out.println("URL:"+links.get(i).getUrl());
      if (urlToTagMap.get(links.get(i).getUrl()) != null)
      {
        String tagType = urlToTagMap.get(links.get(i).getUrl());
        System.out.println("TagType is "+ tagType);
        links.get(i).setTagType(tagType);
        links.get(i).setTagFormat("Obfuscated");
        links.get(i).setName("link" + (i+1));

        if(tagType.equalsIgnoreCase("FBRECOMMEND"))
        {
          NameValuePair pageTtl = new NameValuePair();
          pageTtl.setName("pagettl");
          pageTtl.setValue("test123");

          links.get(i).getTrackingTagProperties().add(pageTtl);

          NameValuePair postUrl = new NameValuePair();
          postUrl.setName("posturl");
          postUrl.setValue(null);

          links.get(i).getTrackingTagProperties().add(postUrl);
        }

      }
      // Other properties like web analytics, link name etc can also be set in the named link object.
    }

    // Assign the updated links to the message part.  
    template.getMessagePart().get(0).getNamedLinks().clear();
    template.getMessagePart().get(0).getNamedLinks().addAll(links);

    DataSourceSample dataSourceSample = new DataSourceSample(service);

    String idsName = "IDS_"+System.currentTimeMillis();
    String catalogName = "CATALOG_"+System.currentTimeMillis();

    dataSourceSample.createIDS(idsName);
    dataSourceSample.createCatalog(catalogName);

    // We could have used the dataSourceId and catalogId from createIDS and createCatalog but
    // the following code is for demonstrating the use of list api

    DataSourceFilter dataSourceFilter = new DataSourceFilter();
    ScalarStringFilterCondition typeCondition = new ScalarStringFilterCondition();
    ScalarStringFilterCondition nameCondition = new ScalarStringFilterCondition();

    nameCondition.setOperator(FilterStringScalarOperator.EQUAL);
    typeCondition.setOperator(FilterStringScalarOperator.EQUAL);

    typeCondition.setValue("INTERNAL");
    nameCondition.setValue(idsName);
    dataSourceFilter.setTypeCondition(typeCondition);
    dataSourceFilter.setNameCondition(nameCondition);
    ListRequest listRequest =  new ListRequest();
    listRequest.setFilter(dataSourceFilter);

    DataSourceId dataSourceId = (DataSourceId)service.list(listRequest).getObjectId().get(0);

    typeCondition.setValue("CATALOG");
    nameCondition.setValue(catalogName);
    dataSourceFilter.setTypeCondition(typeCondition);
    dataSourceFilter.setNameCondition(nameCondition);
    listRequest =  new ListRequest();
    listRequest.setFilter(dataSourceFilter);

    DataSourceId catalogId = (DataSourceId)service.list(listRequest).getObjectId().get(0);

    template.setDataSourceId(dataSourceId);

    CatalogField catalogField = new CatalogField();
    catalogField.setCatalogId((CatalogId)catalogId);

    // The following field should be present in the underlying datasource/target
    catalogField.getMappingFieldName().add("id");

    template.getCatalogField().add(catalogField);

    // Make call
    return (TemplateId)helper.create(template);
  }

  /**
   * Create a template with deepLinks.
   *
   * @param templateName the name of the template
   * @return TemplateId
   * @throws Exception any exception
   */
  public TemplateId createTemplateWithDeepLinks(String templateName) throws Exception
  {

    Template template = new Template();

    template.setBodyEncoding("BASE_64");
    template.setDescription("Template from CXF");
    template.setFromName("From Name");
    template.setHeaderEncoding("EIGHT_BIT");
    template.setIsApproved(true);
    template.setName(templateName);
    template.setOutputBodyCharSet(CharSet.ASCII);
    template.setOutputHeaderCharSet(CharSet.UTF_8);
    template.setSubject("CXF subject");
    template.setLinkTracking("Obfuscated");
    template.setOpenTracking(true);

    template.getHeader().add("CXF template header");

    // Create a message part
    MessagePart messagePart = new MessagePart();
    String content = "<html>" +
                     "<body>" +
                     "Hello, This is a simple HTML message with two links<br/>" +
                     "This is <a href=\"http://www.somedomain.com/link1\">Link 1</a>" +
                     "<br/>This is <a href=\"http://www.somedomain.com/link2\">Link 2</a>" +
                     "<br/>This is <a href=\"#\">Link 3</a> " +
                     "</body>" +
                     "</html>";
    messagePart.setContent(content);
    messagePart.setFormat("HTML");
    messagePart.setIsXsl(false);

    template.getMessagePart().add(messagePart);

    // Make a call to fetchLinks.
    // This will return the list of urls found in the message part, encapsulated as named link objects.
    List<NamedLink> links = fetchLinks(template, "HTML");


    Map<String, String> urlToTagMap = new HashMap<String, String>();
    urlToTagMap.put("http://www.somedomain.com/link1", "CLICK");
    urlToTagMap.put("http://www.somedomain.com/link2", "UNSUB");
    urlToTagMap.put("#", "FBRECOMMEND");

    for(int i = 0; i < links.size(); i++)
    {
      System.out.println("URL:" + links.get(i).getUrl());
      if(urlToTagMap.get(links.get(i).getUrl()) != null)
      {
        String tagType = urlToTagMap.get(links.get(i).getUrl());
        System.out.println("TagType is " + tagType);
        links.get(i).setTagType(tagType);
        links.get(i).setTagFormat("Obfuscated");
        links.get(i).setName("link" + (i + 1));

        if(tagType.equalsIgnoreCase("FBRECOMMEND"))
        {
          NameValuePair pageTtl = new NameValuePair();
          pageTtl.setName("pagettl");
          pageTtl.setValue("test123");

          links.get(i).getTrackingTagProperties().add(pageTtl);

          NameValuePair postUrl = new NameValuePair();
          postUrl.setName("posturl");
          postUrl.setValue(null);

          links.get(i).getTrackingTagProperties().add(postUrl);
        }
      }

      if(template.getLinkTracking().equalsIgnoreCase("Obfuscated") && (links.get(i).getTagType().equalsIgnoreCase("CLICK")
                                                                       || links.get(i).getTagType().equalsIgnoreCase("UNSUB")))
      {
        DeepLink deepLink1 = new DeepLink();
        deepLink1.setType("IOS_App");
        deepLink1.setUrl("www.testIOSAppTestUrl.com");

        DeepLink deepLink2 = new DeepLink();
        deepLink2.setType("IOS_Fallback");
        deepLink2.setUrl("www.testIOSFallbackTestUrl.com");

        DeepLink deepLink3 = new DeepLink();
        deepLink3.setType("Android_App");
        deepLink3.setUrl("www.testAndroidAppTestUrl.com");

        DeepLink deepLink4 = new DeepLink();
        deepLink4.setType("Android_FallBack");
        deepLink4.setUrl("www.testAndroidFallbackTestUrl.com");

        links.get(i).getDeepLinks().add(deepLink1);
        links.get(i).getDeepLinks().add(deepLink2);
        links.get(i).getDeepLinks().add(deepLink3);
        links.get(i).getDeepLinks().add(deepLink4);
      }
      // Other properties like web analytics, link name etc can also be set in the named link object.
    }

    // Assign the updated links to the message part.
    template.getMessagePart().get(0).getNamedLinks().clear();
    template.getMessagePart().get(0).getNamedLinks().addAll(links);

    DataSourceSample dataSourceSample = new DataSourceSample(service);

    String idsName = "IDS_" + System.currentTimeMillis();
    String catalogName = "CATALOG_" + System.currentTimeMillis();

    dataSourceSample.createIDS(idsName);
    dataSourceSample.createCatalog(catalogName);

    // We could have used the dataSourceId and catalogId from createIDS and createCatalog but
    // the following code is for demonstrating the use of list api

    DataSourceFilter dataSourceFilter = new DataSourceFilter();
    ScalarStringFilterCondition typeCondition = new ScalarStringFilterCondition();
    ScalarStringFilterCondition nameCondition = new ScalarStringFilterCondition();

    nameCondition.setOperator(FilterStringScalarOperator.EQUAL);
    typeCondition.setOperator(FilterStringScalarOperator.EQUAL);

    typeCondition.setValue("INTERNAL");
    nameCondition.setValue(idsName);
    dataSourceFilter.setTypeCondition(typeCondition);
    dataSourceFilter.setNameCondition(nameCondition);
    ListRequest listRequest = new ListRequest();
    listRequest.setFilter(dataSourceFilter);

    DataSourceId dataSourceId = (DataSourceId)service.list(listRequest).getObjectId().get(0);

    typeCondition.setValue("CATALOG");
    nameCondition.setValue(catalogName);
    dataSourceFilter.setTypeCondition(typeCondition);
    dataSourceFilter.setNameCondition(nameCondition);
    listRequest = new ListRequest();
    listRequest.setFilter(dataSourceFilter);

    DataSourceId catalogId = (DataSourceId)service.list(listRequest).getObjectId().get(0);

    template.setDataSourceId(dataSourceId);

    CatalogField catalogField = new CatalogField();
    catalogField.setCatalogId((CatalogId)catalogId);

    // The following field should be present in the underlying datasource/target
    catalogField.getMappingFieldName().add("id");

    template.getCatalogField().add(catalogField);

    // Make call
    return (TemplateId)helper.create(template);
  }

  public ChannelTemplateId createChannelTemplate(String templateName) throws Exception
  {
    ChannelTemplate channelTemplate = new ChannelTemplate();

    channelTemplate.setBodyEncoding("BASE_64");
    channelTemplate.setDescription("Template from CXF");
    channelTemplate.setHeaderEncoding("EIGHT_BIT");
    channelTemplate.setIsApproved(true);
    channelTemplate.setName(templateName);
    channelTemplate.setOutputBodyCharSet(CharSet.ASCII);
    channelTemplate.setOutputHeaderCharSet(CharSet.UTF_8);
    channelTemplate.setLinkTracking("Full");


    //mqTemplate.getHeader().add("CXF template header");

    // Create a message part
    MessagePart messagePart = new MessagePart();
    String content = "This is example of push notification. However to run this sample User should MQ License";
    messagePart.setContent(content);
    messagePart.setFormat("push_notification");
    messagePart.setIsXsl(false);

    channelTemplate.getMessagePart().add(messagePart);

    ChannelHeader header1 = new ChannelHeader();
    header1.setHeaderName("CustomUserHeader1");
    header1.setHeaderValue("CustomUserHeaderValue1");

    ChannelHeader header2 = new ChannelHeader();
    header2.setHeaderName("CustomUserHeader2");
    header2.setHeaderValue("CustomUserHeaderValue2");

    ChannelHeader header3 = new ChannelHeader();
    header3.setHeaderName("CustomUserHeader3");
    header3.setHeaderValue("CustomUserHeaderValue3");

    channelTemplate.getChannelHeader().add(header1);
    channelTemplate.getChannelHeader().add(header2);
    channelTemplate.getChannelHeader().add(header3);

    ChannelFilter mqFilter = new ChannelFilter();

    ListRequest listRequest = new ListRequest();

    ScalarStringFilterCondition nameCondition = new ScalarStringFilterCondition();
    nameCondition.setOperator(FilterStringScalarOperator.EQUAL);
    nameCondition.setValue("Push Notification");

    mqFilter.setNameCondition(nameCondition);
    listRequest.setFilter(mqFilter);
    ListResponse listResponse = service.list(listRequest);

    if(listResponse == null || listResponse.getObjectId().size() < 1)
    {
      System.out.println("Either Message Queue with specified name is not exist OR Message Queue license is not present");
      return null;
    }

    channelTemplate.setChannelId((ChannelId)listResponse.getObjectId().get(0));

    // create template
    return (ChannelTemplateId)helper.create(channelTemplate);

  }

  public void copy(TemplateId templateId)
    throws InvalidObjectFault, UnexpectedFault, ObjectNotFoundFault, AuthorizationFault, UnrecognizedObjectTypeFault
  {
    System.out.print("CopingTemplate");

    if(templateId == null)
    {
      templateId = new TemplateId();
      templateId.setId("1001");
    }

    CopyTemplateRequest request = new CopyTemplateRequest();

    request.setFromId(templateId);
    OrganizationId orgId = new OrganizationId();
    orgId.setId("1");
    request.setNewOrganizationId(orgId);
    request.setNewName("CopiedTemplate");

    CopyResponse res =  service.copy(request);

    System.out.print(res.getObjectId().toString());
  }

  public void getAndPrint(BaseTemplateId templateId) throws Exception
  {
    if(templateId instanceof TemplateId)
    {
      getAndPrint((TemplateId)templateId);
    }
    else
    {
      getAndPrint((ChannelTemplateId)templateId);
    }
  }


  public void getAndPrint(TemplateId templateId) throws Exception
  {
    Template template = (Template)helper.get(templateId);

    // Print template to standard out
    System.out.println("\nTemplate:");
    System.out.println("TemplateId: " + template.getObjectId().getId());
    System.out.println("Name: " + template.getName());
    System.out.println("CampaignId: " + template.getCampaignId());
    System.out.println("CreatedTime: " + template.getCreatedTime());
    System.out.println("Description: " + template.getDescription());
    System.out.println("ModifiedTime: " + template.getModifiedTime());
    System.out.println("OrganizationId: " + template.getOrganizationId().getId());
    System.out.println("OwnerId: " + template.getOwnerId().getId());
    System.out.println("AttachmentId: " + SampleHelper.formatIdList(template.getAttachmentId()));
    System.out.println("ContentBlockId: " + SampleHelper.formatIdList(template.getContentBlockId()));
    System.out.println("Headers: " + template.getHeader());
    System.out.println("BodyEncoding: " + template.getBodyEncoding());
    System.out.println("HeaderEncoding: " + template.getHeaderEncoding());
    System.out.println("MessagePart: " + SampleHelper.formatMessagePartList(template.getMessagePart()));
    System.out.println("BodyCharset: " + template.getOutputBodyCharSet().value());
    System.out.println("BodyOutputToken: " + template.getOutputBodyCharSetToken());
    System.out.println("HeaderCharset: " + template.getOutputHeaderCharSet().value());
    System.out.println("HeaderOutputToken: " + template.getOutputHeaderCharSetToken());
    System.out.println("BounceAddressId: " + template.getBounceAddressId());
    System.out.println("FromAddressId: " + template.getFromAddressId());
    System.out.println("FromName: " + template.getFromName());
    System.out.println("ReplyAddressId: " + template.getReplyAddressId());
    System.out.println("Subject: " + template.getSubject());
    System.out.println("Named Links: " + getPrintableNamedLinks(template));
    printCatalogInfo(template.getCatalogField());
  }

  public void getAndPrint(ChannelTemplateId channelTemplateId) throws Exception
  {
    ChannelTemplate channelTemplate = (ChannelTemplate)helper.get(channelTemplateId);

    // Print template to standard out
    System.out.println("\nTemplate:");
    System.out.println("TemplateId: " + channelTemplate.getObjectId().getId());
    System.out.println("Name: " + channelTemplate.getName());
    System.out.println("CampaignId: " + channelTemplate.getCampaignId());
    System.out.println("CreatedTime: " + channelTemplate.getCreatedTime());
    System.out.println("Description: " + channelTemplate.getDescription());
    System.out.println("ModifiedTime: " + channelTemplate.getModifiedTime());
    System.out.println("OrganizationId: " + channelTemplate.getOrganizationId().getId());
    System.out.println("OwnerId: " + channelTemplate.getOwnerId().getId());
    System.out.println("AttachmentId: " + SampleHelper.formatIdList(channelTemplate.getAttachmentId()));
    System.out.println("ContentBlockId: " + SampleHelper.formatIdList(channelTemplate.getContentBlockId()));
    System.out.println("BodyEncoding: " + channelTemplate.getBodyEncoding());
    System.out.println("HeaderEncoding: " + channelTemplate.getHeaderEncoding());
    System.out.println("MessagePart: " + SampleHelper.formatMessagePartList(channelTemplate.getMessagePart()));
    System.out.println("BodyCharset: " + channelTemplate.getOutputBodyCharSet().value());
    System.out.println("BodyOutputToken: " + channelTemplate.getOutputBodyCharSetToken());
    System.out.println("HeaderCharset: " + channelTemplate.getOutputHeaderCharSet().value());
    System.out.println("HeaderOutputToken: " + channelTemplate.getOutputHeaderCharSetToken());
    printCatalogInfo(channelTemplate.getCatalogField());
    System.out.println("Channel Id "+ channelTemplate.getChannelId().getId());
    printMqHeaders(channelTemplate.getChannelHeader());
  }

  private void printMqHeaders(List<ChannelHeader> channelHeaders)
  {
    if(channelHeaders != null)
    {
      System.out.println("ChannelHeaders:");
      for(ChannelHeader header:channelHeaders)
      {
        System.out.println("Name:"+header.getHeaderName() +" ,  Value:"+header.getHeaderValue());
      }
    }
  }

  private void printCatalogInfo(List<CatalogField> catalogFields)
  {
    if(catalogFields != null)
    {
      for(CatalogField catalogField:catalogFields)
      {
        System.out.println("Catalog ID :"+ catalogField.getCatalogId() +" Mapping Field : "+ catalogField.getMappingFieldName());
      }
    }

  }


  /**
   * Import message part using a zip file and media server.
   *
   * @param templateId    the template that message part will be added to
   * @param mediaServerId the media server that is used to store the image files.
   * @param zipFile       the zip file that contains HTML and image files.
   * @param folderName    the directory to unzip the zip file
   * @return if the importing operation is successful
   * @throws Exception any exception
   */
  public boolean importMessagePart(TemplateId templateId,
                                   MediaServerId mediaServerId,
                                   File zipFile,
                                   String folderName) throws Exception
  {

    // Create request
    ImportMessagePartRequest request = new ImportMessagePartRequest();
    request.setFolderName(folderName);
    request.setIsXsl(false);
    request.setMediaServerId(mediaServerId);
    request.setTemplateId(templateId);

    // Set zip file
    InputStream is = new FileInputStream(zipFile);
    DataHandler dh = new DataHandler(new ByteArrayDataSource(is, "application/octet-stream"));
    request.setZipFile(dh);

    return service.importMessagePart(request).isSuccess();
  }


  /**
   * Test template for it dynamic content. Email will be sent and tokens in the template will be
   * replaced with values.
   *
   * @param templateId the TemplateId that will be sent out.
   * @return if the test request is sent successfully.
   * @throws Exception any exception
   */
  public boolean testTemplate(BaseTemplateId templateId) throws Exception
  {
    TestTemplateRequest request = new TestTemplateRequest();
    request.setSubjectPrefix("Subject Prefix: ");
    request.setTemplateId(templateId);
    request.setUseMultiPart(false);
    request.getAddress().add("test_address@testtemplate.com");
    request.getFormat().add("HTML");

    // Make sure you have these two tokens in your template
    NameValuePair pair1 = new NameValuePair();
    pair1.setName("EMAIL");
    pair1.setValue("testemail@testtemplate.com");
    request.getTokenValue().add(pair1);

    NameValuePair pair2 = new NameValuePair();
    pair2.setName("NAME");
    pair2.setValue("testemail name");
    request.getTokenValue().add(pair2);

    return service.test(request).isSuccess();
  }


  /**
   * Validate if a template's xsl content is valid.
   *
   * @param templateId the TemplateId whose XSL content will be validated.
   * @return if the content is valid XSL
   * @throws Exception any exception
   */
  public boolean validateXsl(TemplateId templateId) throws Exception
  {
    ValidateXslRequest request = new ValidateXslRequest();
    request.setMessageFormat("HTML");
    request.setTemplateId(templateId);

    return service.validateXsl(request).isValid();
  }


  private List<NamedLink> fetchLinks(Template template, String format) throws Exception
  {
    List<NamedLink> links = new ArrayList<NamedLink>();
    FetchLinksTemplateRequest request = new FetchLinksTemplateRequest();
    request.setTemplate(template);
    request.setMessageFormat(format);
    FetchLinksResponse response = service.fetchLinks(request);
    List<FetchLinkResponse> individualLinkResponse = response.getFetchLinkResponse();
    if (individualLinkResponse != null && individualLinkResponse.size() > 0)
    {
      for (FetchLinkResponse individualLink : individualLinkResponse)
      {
        links.add(individualLink.getNamedLink());
      }
    }

    return links;
  }

  private String getPrintableNamedLinks(Template template)
  {
    StringBuilder printableNamedLinks = new StringBuilder("[");
    List<MessagePart> messageParts = template.getMessagePart();
    if (messageParts != null)
    {
      for (int p = 0; p < messageParts.size(); p++)
      {
        if (messageParts.get(p).getNamedLinks() != null && messageParts.get(p).getNamedLinks().size() > 0)
        {
          if (p > 0)
          {
            printableNamedLinks.append(", ");
          }
          printableNamedLinks.append(" [").append(messageParts.get(p).getFormat()).append(" - ");
          List<NamedLink> links = messageParts.get(p).getNamedLinks();
          for (int n = 0; n < links.size(); n++)
          {
            if (n > 0)
            {
              printableNamedLinks.append(", ");
            }
            printableNamedLinks.append(links.get(n).getName()).append(";").append(links.get(n).getUrl()).append(";").append(links.get(n).getTagType());
          }

          printableNamedLinks.append("]");
        }

      }
    }

    printableNamedLinks.append("]");

    return printableNamedLinks.toString();
  }
}
