/*
 * -------------------------------------------------------------------- *
 *  STRONGMAIL SYSTEMS                                                  *
 *                                                                      *
 *  Copyright 2014 StrongMail Systems, Inc. - All rights reserved.      *
 *                                                                      *
 *  Visit http://www.strongmail.com for more information                *
 *                                                                      *
 *  You may incorporate this Source Code in your application only if    *
 *  you own a valid license to do so from StrongMail Systems, Inc.      *
 *  and the copyright notices are not removed from the source code.     *
 *                                                                      *
 *  Distributing our source code outside your organization              *
 *  is strictly prohibited                                              *
 *                                                                      *
 * -------------------------------------------------------------------- *
 */

package com.strongmail.webserviceclient.samples.cxf;

import com.strongmail.webserviceclient.samples.cxf.generated.MailingService;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.DataSourceId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.InternalDataSourceId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.MailingId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.ObjectId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.SystemAddress;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TargetId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TemplateId;
import com.strongmail.webserviceclient.samples.cxf.generated.objects.TransactionalMailingId;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: nikhil
 * Date: 8/10/11
 * Time: 3:17 PM
 */
public class Tier3Sample {
    private static final String IDS_DATA_FILE = "samples/v2/data/Shared_DataSource_Records.txt";

    private DataSourceSample dsSample;
    private TargetSample targetSample;
    private TemplateSample templateSample;
    private MailingSample mailingSample;

    public Tier3Sample(MailingService svc) throws Exception {
        dsSample = new DataSourceSample(svc);
        targetSample = new TargetSample(svc);
        templateSample = new TemplateSample(svc);

        SystemAddressSample addressSample;
        addressSample = new SystemAddressSample(svc);

        // You need to create these data from UI
        SystemAddress bounce = addressSample.listByEmailAddress("BOUNCE",
                "bounce@sample.com");
        SystemAddress from = addressSample.listByEmailAddress("FROM",
                "from@sample.com");
        SystemAddress reply = addressSample.listByEmailAddress("REPLY",
                "reply@sample.com");

        mailingSample = new MailingSample(svc);
    }

    public List<ObjectId> createAllData(String sequence) throws Exception {
        List<ObjectId> objectIds = new ArrayList<ObjectId>();

        objectIds.add(dsSample.createIDS("ids_cxf_" + sequence));
        objectIds.add(targetSample.createTarget(
                "target_cxf_" + sequence,
                (InternalDataSourceId) getObjectId(objectIds, InternalDataSourceId.class))
        );

        objectIds.add(templateSample.createTemplate("template_cxf_" + sequence));


        objectIds.add(mailingSample.createTransactionalMailing("txmailing_cxf_" + sequence));

        return objectIds;
    }

    public void getAndPrintAllData(List<ObjectId> objectIds) throws Exception {
        for (ObjectId objectId : objectIds) {
            if (objectId instanceof DataSourceId) {
                dsSample.getAndPrint((DataSourceId) objectId);
            } else if (objectId instanceof TargetId) {
                targetSample.getAndPrint((TargetId) objectId);
            } else if (objectId instanceof TemplateId) {
                templateSample.getAndPrint((TemplateId) objectId);
            } else if (objectId instanceof MailingId) {
                mailingSample.getAndPrint((MailingId) objectId);
            }
        }
    }

    private ObjectId getObjectId(List<ObjectId> objectIds, Class clazz) {
        for (ObjectId objectId : objectIds) {
            if (clazz.isInstance(objectId)) {
                return objectId;
            }
        }

        return null;
    }

    public static MailingService createService() {

        // Read key store, key store password, wsdl file location and endpoint address
        // Endpoint address will replace the one in WSDL file
        System.out.println("Store: " + System.getProperty("javax.net.ssl.trustStore"));
        System.out.println("Password: " + System.getProperty("javax.net.ssl.trustStorePassword"));
        System.out.println("WSDL: " + System.getProperty("wsdl"));
        System.out.println("endpoint: " + System.getProperty("endpoint.address"));

        File wsdlFile = new File(System.getProperty("wsdl"));
        String endpoint = System.getProperty("endpoint.address");

        try {
            // Create service
            ClientSetup clientSetup = new ClientSetup(System.getProperty("services.namespace"),
                    System.getProperty("services.namespace.schema"));
            MailingService service = clientSetup.createService(wsdlFile, endpoint);

            // Apply interceptor to add SOAP header
            // username/password/organization are the same as UI login.
            clientSetup.applyInterceptor(service, "admin", "admin", "admin", null, false);

            return service;

        } catch (Exception e) {
            e.printStackTrace();
            System.exit(1);
        }

        return null;
    }


    public static void main(String[] args) throws Exception {
        MailingService svc = Tier3Sample.createService();
        List<ObjectId> objectIds = null;
        Tier3Sample samples = new Tier3Sample(svc);
        TransactionalMailingId txMailingId = null;
        EASService easSvc = new EASService();

        try {
            // Create all data
            objectIds = samples.createAllData(args[0]);

            // Print out all data
            samples.getAndPrintAllData(objectIds);

            // Add data to IDS using file
            InternalDataSourceId dataSourceId =
                    (InternalDataSourceId) samples.getObjectId(objectIds, InternalDataSourceId.class);
            samples.dsSample.addRecordsUsingFile(dataSourceId, new File(IDS_DATA_FILE));

            // List mailing in an organization
            samples.mailingSample.listByOrganization(true);

            // List mailing by type and status
            samples.mailingSample.listByTypeAndStatus(true);
            txMailingId = (TransactionalMailingId) samples.getObjectId(objectIds, TransactionalMailingId.class);
            // Load a transactional mailing
            boolean loaded = samples.mailingSample.loadMailing(txMailingId);
            System.out.println("Load mailing " + txMailingId.getId() + ": " + loaded);

            // Wait for the transactional mailing to become ACTIVE.
            samples.mailingSample.waitForStatus(txMailingId, "ACTIVE");

            // txnSend and related APIs.
            samples.mailingSample.printTxnEASInfo(txMailingId);
            String txMailingHandle = samples.mailingSample.getTxnMailingHandle(txMailingId);
            System.out.println("txMailingHandle: " + txMailingHandle);

            Thread.sleep(4000);
            easSvc.roundRobinEASSend(svc, txMailingId);

            // Close mailing
            boolean closed = samples.mailingSample.close(txMailingId);
            System.out.println("Close mailing " + txMailingId.getId() + ": " + closed);
        } finally {
            if ("true".equals(args[1])) {
                // true=delete the data
                // sleep to 5 seconds to let mailing finish.
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e) {
                    // ignore
                }

                // Delete all objects
                SampleHelper helper = new SampleHelper();
                if (objectIds != null) {
                    try {
                        // Must wait for mailings to complete before they can be deleted
                        // samples.mailingSample.waitForStatus(stdMailingId, MailingStatus.COMPLETED);
                        samples.mailingSample.waitForStatus(txMailingId, "COMPLETED");
                        helper.delete(objectIds);
                    } catch (Exception e) {
                        // ignore
                    }
                }

            }
        }

    }

}
